/**
 * @file      com.h
 * @brief     Basic communcation structure
 * @copyright (c) 2018 Kneron Inc. All right reserved.
 */
#ifndef __COM_H__
#define __COM_H__

#include "base.h"

typedef enum {
    CMD_NONE = 0,
    CMD_MEM_READ,
    CMD_MEM_WRITE,
    CMD_DATA,
    CMD_ACK_NACK,
    CMD_STS_CLR,
    CMD_MEM_CLR,
    CMD_CRC_ERR,
    CMD_TEST_ECHO,
    CMD_FILE_WRITE,
    CMD_FLASH_MEM_WRITE,  // single sector flash write

    CMD_RESET = 0x20,
    CMD_SYSTEM_STATUS,
    CMD_UPDATE_FW,
    CMD_DOWNLOAD_MODEL,
    
    CMD_QUERY_APPS = 0x30,
    CMD_SELECT_APP,
    CMD_SET_MODE,
    CMD_SET_EVENTS,
    CMD_UPDATE,
    CMD_IMG_RESULT,  // only RESP message is used.  No CMD version is implemented
    CMD_ABORT,

    CMD_SFID_START = 0x108,
    CMD_SFID_NEW_USER,
    CMD_SFID_ADD_DB,
    CMD_SFID_DELETE_DB,
    CMD_SEND_IMAGE,
    CMD_SFID_LW3D_START,
    CMD_SEND_LW3D,
    CMD_SFID_EDIT_DB,
	
    CMD_DME_START = 0x118,
    CMD_DME_CONFIG,
    CMD_DME_SEND_IMAGE,

    // Flash command
    CMD_FLASH_INFO = 0x1000,
    CMD_FLASH_CHIP_ERASE,
    CMD_FLASH_SECTOR_ERASE,
    CMD_FLASH_READ,
    CMD_FLASH_WRITE,    

    // debug utility command
    CMD_DBG_USB_MEM_WR = 0x2000,
    CMD_DBG_SET_MODEL_TYPE = 0x2001,
} Cmd;

typedef struct {
    u16 preamble;
    u16 pSize;
    u16 cmd;
    u16 mSize;
} __attribute__((packed)) MsgHdr;

typedef struct {
    u32 addr;
    u32 len;
    u8	data[];
} __attribute__((packed)) CmdPram;

typedef struct {
    u32 error;
    u32 bytes;
    u8  data[];
} __attribute__((packed)) RspPram;

typedef struct {
    u32 op_parm1;
    u32 op_parm2;
    u8  data[];
} __attribute__((packed)) OpPram;
    

#define MSG_HDR_CMD     0xA583
#define	MSG_HDR_RSP     0x8A35
#define MSG_HDR_VAL     0xA553  // this is used by the pre-packet code
#define MSG_HDR_SIZE    16  // includes both MsgHdr and CmdPram addr & len
#define PKT_CRC_FLAG    0x4000

#if (HAPS_ID == 2)
#define MSG_DATA_BUF_MAX    0x890
#else
#define MSG_DATA_BUF_MAX    0x1400  // used for testing (worst case msg payload is 4096+72)
#endif

#define MSG_BUF_MAX         (MSG_DATA_BUF_MAX + MSG_HDR_SIZE)

#endif
