/*
 * Kneron Delay driver
 *
 * Copyright (C) 2019 Kneron, Inc. All rights reserved.
 *
 */

#include <stdint.h>
#include "delay.h"

extern uint32_t SystemCoreClock;

#define CYCLES_PER_LOOP     2   // Typical: SUBS (1) + BCS (1 + P)

void delay_us(unsigned int usec)
{
    uint32_t cycles_per_us, count;

    cycles_per_us = SystemCoreClock / 1000000;
    count = usec * cycles_per_us / CYCLES_PER_LOOP;
    while(count--);
}

#ifdef TARGET_SCPU

#include "DrvPWMTMR010.h"

void delay_ms_enable(void)
{
    fLib_Timer_Init(DRVPWMTMR1, PWMTMR_1MSEC_PERIOD);
}

void delay_ms_disable(void)
{
    fLib_Timer_Close(DRVPWMTMR1);
}

void delay_ms(unsigned int msec)
{
    uint32_t tick_end;

    tick_end = fLib_CurrentT1Tick() + msec;
    do {
        __WFE();
    } while (tick_end > fLib_CurrentT1Tick());
}

#endif
