/**
 * @file      user_test_dme.cpp
 * @brief     kdp host lib user test examples 
 * @version   0.1 - 2019-08-01
 * @copyright (c) 2019 Kneron Inc. All right reserved.
 */

#include "errno.h"
#include "kdp_host.h"
#include "stdio.h"

#include <string.h>
#include <unistd.h>
#include "user_util.h"

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif

#define DME_IMAGE_FILE      (TEST_DME_DIR "image.bin")
#define DME_MODEL_FILE      (TEST_DME_DIR "all_models.bin")
#define DME_FW_SETUP        (TEST_DME_DIR "fw_info.bin")
#define DME_CONF_DETECT     (TEST_DME_DIR "config_detection.bin")
#define DME_CONF_RAW        (TEST_DME_DIR "config_raw.bin")

#define DME_IMG_SIZE (224 * 224 * 4)
#define DME_MODEL_SIZE (10 * 1024 * 1024)
#define DME_FWINFO_SIZE 128
#define DME_CONF_SIZE 32


int user_test_dme(int dev_idx, int user_id)
{

    if (1) {
        uint32_t model_size = 0;
        char data[DME_FWINFO_SIZE];
        int dat_size = 0;
        char* p_buf = NULL;
        uint32_t buf_len = 0;
        uint32_t ret_size = 0;

        //read firmware setup data
        memset(data, 0, sizeof(data));
        int n_len = read_file_to_buf(data, DME_FW_SETUP, DME_FWINFO_SIZE);
        if (n_len <= 0) {
            printf("reading fw setup file failed:%d...\n", n_len);
            return -1;
        }
        dat_size = n_len;

        p_buf = new char[DME_MODEL_SIZE];
        memset(p_buf, 0, DME_MODEL_SIZE);
        n_len = read_file_to_buf(p_buf, DME_MODEL_FILE, DME_MODEL_SIZE);
        if (n_len <= 0) {
            printf("reading model file failed:%d...\n", n_len);
            delete[] p_buf;
            return -1;
        }
        buf_len = n_len;
        model_size = n_len;

        printf("starting DME mode ...\n");
        int ret = kdp_start_dme(
            dev_idx, model_size, data, dat_size, &ret_size, p_buf, buf_len);
        if (ret != 0) {
            printf("could not set to DME mode:%d..\n", ret_size);
            delete[] p_buf;
            return -1;
        }

        delete[] p_buf;
        printf("DME mode succeeded...\n");
        usleep(SLEEP_TIME);
    }

    if (1) {
        char data[DME_CONF_SIZE];
        int dat_size = 0;

        //read config data
        memset(data, 0, sizeof(data));

        int n_len;
        if (user_id == 1) {
            n_len = read_file_to_buf(data, DME_CONF_DETECT, DME_CONF_SIZE);
        } else {
            n_len = read_file_to_buf(data, DME_CONF_RAW, DME_CONF_SIZE);
        }

        if (n_len <= 0) {
            printf("reading config file failed:%d...\n", n_len);
            return -1;
        }

        dat_size = n_len;
        printf("starting DME configure ...\n");
        int ret;
        if(dat_size == 20) {
	        ret = kdp_dme_configure_param(dev_idx, 1, data, dat_size); //20byte
        }
	    else
	    {
	        ret = kdp_dme_configure_param(dev_idx, *((uint32_t* )data), data + 4, dat_size-4);
	    }
        if (ret != 0) {
            printf("could not set to DME configure mode..\n");
            return -1;
        }
        printf("DME configure succeeded...\n");
        usleep(SLEEP_TIME);
    }

    if (1) {
        uint32_t inf_size = 0;
        char inf_res[256000];
        bool res_flag = true;

        char img_buf[DME_IMG_SIZE];
        uint32_t buf_len = 0;

        // TODO need set image name here
        memset(img_buf, 0, sizeof(img_buf));
        int n_len = read_file_to_buf(img_buf, DME_IMAGE_FILE, DME_IMG_SIZE);
        if (n_len <= 0) {
            printf("reading dme image file failed:%d...\n", n_len);
            return -1;
        }

        buf_len = n_len;

        printf("starting DME inference ...\n");
        int ret = kdp_dme_inference(dev_idx, img_buf, buf_len, &inf_size, &res_flag, inf_res);
        if (ret != 0) {
            printf("could not set to DME inference mode..\n");
            return -1;
        }
        printf("DME inference succeeded...\n");
        usleep(SLEEP_TIME);

        res_flag = false;
        ret = kdp_dme_inference(dev_idx, img_buf, buf_len, &inf_size, &res_flag, inf_res);
        if (ret != 0) {
            printf("could not set to DME inference mode..\n");
            return -1;
        }
        printf("DME inference succeeded...\n");
        usleep(SLEEP_TIME);
    }
    return 0;
}



int user_test(int dev_idx, int user_id)
{
    //dme test
    user_test_dme(dev_idx, user_id);

    return 0;
}

#if defined(__cplusplus) || defined(c_plusplus)
}
#endif
