/**
 * @file      Kn_FM_comparison.cpp
 * @brief     implementation file for feature map comparison
 * @version   0.1 - 2019-11-16
 * @copyright (c) 2019 Kneron Inc. All right reserved.
 */


#include "errno.h"
#include "kdp_host.h"
#include "stdio.h"

#include <string.h>
#include <unistd.h>
#include "user_util.h"

#include <math.h>

#include "KdpFMComparison.h"

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif

int user_test_extract_fm(int dev_idx, const char *img_name, char *user_fm_buf)
{
	
	int ret = 0;
	
    int buf_len = 0;
	char img_buf[IMAGE_SIZE];

	bool fd_flag = false;
	char fd_res[8];
	
    bool fr_flag = true; 
	
    bool lm_flag = false;
    char lm_res[200];
	
    bool live_flag = false;
    char live_res[200];
	
    if (1) {
        //TODO need set image name here
        memset(img_buf, 0, sizeof(img_buf));

        printf("registering with image...\n");
        int n_len = read_file_to_buf(img_buf, img_name, IMAGE_SIZE);
        if (n_len <= 0) {
            printf("reading file to buf failed:%d...\n", n_len);
            n_len = IMAGE_SIZE;
        }

        buf_len = n_len;

        printf("starting extract feature all res ...\n");

        ret = kdp_extract_feature_all_res(dev_idx, img_buf, buf_len,
            &fd_flag, fd_res,
            &fr_flag, user_fm_buf, &lm_flag, lm_res,
            &live_flag, live_res);
        if (ret != 0) {
            printf("could not extract feature all res..\n");
            return -1;
        }
        usleep(SLEEP_TIME);
	}
	return 0;
}



int user_test(int dev_idx, int user_id) 
{
   	 char user_fm_a[2048]={0};
     char user_fm_b[2048]={0}; 
    float sim_score = 100.0f;

	uint32_t img_size = IMAGE_SIZE;
	
    printf("starting verify mode ...\n");
    int ret = kdp_start_verify_mode_thresh(dev_idx, &img_size, THRESHOLD_DEFAULT);
    if (ret != 0 || img_size == 0) {
        printf("start verify mode failed :%d, buf size:%d.\n", ret, img_size);
        return -1;
    }
    printf("starting verify mode successfuly, img size:%d...\n", img_size);

	uint16_t img_idx = 2;
    uint16_t u_id = user_id;
    if (u_id == 0)
        u_id = 2;
	
	
	
    printf("starting register mode 2 ...\n");
    ret = kdp_start_reg_user_mode(dev_idx, u_id, img_idx);
    if (ret != 0) {
        printf("could not set to register user mode..\n");
        return -1;
    }
    printf("register mode 2 succeeded...\n");

    usleep(SLEEP_TIME);
	
	
    ret = user_test_extract_fm(dev_idx, IMAGE_DATA_FILE, user_fm_a);
	if(ret != 0)
	{
		printf(" extract feature map a failed :%d\n", ret);
		return -1;
	}
    
	user_test_extract_fm(dev_idx, IMAGE_DATA_FILE_U18, user_fm_b);
	if(ret != 0)
	{
		printf(" extract feature map a failed :%d\n", ret);
		return -1;
	}
	
		
    

    printf("test kdp feature map comparison....\n");
    
    if(sizeof(user_fm_a)!= sizeof(user_fm_b))
    {
        printf("fm buffer size error \n");
        return -1;
    }
    
    sim_score = kdp_FM_comparison(user_fm_a,user_fm_b,sizeof(user_fm_a));
    printf("feature map similarity score :%f\n",sim_score);

    return 0;
}
	
#if defined(__cplusplus) || defined(c_plusplus)
}
#endif



