/**
 * @file      user_test_lw3d.cpp
 * @brief     kdp host lib user test examples 
 * @version   0.1 - 2019-08-01
 * @copyright (c) 2019 Kneron Inc. All right reserved.
 */

#include "errno.h"
#include "kdp_host.h"
#include "stdio.h"

#include <string.h>
#include <unistd.h>
#include "user_util.h"

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif

#define RGB_IMAGE_FILE (TEST_IMG_DIR "u18_f1_rgb.bin")
#define NIR_IMAGE_FILE (TEST_IMG_DIR "u18_f1_nir.bin")

#define NIR_IMAGE_SIZE (640 * 480)

//user test for removing user
int user_test_rm_lw3d_user(int dev_idx, int user_id)
{
    //remove user id
    uint32_t _id = user_id;
    uint32_t rgb_size = 0;
    uint32_t nir_size = 0;
    float rgb_thresh  = RGB_THRESHOLD_DEFAULT;
    float nir_thresh  = NIR_THRESHOLD_DEFAULT;
    printf("starting lw3d mode ...\n");
    int ret = kdp_start_lw3d_mode_thresh(dev_idx, &rgb_size, &nir_size,
                                         rgb_thresh, nir_thresh);
    if (ret != 0) {
        printf("could not start lw3d mode ..\n");
        return -1;
    }
    printf("start lw3d mode succeeded...\n");
    usleep(SLEEP_TIME);
    
    printf("removing user ...\n");
    ret = kdp_remove_user(dev_idx, _id);
    if (ret != 0) {
        printf("remove user :%d failed.\n", user_id);
        return -1;
    }
    printf("removing user succeeded ...\n");

    return 0;
}

//user test for registering user
int user_test_reg_lw3d_user(int dev_idx, int user_id)
{
    uint32_t rgb_size = 0;
    uint32_t nir_size = 0;
	
    float rgb_thresh  = RGB_THRESHOLD_DEFAULT;
    float nir_thresh  = NIR_THRESHOLD_DEFAULT;
	
    printf("starting lw3d mode ...\n");
    int ret = kdp_start_lw3d_mode_thresh(dev_idx, &rgb_size, &nir_size,
        rgb_thresh, nir_thresh);
    if (ret != 0) {
        printf("could not start lw3d mode ..\n");
        return -1;
    }
    printf("start lw3d mode succeeded...\n");
    usleep(SLEEP_TIME);

    //register
    uint16_t u_id = user_id;
    if (u_id == 0)
        u_id = 2;

    char img_buf[IMAGE_SIZE];
    int buf_len = 0;

    int n_len = read_file_to_buf(img_buf, RGB_IMAGE_FILE, IMAGE_SIZE);
    if (n_len <= 0) {
        printf("reading file to buf failed:%d...\n", n_len);
        return -1;
    }
    buf_len = n_len;

    char img_buf_nir[NIR_IMAGE_SIZE];
    int buf_len_nir = read_file_to_buf(img_buf_nir, NIR_IMAGE_FILE, NIR_IMAGE_SIZE);
    if (buf_len_nir <= 0) {
        printf("reading file to buf failed:%d...\n", buf_len_nir);
        return -1;
    }

    uint16_t img_idx = 1;
    printf("starting register mode 1...\n");
    ret = kdp_start_reg_user_mode(dev_idx, u_id, img_idx);
    if (ret != 0) {
        printf("could not set to register user mode..\n");
        return -1;
    }
    printf("register mode 1 succeeded...\n");

    usleep(SLEEP_TIME);

    bool fd_flag = false;
    char fd_res[200];
    bool fr_flag = false;
    char fr_res[2048];
    bool lm_flag = false;
    char lm_res[200];
    bool fd_flag_nir = false;
    char fd_res_nir[200];
    bool fr_flag_nir = false;
    char fr_res_nir[2048];
    bool lm_flag_nir = false;
    char lm_res_nir[200];
    bool live_flag = false;
    char live_res[200];
    ret = kdp_extract_lw3D_feature_all_res(dev_idx, img_buf, buf_len,
        img_buf_nir, buf_len_nir, &fd_flag, fd_res,
        &fr_flag, fr_res, &lm_flag, lm_res,
        &fd_flag_nir, fd_res_nir, &fr_flag_nir, fr_res_nir,
        &lm_flag_nir, lm_res_nir, &live_flag, live_res);
    if (ret != 0) {
        printf("could not start verify lw3D image..\n");
        return -1;
    }

    usleep(SLEEP_TIME);
    img_idx = 2;
    printf("starting register mode 2 ...\n");
    ret = kdp_start_reg_user_mode(dev_idx, u_id, img_idx);
    if (ret != 0) {
        printf("could not set to register user mode..\n");
        return -1;
    }
    printf("register mode 2 succeeded...\n");

    usleep(SLEEP_TIME);

    uint16_t mask = 0;
    char res[6000];
    printf("starting verify lw3D image generic ...\n");
    ret = kdp_extract_lw3D_feature_generic(dev_idx, img_buf, buf_len,
        img_buf_nir, buf_len_nir, &mask, res);
    if (ret != 0) {
        printf("could not verify lw3D image generic..\n");
        return -1;
    }

    usleep(SLEEP_TIME);

    uint32_t _id = u_id;
    printf("registering user :%d ...\n", _id);
    ret = kdp_register_user(dev_idx, _id);
    if (ret != 0) {
        printf("register user failed.\n");
        return -1;
    }
    printf("registered with user id:%d.\n", _id);
    usleep(SLEEP_TIME);
    ret = kdp_extract_lw3D_feature(dev_idx, img_buf, buf_len,
        img_buf_nir, buf_len_nir);
    if (ret != 0) {
        printf("could not start verify lw3D image..\n");
        return -1;
    }

    return 0;
}

//user test for verifying user
int user_test_ver_lw3d_user(int dev_idx)
{
    uint32_t rgb_size = 0;
    uint32_t nir_size = 0;
    float rgb_thresh  = RGB_THRESHOLD_DEFAULT;
    float nir_thresh  = NIR_THRESHOLD_DEFAULT;
    printf("starting lw3d mode ...\n");
    int ret = kdp_start_lw3d_mode(dev_idx, &rgb_size, &nir_size,
        rgb_thresh, nir_thresh, 640, 480, 640, 480, IMG_FORMAT_RGB565, IMG_FORMAT_RAW8);
    if (ret != 0) {
        printf("could not start lw3d mode ..\n");
        return -1;
    }
    printf("start lw3d mode succeeded...\n");
    usleep(SLEEP_TIME);

    printf("starting verify lw3D image ...\n");
    char img_buf[IMAGE_SIZE];
    int n_len = read_file_to_buf(img_buf, RGB_IMAGE_FILE, IMAGE_SIZE);
    if (n_len <= 0) {
        printf("reading file to buf failed:%d...\n", n_len);
        return -1;
    }
    int buf_len = n_len;

    char img_buf_nir[NIR_IMAGE_SIZE];
    int buf_len_nir = read_file_to_buf(img_buf_nir, NIR_IMAGE_FILE, NIR_IMAGE_SIZE);
    if (buf_len_nir <= 0) {
        printf("reading file to buf failed:%d...\n", buf_len_nir);
        return -1;
    }

    uint16_t user_id = 0;
    bool fd_flag = false;
    char fd_res[200];
    bool fr_flag = false;
    char fr_res[2048];
    bool lm_flag = false;
    char lm_res[200];
    bool fd_flag_nir = false;
    char fd_res_nir[200];
    bool fr_flag_nir = false;
    char fr_res_nir[2048];
    bool lm_flag_nir = false;
    char lm_res_nir[200];
    bool live_flag = false;
    char live_res[200];
    ret = kdp_verify_lw3D_image_all_res(dev_idx, &user_id, img_buf, buf_len,
        img_buf_nir, buf_len_nir, &fd_flag, fd_res,
        &fr_flag, fr_res, &lm_flag, lm_res,
        &fd_flag_nir, fd_res_nir, &fr_flag_nir, fr_res_nir,
        &lm_flag_nir, lm_res_nir, &live_flag, live_res);
    if (ret != 0) {
        printf("could not start verify lw3D image..\n");
        return -1;
    }
    printf("start verify lw3D image succeeded...\n");
    usleep(SLEEP_TIME);

    fd_flag = true;
    fr_flag = true;
    lm_flag = true;
    fd_flag_nir = true;
    fr_flag_nir = true;
    lm_flag_nir = true;
    live_flag = true;
    ret = kdp_verify_lw3D_image_all_res(dev_idx, &user_id, img_buf, buf_len,
        img_buf_nir, buf_len_nir, &fd_flag, fd_res,
        &fr_flag, fr_res, &lm_flag, lm_res,
        &fd_flag_nir, fd_res_nir, &fr_flag_nir, fr_res_nir,
        &lm_flag_nir, lm_res_nir, &live_flag, live_res);
    if (ret != 0) {
        printf("could not start verify lw3D image..\n");
        return -1;
    }
    printf("start verify lw3D image succeeded...\n");
    usleep(SLEEP_TIME);

    fr_flag = false;
    fr_flag_nir = false;
    live_flag = false;

    printf("starting verify lw3D image ...\n");
    ret = kdp_verify_lw3D_image_all_res(dev_idx, &user_id, img_buf, buf_len,
        img_buf_nir, buf_len_nir, &fd_flag, fd_res,
        &fr_flag, fr_res, &lm_flag, lm_res,
        &fd_flag_nir, fd_res_nir, &fr_flag_nir, fr_res_nir,
        &lm_flag_nir, lm_res_nir, &live_flag, live_res);
    if (ret != 0) {
        printf("could not start verify_lw3D_image..\n");
        return -1;
    }
    printf("start verify lw3D image succeeded...\n");
    usleep(SLEEP_TIME);

    uint16_t mask = 0;
    char res[6000];

    printf("starting verify lw3D image generic ...\n");
    ret = kdp_verify_lw3D_image_generic(dev_idx, &user_id, img_buf, buf_len,
        img_buf_nir, buf_len_nir, &mask, res);
    if (ret != 0) {
        printf("could not verify lw3D image generic..\n");
        return -1;
    }
    printf("verify lw3D image generic succeeded...\n");

    usleep(SLEEP_TIME);

    mask = 0x17;

    printf("starting verify lw3D image generic ...\n");
    ret = kdp_verify_lw3D_image_generic(dev_idx, &user_id, img_buf, buf_len,
        img_buf_nir, buf_len_nir, &mask, res);
    if (ret != 0) {
        printf("could not verify lw3D image generic..\n");
        return -1;
    }
    printf("verify lw3D image generic succeeded...\n");

    usleep(SLEEP_TIME);
    mask = 0x77;

    printf("starting verify lw3D image generic ...\n");
    ret = kdp_verify_lw3D_image_generic(dev_idx, &user_id, img_buf, buf_len,
        img_buf_nir, buf_len_nir, &mask, res);
    if (ret != 0) {
        printf("could not verify lw3D image generic..\n");
        return -1;
    }
    printf("verify lw3D image generic succeeded...\n");

    usleep(SLEEP_TIME);
    // mask = 0;

    // printf("starting verify lw3D image &mask, NULL ...\n");
    // ret = kdp_verify_lw3D_image_generic(dev_idx, &user_id, img_buf, buf_len,
    //     img_buf_nir, buf_len_nir, &mask, NULL);
    // if (ret != 0) {
    //     printf("could not verify lw3D image generic..\n");
    //     return -1;
    // }
    // printf("verify lw3D image generic succeeded...\n");

    // usleep(SLEEP_TIME);

    printf("starting verify lw3D image generic ...\n");

    kdp_verify_lw3D_user(dev_idx, &user_id, img_buf, buf_len,
        img_buf_nir, buf_len_nir);
    if (ret != 0) {
        printf("could not kdp_verify_lw3D_user..\n");
        return -1;
    }

    usleep(SLEEP_TIME);

    return 0;
}

int user_test(int dev_idx, int user_id)
{
    //doing remove user test
    user_test_rm_lw3d_user(dev_idx, user_id);

    //register user id
    user_test_reg_lw3d_user(dev_idx, user_id);

    //verify user id
    user_test_ver_lw3d_user(dev_idx);

    return 0;
}

#if defined(__cplusplus) || defined(c_plusplus)
}
#endif
