/**
 * @file      user_test_reg.cpp
 * @brief     kdp host lib user test examples 
 * @version   0.1 - 2019-04-25
 * @copyright (c) 2019 Kneron Inc. All right reserved.
 */

#include "stdio.h"
#include "errno.h"
#include "kdp_host.h"

#include <string.h>
#include <unistd.h>
#include "user_util.h"

#if defined (__cplusplus) || defined (c_plusplus)
extern "C"{
#endif

int user_test(int dev_idx, int user_id) 
{
    //verify user id
    char img_buf[IMAGE_SIZE];
    uint16_t u_id = 0;

    bool fd_flag = false;
    bool fr_flag = false;
    char fd_res[8];
    char fr_res[2048];

    uint32_t img_size = 0;
    printf("starting verify mode ...\n");
    int ret = kdp_start_verify_mode_thresh(dev_idx, &img_size, THRESHOLD_DEFAULT);
    if(ret != 0 || img_size == 0) {
        printf("start verify mode failed :%d, buf size:%d.\n", ret, img_size);
        return -1;
    }
    printf("starting verify mode successfuly, img size:%d...\n", img_size);

    //register
    u_id = user_id;

    uint16_t img_idx = 1;
    printf("starting register mode ...\n");
    ret = kdp_start_reg_user_mode(dev_idx, u_id, img_idx);
    if(ret != 0) {
        printf("could not set to register user mode..\n");
        return -1;
    }
    printf("register mode succeeded...\n");

    usleep(SLEEP_TIME);

    if(1) {
        //TODO need set image name here 
        memset(img_buf, 0, sizeof(img_buf));

        printf("registering with image...\n");
        int n_len = read_file_to_buf(img_buf, IMAGE_DATA_FILE, IMAGE_SIZE);
        if(n_len <= 0) {
            printf("reading file to buf failed:%d...\n", n_len);
            n_len = IMAGE_SIZE;
        }

        ret = kdp_extract_feature(dev_idx, img_buf, n_len);
        if(ret != 0) {
            printf("could not extract feature for.\n");
        }  else {
            printf("feature extracted successfully for img index.\n");
        }
    }
    
    usleep(SLEEP_TIME);

    img_idx = 2;
    printf("starting register mode 2 ...\n");
    ret = kdp_start_reg_user_mode(dev_idx, u_id, img_idx);
    if(ret != 0) {
        printf("could not set to register user mode..\n");
        return -1;
    }
    printf("register mode 2 succeeded...\n");

    usleep(SLEEP_TIME);

    if(1) {
        //TODO need set image name here 
        memset(img_buf, 0, sizeof(img_buf));

        printf("registering with image...\n");
        int n_len = read_file_to_buf(img_buf, IMAGE_DATA_FILE, IMAGE_SIZE);
        if(n_len <= 0) {
            printf("reading file to buf failed:%d...\n", n_len);
            n_len = IMAGE_SIZE;
        }

        fd_flag = true;
        ret = kdp_extract_feature_fdfr_res(dev_idx, img_buf, n_len, &fd_flag, fd_res, NULL, NULL);
        if(ret != 0) {
            printf("could not extract feature for.\n");
        }  else {
            printf("feature extracted successfully for img index.\n");
        }
    }

    usleep(SLEEP_TIME);

    img_idx = 3;
    printf("starting register mode 3 ...\n");
    ret = kdp_start_reg_user_mode(dev_idx, u_id, img_idx);
    if(ret != 0) {
        printf("could not set to register user mode..\n");
        return -1;
    }
    printf("register mode 3 succeeded...\n");

    usleep(SLEEP_TIME);

    if(1) {
        //TODO need set image name here 
        memset(img_buf, 0, sizeof(img_buf));

        printf("registering with image...\n");
        int n_len = read_file_to_buf(img_buf, IMAGE_DATA_FILE, IMAGE_SIZE);
        if(n_len <= 0) {
            printf("reading file to buf failed:%d...\n", n_len);
            n_len = IMAGE_SIZE;
        }

        fd_flag = true;
        fr_flag = true;
        ret = kdp_extract_feature_fdfr_res(dev_idx, img_buf, n_len, &fd_flag, fd_res, &fr_flag, fr_res);
        if(ret != 0) {
            printf("could not extract feature for.\n");
        }  else {
            printf("feature extracted successfully for img index.\n");
        }
    }

    usleep(SLEEP_TIME);

    uint32_t _id = user_id;
    printf("registering user :%d ...\n", _id);
    ret = kdp_register_user(dev_idx, _id);
    if(ret != 0) {
        printf("register user failed.\n");
        return -1;
    }
    printf("registered with user id:%d.\n", _id);
    return 0;
}

#if defined (__cplusplus) || defined (c_plusplus)
}
#endif
