/**
 * @file      user_test_ver.cpp
 * @brief     kdp host lib user test examples 
 * @version   0.1 - 2019-04-25
 * @copyright (c) 2019 Kneron Inc. All right reserved.
 */

#include "stdio.h"
#include "errno.h"
#include "kdp_host.h"

#include <string.h>
#include <unistd.h>
#include "user_util.h"

#if defined (__cplusplus) || defined (c_plusplus)
extern "C"{
#endif

int user_test(int dev_idx, int user_id) 
{
    //verify user id
    uint32_t img_size = 0;
    printf("starting verify mode ...\n");
    //int ret = kdp_start_verify_mode(dev_idx, &img_size);
    int ret = kdp_start_verify_mode_thresh(dev_idx, &img_size, THRESHOLD_DEFAULT);
    if(ret != 0 || img_size == 0) {
        printf("start verify mode failed :%d, buf size:%d.\n", ret, img_size);
        return -1;
    }

    ret = kdp_start_sfid_mode(dev_idx, &img_size, THRESHOLD_DEFAULT, 640, 480, IMG_FORMAT_RGB565);
    if(ret != 0 || img_size == 0) {
        printf("start sfid mode failed :%d, buf size:%d.\n", ret, img_size);
        return -1;
    }
    printf("starting verify mode successfuly, img size:%d...\n", img_size);

    usleep(SLEEP_TIME);

    char img_buf[IMAGE_SIZE];
    uint16_t u_id = 0;

    bool fd_flag = false;
    bool fr_flag = false;
    char fd_res[8];
    char fr_res[2048];

    for(int i = 0; i < 1; i++) { //do 100 times
        memset(img_buf, 0, sizeof(img_buf));

        printf("verifying for image ...\n");
        int n_len = read_file_to_buf(img_buf, IMAGE_DATA_FILE, IMAGE_SIZE);
        if(n_len <= 0) {
            printf("reading file to buf failed:%d...\n", n_len);
            n_len = IMAGE_SIZE;
        }

        u_id = 0;
        ret = kdp_verify_user_id(dev_idx, &u_id, img_buf, n_len);
        if(ret != 0) {
            printf("could not find matched user id.\n");
        } else {
            user_id = u_id;
            printf("find matched user id:%d.\n", user_id);
        }

        usleep(SLEEP_TIME);
        fd_flag = true;
        ret = kdp_verify_user_id_fdfr_res(dev_idx, &u_id, img_buf, n_len, &fd_flag, fd_res, NULL, NULL);
        if(ret != 0) {
            printf("could not find matched user id.\n");
        } else {
            user_id = u_id;
            printf("find matched user id:%d.\n", user_id);
        }

        usleep(SLEEP_TIME);
        fd_flag = true;
        fr_flag = true;
        ret = kdp_verify_user_id_fdfr_res(dev_idx, &u_id, img_buf, n_len, &fd_flag, fd_res, &fr_flag, fr_res);
        if(ret != 0) {
            printf("could not find matched user id.\n");
        } else {
            user_id = u_id;
            printf("find matched user id:%d.\n", user_id);
        }
    }
             
    return 0;
}

#if defined (__cplusplus) || defined (c_plusplus)
}
#endif
