/*
 * Kneron Example CPU Operation driver
 *
 * Copyright (C) 2019 Kneron, Inc. All rights reserved.
 *
 */
 
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "base.h"
#include "kdpio.h"
#include "ipc.h"
#include "cpu_ops_ex.h"
#include "dbg.h"

inline static int pad_up_16(int a)
{
    return ceil((float)a / 16) * 16;
}

int nearest_upsample_cpu(int cpu_op, struct kdp_image_s *image_p) {
    int row_num_in = CPU_OP_NODE_INPUT_ROW(image_p);
    int col_num_in = CPU_OP_NODE_INPUT_COL(image_p);
    int chnl_num_in = CPU_OP_NODE_INPUT_CH(image_p);

    int row_num_out = CPU_OP_NODE_OUTPUT_ROW(image_p);
    int col_num_out = CPU_OP_NODE_OUTPUT_COL(image_p);

    int col_num_pad_in = pad_up_16(col_num_in);
    int col_num_pad_out = pad_up_16(col_num_out);

    int oi, oj, ii, ij, k;

    int h_scale = row_num_out / row_num_in;
    int w_scale = col_num_out / col_num_in;
    int out_index, in_index;

    uint8_t *data_array = (uint8_t*)CPU_OP_NODE_INPUT_ADDR(image_p);
    uint8_t *dst_data_array = (uint8_t*)CPU_OP_NODE_OUTPUT_ADDR(image_p);
    //HCW for DRAM
    for (oj = 0; oj < row_num_out; oj++) {
        for (k = 0; k < chnl_num_in; ++k) {
            for (oi = 0; oi < col_num_out; oi++) {
                ij = oj / h_scale;
                ii = oi / w_scale;
                out_index = oi + col_num_pad_out * (k + chnl_num_in * oj);
                in_index = ii + col_num_pad_in * (k + chnl_num_in * ij);
                dst_data_array[out_index] = data_array[in_index];
            }
        }
    }
    return 0;
}
