#include "kl520_include.h"
#include "base.h"
#include "framework/v2k_image.h"
#include "dbg.h"
#include "kdp_system.h"
#include "media/display/kdp_display.h"
#include "media/kdp_inference.h"
#include "media/display/kdp_2d.h"

static bool m_dp_inited = FALSE;

/*-----------------------------------------------------------------------
 *                          Display layout API 
 *-----------------------------------------------------------------------*/
/**
* @brief  Setup driver and turn on display
* Depend on kl520_api_display_setting()
*
* @param  cam_idx: camera source on dispaly
* 
* @return TRUE: succeed to setup/init display driver
*         FALSE: fail to setup/init display driver
*/
bool kl520_api_dp_open(uint8_t cam_idx, unsigned int width, unsigned int height, unsigned int pixelformat)
{
    int cam_state = kdp_get_camera_ctx_state(cam_idx);
    struct video_input_params params;

    if (CAMERA_STATE_RUNNING == cam_state) {
            params.input_fmt = pixelformat;
            params.input_xres = width;
            params.input_yres = height;
            DSG("[%s] cam_idx:%d, width:%d, height:%d", __func__, cam_idx, params.input_xres, params.input_yres);
            kdp_video_renderer_open(&params);
            kdp_video_renderer_set_camera(cam_idx);
            kdp_display_set_pen_rgb565(BLACK, 4);

            kdp_video_renderer_start();
            m_dp_inited = TRUE;
    }
    else {
        DSG("cam_idx[%d] state:%d", cam_idx, cam_state);
    }

    return m_dp_inited;
}

/**
* Stop/Close display preview
*
* @param  N/A
* 
* @return N/A
*/
void kl520_api_dp_close(void)
{
   // uint32_t frame_addr;
 
    kdp_display_set_pen_rgb565(BLACK, 2);
   // frame_addr = kdp_fb_mgr_next_inf(0, &g_buf_idx[0]);
   // kdp_display_fill_rect(frame_addr, 0, 0, LCD_WIDTH, LCD_HEIGHT);
    
    kdp_video_renderer_stop();
    m_dp_inited = FALSE;
}

/**
* Get display init. status
*
* @param  N/A
* 
* @return TRUE: initialized\n
*         FALSE: not initialized
*/
bool kl520_api_dp_init_get(void)
{
    return m_dp_inited;
}

