#ifndef __KL520_API_CAMERA_H__
#define __KL520_API_CAMERA_H__

#include <cmsis_os2.h>
#include "media/kdp_camera.h"
#include "kl520_include.h"


extern struct camera_ctx m_camera_ctx[IMGSRC_NUM + 2];

/*-----------------------------------------------------------------------
 *                             Camera API
 *-----------------------------------------------------------------------*/
/**
* Initialize parameter of camera driver
*
* @param  cam_idx : camera index ref to defgroup@Camera_setting
*         *cam_ctx: ref to @struct camera_ctx
* 
* @return -1: Invalid camera index
*         -2: Camera is already initialized
*          0: Setting success
*/
int kl520_api_camera_setting(uint8_t cam_idx, unsigned int width, unsigned int height, unsigned int pixelformat);


/**
 * @brief  Open the specific camera
 *
 * @param  cam_idx
 * 
 * @return 0: Succeed to open the specific camera\n
 *        <0: Fail to open the specific camera
 */
int kl520_api_camera_open(uint8_t cam_idx);

/**
* @brief  Start to capture image stream
*
* @param  cam_idx
* 
* @return 0: Succeed to start capturing image stream on the specific camera\n
*        <0: Fail to start capturing image stream on the specific camera
*/
int kl520_api_camera_start(uint8_t cam_idx);

/**
* @brief  Close the specific camera
*
* @param  cam_idx
* 
* @return 0: Succeed to close the specific camera\n
*        <0: Fail to close the specific camera
*/
int kl520_api_camera_close(uint8_t cam_idx);

/**
* @brief  Stop to capture image stream
*
* @param  cam_idx
* 
* @return 0: Succeed to stop capturing image stream on the specific camera\n
*        <0: Fail to stop capturing image stream on the specific camera
*/
int kl520_api_camera_stop(uint8_t cam_idx);

/**
* @brief  Check the status if the specific camera is running
*
* @param  cam_idx
* 
* @return TRUE: The specific camera is running
*         FALSE: The specific camera is NOT running
*/
bool kl520_api_camera_is_running(uint8_t cam_idx);

/*-----------------------------------------------------------------------
 *                          Display layout API 
 *-----------------------------------------------------------------------*/
/**
* @brief  Check the status if both cameras are running
*
* @param  N/A
* 
* @return TRUE: Both cameras are running
*         FALSE: Both cameras are NOT running
*/
bool kl520_api_all_cameras_are_running(void);

/**
* @brief  Switch camera source to display
*
* @param  N/A
* 
* @return N/A
*/
void kl520_api_camera_switch_rbg_nir(void);

#endif

