#ifndef __KL520_API_FDFR_H__
#define __KL520_API_FDFR_H__


#include "base.h"
#include "framework/result.h"
#include "framework/v2k_image.h"
#include "kdp520_pwm_timer.h"


/**
* @brief  Initialize FDFR models
*
* @param  N/A
* 
* @return TRUE:  Succeed to Initialize
*         FALSE: Fail to initialize
*/
bool kl520_api_fdfr_model_init(void);

/**
* @brief  Run FDFR recognition in loop until received stop FDFR inference loop command 
*         or call kl520_api_face_recognition_loop_stop()
*
* @param  N/A
* 
* @return 0: Get started to run FDFR recognition\n
*        -1: Both of RGB and NIR cameras aren't opened yet
*/
bool kl520_api_face_recognition_loop(void);

/**
* @brief  Run FDFR recognition in loop until received stop FDFR inference loop command 
*         or call kl520_api_face_recognition_loop_stop()
*
* @param  N/A
* 
* @return 0: Get started to run FDFR recognition\n
*        -1: Both of RGB and NIR cameras aren't opened yet
*/
bool kl520_api_face_recognition_loop(void);

/**
* @brief  Stop loop running FDFR recognition
*
* @param  N/A
* 
* @return 0: Stop running FDFR recognition
*        -1: Both of RGB and NIR cameras aren't opened yet
*/
bool kl520_api_face_recognition_loop_stop(void);

/**
* @brief  Run FDFR recognition in loop until FDFR result compare DB ok
*
* @param  N/A
* 
* @return 0: Get started to run FDFR recognition
*        -1: Both of RGB and NIR cameras aren't opened yet
*/
int kl520_api_face_recognition(void);

/**
* @brief  Register user face into DDR DB
*
* @param  user_idx: index of user to be registered into DDR DB
* 
* @return 0: Stop running FDFR recognition
*        -1: Both of RGB and NIR cameras aren't opened yet
*        -2: kl520 is running FDFR inference
*         1: No space of database
*        >1: enum kdp_app_fid_status_code_e
*/
int kl520_api_face_register(uint8_t user_idx);

/**
* @brief  Register user face into DDR DB
*
* @param  user_idx: index of user to be registered into DDR DB
* 
* @return 0: Stop running FDFR recognition
*        -1: Both of RGB and NIR cameras aren't opened yet
*        -2: kl520 is running FDFR inference
*         1: No space of database
*        >1: enum kdp_app_fid_status_code_e
*/
int kl520_api_face_auto_register(uint8_t user_idx, int reg_index);

/**
* @brief  Register user face into DDR DB
*
* @param  user_idx: index of user to be registered into DDR DB
* 
* @return 0: Stop running FDFR recognition
*        -1: Both of RGB and NIR cameras aren't opened yet
*        -2: kl520 is running FDFR inference
*         1: No space of database
*        >1: enum kdp_app_fid_status_code_e
*/
int kl520_api_face_register_cfm(uint8_t user_idx);

int kl520_api_face_auto_register_both(uint8_t user_idx, int reg_index);

int kl520_api_face_register_cfm_both(uint8_t user_idx);

int kl520_api_face_auto_register_5_face(uint8_t user_idx, int reg_index);


/**
* @brief  Clear the register user face record in DDR DB
*
* @param  N/A
* 
* @return 0: Succeed to cancel register\n
*        -1: Fail to cancel register
*/
int kl520_api_face_cancel_register(void);

/**
* @brief  Add user face into Flash DB
*
* @param  user_idx: index of user to be added into Flash DB
* 
* @return 0: Stop running FDFR recognition
*        -1: Both of RGB and NIR cameras aren't opened yet
*         1: No space of database
*        >1: enum kdp_app_fid_status_code_e
*/
int kl520_api_face_add(uint8_t user_idx);

/**
* @brief  Delete user face into Flash DB
*
* @param  user_idx: index of user to be delete from Flash DB
* 
* @return 0: Stop running FDFR recognition
*        -1: Both of RGB and NIR cameras aren't opened yet
*         1: No space of database
*        >1: enum kdp_app_fid_status_code_e
*/
int kl520_api_face_del(bool all, u8 face_id);

/**
* @brief  Stop/Close both cameras and display preview
*
* @param  N/A
* 
* @return N/A
*/
void kl520_api_face_close(void);

/**
* @brief  Use to set timeout of fdfr inference
*
* @param  timeout: second unit
* 
* @return N/A
*/
void kl520_api_face_recognition_set_timeout(int timeout);

bool kl520_api_timer_init(pwmtimer timer, u32 tick);

void kl520_api_timer_tick_reset(pwmtimer timer);

int kl520_api_timer_close(pwmtimer timer);

#endif
