#include "kl520_api_touch.h"
#include <string.h>
#include "dbg.h"
#include "framework/event.h"
#include "framework/v2k_color.h"
#include "media/display/kdp_2d.h"
#include "media/display/video_renderer.h"
#include "board_ddr_table.h"
#include "touch.h"
#include "kl520_include.h"


osThreadId_t tid_app_touchpanel = 0;

#if CFG_UI_USR_IMG == NO
static void _api_touch_app_icon_handler(kl520_mouse_info * info)
{
    if (MOUSE_DOWN == info->state)
    {
        if ( (80 > info->x) && ((320-80) < info->y) ) {
            kl520_api_dp_set_pen_rgb565(GREEN, 2);
            kl520_api_dp_fill_rect(0, (320-45), 80, 45);
            kl520_api_lcd_fresh();
        }
    }
    else
    {
        kl520_api_dp_draw_bitmap(0, (320-45), 80, 45, (void *)USR_DDR_IMG_02_ICON_01_ADDR);
        kl520_api_lcd_fresh();
    }
}
#endif

static void _api_touch_app_print_data(kl520_mouse_info * info)
{
    DSG2("TOUCH STATE : %d, [%d,%d]", info->state, info->x, info->y);
    //DSG2("TOUCH X : %d", info->x);
    //DSG2("TOUCH Y : %d", info->y);
}

void _api_touch_app_thread(void *arg)
{
    u32 flags;
    while(1)
    {
        kl520_mouse_info info = {MOUSE_NONE, 0, 0};
    #if 0//def USE_SAME_EVENT_DEF
        wait_event(kl520_api_get_event(), KL520_APP_FLAG_TOUCH_ALL);
        clear_event(kl520_api_get_event(), KL520_APP_FLAG_TOUCH_ALL);
    #else
        flags = wait_event(evt_touch_id, KL520_APP_FLAG_TOUCH_ALL);
        clear_event(evt_touch_id, KL520_APP_FLAG_TOUCH_ALL);
    #endif

        if (flags & KL520_APP_FLAG_TOUCH_DEINIT) {
            DSG2("touch panel : app thread, KL520_APP_FLAG_TOUCH_DEINIT");
            break;
        }

        //==============================================
        //customer application start here
        //==============================================
        if (flags & KL520_APP_FLAG_TOUCH) {
            kl520_api_mouse_info_get(&info);

            //example
        #if CFG_UI_USR_IMG == NO
            if (g_touch_sample_app_enable_flag)
                _api_touch_app_icon_handler(&info);
        #endif
            //for printing data (for customer)
        #ifdef KL520_API_SHOW_TOUCH_LOG
            if (MOUSE_NONE != info.state)
                _api_touch_app_print_data(&info);
        #endif
        }            
    }
    osThreadExit();
}

BOOL kl520_api_touch_app_is_enabled(void)
{
    return (NULL != tid_app_touchpanel);
}

void kl520_api_touch_app_enable(void)
{
    kl520_api_touch_set_x_range_max(X_RANGE_MAX);
    kl520_api_touch_set_y_range_max(Y_RANGE_MAX);
    kl520_api_touch_set_x_axis_inverse(0);
    kl520_api_touch_set_y_axis_inverse(1);
    
    {
        osThreadAttr_t attr;
        memset(&attr, 0, sizeof(attr));
        attr.attr_bits = osThreadJoinable;
        DSG2("touch panel : app thread start");    
        if (NULL == tid_app_touchpanel)
            tid_app_touchpanel = osThreadNew(_api_touch_app_thread, NULL, &attr);
    }
}

void kl520_api_touch_app_disable(void)
{
    if (tid_app_touchpanel) {
        set_event(evt_touch_id, KL520_APP_FLAG_TOUCH_DEINIT);
        osThreadJoin(tid_app_touchpanel);
        DSG2("touch panel : app thread end");
        //osThreadTerminate(tid_app_touchpanel);
        tid_app_touchpanel = 0;
    }  
}

