#include "config/board_kdp520.h"
#include "sample_app_console.h"
#include <string.h>
#include <stdlib.h>
#include <cmsis_os2.h>
#include "utility.h"
#include "media/kdp_camera.h"
#include "framework/event.h"
#include "framework/v2k.h"
#include "framework/v2k_color.h"
#include "framework/v2k_image.h"
#include "framework/framework_driver.h"
#include "kdp_uart.h"
#include "kdp520_pwm_timer.h"
#include "kdp520_adc.h"
#include "kdp520_gpio.h"
#include "kdp_i2c.h"
#include "dbg.h"
#include "sample_kdp_fdr.h"
#include "kdp_memxfer.h"
#include "sample_app_touch.h"
#include "touch_panel.h"
#include "system.h"
#include "power.h"
#include "ddr.h"
#include "rtc.h"
#include "io.h"
#include "delay.h"
#include "power_manager.h"
#include "uart_com.h"
#include "kdp_app.h"
#include "kdp_app_db.h"
#include "kl520_include.h"
#include "kl520_api.h"
#include "kl520_api_camera.h"
#include "kl520_api_fdfr.h"
#include "kl520_api_device_id.h"
#include "media/kdp_inference.h"
#include "media/camera/sensor.h"

#define DUMP_IMG_HEAD 0x62000000
#define IMG_SIZE    640*480*2
#define SAC_PROFILE_ENABLE
#define VERIFICATION_USE    (0)
#ifdef SAC_PROFILE_ENABLE 
#define SAC_PROFILE_START() _prof_start = osKernelGetTickCount(); 
#define SAC_PROFILE_STOP() { \
                _prof_end = osKernelGetTickCount(); \
                _prof_offset = _prof_end - _prof_start; \
                _prof_total += _prof_offset; \
                ++_prof_frame_cnt; \
                /*DSG("[%s] offset=%u average=%u", __func__, _prof_offset, _prof_total / _prof_frame_cnt);*/ \
                }
#else
#define SAC_PROFILE_START() 
#define SAC_PROFILE_STOP() 
#endif

typedef void  (*console_cmd_func)(void);
struct console_cmd_info
{
    char *desc;
    console_cmd_func func;
    //void (*func)(void);
};

static u32 m_curr_user_idx = 0;
static int reg_index;
osThreadId_t tid_doorlock_console = 0;

#ifdef BOARD_CUSTOMER1
void sample_open_nir_led(void)
{
    DSG("open nir led!\n");    
    nir_led_open();
}

void sample_close_nir_led(void)
{
    DSG("close nir led!\n");
    nir_led_close();
}
#endif

void sample_change_draw_fdr_result_mode(void)
{
    char buf[16];
    int mode;

    DSG_NOLF(" 1(Only hint boundingbox)/2(Only fdfr result boundingbox)/3(All boundingbox). Please enter 1 ~ 3 :");
    uart_com_log_echo_gets(buf, sizeof(buf));
    mode = atoi(strtok(buf, " \r\n\t"));
    
    if(mode <= ALL_DRAW_BOUNDINGBOX_MODE) {
         kdp_fdfr_set_boundingbox_mode(mode);
    }
}

void sample_prepare_inference(void)
{
    kl520_api_camera_open(CAMERA_RGB_IDX);
    kl520_api_camera_start(CAMERA_RGB_IDX);
    kl520_api_dp_open(CAMERA_RGB_IDX, LCD_WIDTH, LCD_HEIGHT, V2K_PIX_FMT_RGB565);

    kl520_api_camera_open(CAMERA_NIR_IDX);
    kl520_api_camera_start(CAMERA_NIR_IDX);

    kl520_api_fdfr_model_init();
}

#ifdef PERIPHERAL_TEST  
void sample_pwm5_test(void)
{
    char buf[256];
    unsigned int period, duty, polarity;

    DSG("[PWM5 Test]");
    DSG("Please input the period(10ns)");
    uart_com_log_echo_gets(buf, sizeof(buf));
    period = atoi(buf);
    DSG("Please input duty(10ns)");
    uart_com_log_echo_gets(buf, sizeof(buf));
    duty = atoi(buf);
    DSG("Please input polarity(0:low, 1:high)");
    uart_com_log_echo_gets(buf, sizeof(buf));
    polarity = atoi(buf);
    DSG("period = %d, duty = %d, polarity = %d", period, duty, polarity);

    kdp_pwm_config((pwmtimer)PWMTIMER6, (pwmpolarity)polarity, duty, period);
    kdp_pwm_enable((pwmtimer)PWMTIMER6);
    DSG("setting done!");

}	

void sample_adc_test(void)
{
    kdp_adc_init(0);
}	

void sample_gpio_test(void)
{
    char buf[256];
    unsigned int gpio_pin, dir, trigger_level, trigger_mode, single_both_edge;
    unsigned int temp;

    DSG("[GPIO Test]");
    DSG("Please input the gpio pin");
    uart_com_log_echo_gets(buf, sizeof(buf));
    gpio_pin = atoi(buf);
    kdp520_gpio_setmode(gpio_pin);
    
    DSG("Please select the direction of gpio(input=0; output=1):");
    uart_com_log_echo_gets(buf, sizeof(buf));
    dir = atoi(buf);
    kdp520_gpio_setdir(gpio_pin, dir);
    
    if(0 == dir)//input
    {
        DSG("Please input the trigger mode(edge=0; level=1):");
        uart_com_log_echo_gets(buf, sizeof(buf));
        trigger_level = atoi(buf);
        
        if(0 == trigger_level)//edge
        {
            DSG("Trigger by single edge or both edge?(single=0; both=1)");
            uart_com_log_echo_gets(buf, sizeof(buf));
            single_both_edge = atoi(buf);
            if(0 == single_both_edge)
            {
                DSG("Trigger by rising edge or falling edge?(rising=0; falling=1)");
                uart_com_log_echo_gets(buf, sizeof(buf));
                trigger_mode = atoi(buf);
            }
        }
        else//level
        {
            DSG("Trigger by high level or low level?(high=0; low=1)");
            uart_com_log_echo_gets(buf, sizeof(buf));
            trigger_mode = atoi(buf);
        }
        kdp520_gpio_setedgemode(gpio_pin, single_both_edge);
        kdp520_gpio_settrigger(gpio_pin, trigger_level);
        kdp520_gpio_setactivemode(gpio_pin, trigger_mode);
        kdp520_gpio_setintenable(gpio_pin);
        kdp520_gpio_setintunmask(gpio_pin);
    }
    else//output
    {
        DSG("data set or data clear(1/0)");
        uart_com_log_echo_gets(buf, sizeof(buf));
        temp = atoi(buf);
        
        if(temp)//set
            kdp520_gpio_setdata(1<<gpio_pin);
        else//clear
            kdp520_gpio_cleardata(1<<gpio_pin);
    }
    DSG("Exit gpio test");
}	

void sample_i2c_test(void)  
{
    char buf[256];
    unsigned int cmd, reg_len;
    u8 i2c_address;
    u16 reg_address;
    u8 read_data = 0;
    u8 write_data = 0;

    DSG("Please input the slave address(Hex):");
    uart_com_log_echo_gets(buf, sizeof(buf));
    i2c_address = strtol(buf, NULL, 16);

    DSG("Please input command(read=0; write=1):")
    uart_com_log_echo_gets(buf, sizeof(buf));
    cmd = atoi(buf);

    DSG("Please input register address(Hex):");
    uart_com_log_echo_gets(buf, sizeof(buf));
    reg_address = strtol(buf, NULL, 16);

    DSG("Please input the bytes of register's value:");
    uart_com_log_echo_gets(buf, sizeof(buf));
    reg_len = atoi(buf);
    if (reg_len == 0) reg_len = 1;
    if (reg_len > 2) reg_len = 2;

    if(cmd == 1)
    {
        DSG("Please input the value to write:");
        uart_com_log_echo_gets(buf, sizeof(buf));
        write_data = strtol(buf, NULL, 16);
        kdp_i2c_transmit(KDP_I2C_CTRL_0, (u16) i2c_address, reg_address, reg_len, 1, &write_data);
        //kdp_drv_i2c_write(I2C_ADAP_0, i2c_address, reg_address, reg_len, write_data);
    }

    else if(cmd == 0)
    {
        kdp_i2c_receive(KDP_I2C_CTRL_0, i2c_address, reg_address, reg_len, 1, &read_data);
        //kdp_drv_i2c_read(I2C_ADAP_0, i2c_address, reg_address, reg_len, &read_data);
        DSG("the register 0x%x of slave address 0x%x is 0x%x", reg_address, i2c_address, read_data);
    }

}
#endif

static int _cmd_get_answer_int(const char *str)
{
    int ans;
    char buf[256];

    DSG_NOLF(str);
    uart_com_log_echo_gets(buf, sizeof(buf));
    DSG_NOLF("");
    ans = atoi(strtok(buf, " \r\n\t"));
    return ans;
}

void sample_camera_open(void)
{
    int ret, cam_idx;
    cam_idx = _cmd_get_answer_int("open camera, index >>");
    kl520_api_camera_open(cam_idx);
    info_msg("%s %d - ret:%d\n", __func__, cam_idx, ret);
}

void sample_camera_start(void)
{
    int ret, cam_idx;
    cam_idx = _cmd_get_answer_int("start camera, index >>");
    kl520_api_camera_start(cam_idx);
    info_msg("%s %d - ret:%d\n", __func__, cam_idx, ret);
}

void sample_camera_stop(void)
{
    int ret, cam_idx;
    cam_idx = _cmd_get_answer_int("stop camera, index >>");
    kl520_api_camera_stop(cam_idx);
    info_msg("%s %d - ret:%d\n", __func__, cam_idx, ret);
}

void sample_camera_close(void)
{
    int ret, cam_idx;
    cam_idx = _cmd_get_answer_int("close camera, index >>");
    ret = kl520_api_camera_close(cam_idx);
    info_msg("%s %d - ret:%d\n", __func__, cam_idx, ret);
}

void sample_open_video_renderer(void)
{
    bool ret;
    ret = kl520_api_dp_open(CAMERA_RGB_IDX, LCD_WIDTH, LCD_HEIGHT, V2K_PIX_FMT_RGB565);
    info_msg("%s %d - ret:%d\n", __func__, CAMERA_RGB_IDX, ret);
}

void sample_switch_video_renderer_src(void)
{
    kl520_api_camera_switch_rbg_nir();
}

void sample_close_video_renderer(void)
{
    kl520_api_dp_close();
}

void sample_face_recognition(void)
{
    int ret;

    kl520_api_face_recognition_set_timeout(10);
    ret = kl520_api_face_recognition();
    dbg_msg("%s - ret:%d\n", __func__, ret);
}

void sample_face_loop_recognition(void)
{
    int ret;
    ret = kl520_api_face_recognition_loop();
    dbg_msg("%s - ret:%d\n", __func__, ret);
}

void sample_face_loop_recognition_stop(void)
{
    int ret;
    ret = kl520_api_face_recognition_loop_stop();
    dbg_msg("%s - ret:%d\n", __func__, ret);
}

void sample_face_register(void)
{
    int ret;
    ret = kl520_api_face_register(m_curr_user_idx);
    dbg_msg("%s m_curr_user_idx:%d - ret:0x%x\n", __func__, m_curr_user_idx, ret);
}

void sample_face_auto_register(void)
{
    int ret;
    ret = kl520_api_face_auto_register(m_curr_user_idx, reg_index);
    dbg_msg("%s m_curr_user_idx:%d, reg_index:%d - ret:0x%x\n", __func__, m_curr_user_idx, reg_index, ret);
}

void sample_face_cancel_register(void)
{
    int ret;
    ret = kl520_api_face_cancel_register();
    dbg_msg("%s - ret:0x%x\n", __func__, ret);
}

void sample_fdr_auto_register_user_both(void)
{
    int ret;
    ret = kl520_api_face_auto_register_both(m_curr_user_idx, reg_index);
    dbg_msg("%s - ret:0x%x\n", __func__, ret);
}

void sample_fdr_register_user_with_pose_justify(void)
{
    int ret;
    ret = kl520_api_face_auto_register_5_face(m_curr_user_idx, reg_type);
    dbg_msg("%s - ret:0x%x\n", __func__, ret);
}

void sample_face_add(void)
{
    int ret;
    ret = kl520_api_face_add(m_curr_user_idx);
    dbg_msg("%s m_curr_user_idx:%d - ret:0x%x\n", __func__, m_curr_user_idx, ret);
}

void sample_face_del_user(void)
{
    int ret;
    ret = kl520_api_face_del(FALSE, m_curr_user_idx);
    dbg_msg("%s m_curr_user_idx:%d - ret:0x%x\n", __func__, m_curr_user_idx, ret);
}

void sample_face_del_all(void)
{
    int ret;
    ret = kl520_api_face_del(TRUE, m_curr_user_idx);
    dbg_msg("%s m_curr_user_idx:%d - ret:0x%x\n", __func__, m_curr_user_idx, ret);
}

void sample_face_close(void)
{
    kl520_api_face_close();
}

void sample_fdr_init(void)
{
    kl520_api_fdfr_model_init();
}

void sample_stability_recognition_loop_test(void)
{
    int count = 0;
    while(1)
    {
        kl520_api_face_recognition_loop();
        osDelay(3000);
        kl520_api_face_recognition_loop_stop();
        sample_face_close();
        osDelay(1000);
        count++;
        DSG("==========================================================================%d", count);
        DSG("Thread NUM:%d",osThreadGetCount());
    }
}

void sample_stability_recognition_test(void)
{
    int count = 0;
    int ret;
    while(1)
    {
        ret = kl520_api_face_recognition();
        if (FDR_RET_DB_COMP_OK == ret)
        {
            info_msg("sample_face_recognition, KL520_FACE_OK\n");
        }
        else {
            info_msg("sample_face_recognition, ERROR, ret=0x%x\n", ret);
            continue;
        }
        osDelay(3000);
        sample_face_close();
        osDelay(1000);
        count++;
        DSG("==========================================================================%d", count);
        DSG("Thread NUM:%d",osThreadGetCount());
    }
}

void sample_get_system_info(void)
{
    system_info t_sys_info;

    kl520_api_fdfr_model_init();
    kl520_api_get_device_info(&t_sys_info);

    DSG_NOLF("unique_id : 0x%x\n", t_sys_info.unique_id);
    DSG_NOLF("boot loader version : 0x%x\n", t_sys_info.spl_version);
    DSG_NOLF("scpu firmware version : 0x%x\n", t_sys_info.fw_scpu_version);
    DSG_NOLF("ncpu firmware version : 0x%x\n", t_sys_info.fw_ncpu_version);
    DSG_NOLF("device name : %s, id : 0x%x\n", t_sys_info.device_id_0.device_name, t_sys_info.device_id_0.id);
    DSG_NOLF("device name : %s, id : 0x%x\n", t_sys_info.device_id_1.device_name, t_sys_info.device_id_1.id);
    DSG_NOLF("device name : %s, id : 0x%x\n", t_sys_info.device_id_2.device_name, t_sys_info.device_id_2.id);
    DSG_NOLF("device name : %s, id : 0x%x\n", t_sys_info.device_id_3.device_name, t_sys_info.device_id_3.id);
   // DSG_NOLF("device name : %s, id : %x\n", t_sys_info.device_id_4.device_name, t_sys_info.device_id_4.id);
   // DSG_NOLF("device name : %s, id : %x\n", t_sys_info.device_id_5.device_name, t_sys_info.device_id_5.id);
}

void sample_app_console_quit(void)
{
    DSG("bye bye !!");
    power_manager_shutdown();
}

struct console_cmd_info console_cmd_array[] = {
    {"Combo: Prepare RGB/NIR Inference",            sample_prepare_inference  },
    {"Combo: Close RGB/NIR/PANEL Display",          sample_face_close},
    {"Switch camera source on display",             sample_switch_video_renderer_src},
    {"Start face recognition",                      sample_face_loop_recognition},
    {"Stop face recognition",                       sample_face_loop_recognition_stop},
    {"Manual register user face",                   sample_face_register},
    {"Auto register user face ",                    sample_face_auto_register},
    {"Auto register 5 position face",               sample_fdr_register_user_with_pose_justify },
    {"Abort registration record",                   sample_face_cancel_register},
    {"Add user face to database",                   sample_face_add},
    {"Delete user face from database",              sample_face_del_user},
    {"Delete all user faces from database",         sample_face_del_all},
    {"Stability recognition test",                  sample_stability_recognition_loop_test},
    {"Get system information",                      sample_get_system_info},
    {"Change draw FDR result mode",                 sample_change_draw_fdr_result_mode },
    {"open NIR led",							    sample_open_nir_led  },
    {"close NIR led",							    sample_close_nir_led  },

#if VERIFICATION_USE == 1
    {"Open camera",                                 sample_camera_open },
    {"Start camera",                                sample_camera_start },
    {"Stop camera",                                 sample_camera_stop },
    {"Close camera",                                sample_camera_close },
    {"sample_open_video_renderer",                  sample_open_video_renderer},
    {"sample_close_video_renderer",                 sample_close_video_renderer},
    {"Init Kneron Face Recognition",                sample_fdr_init },
    {"Auto register user face (RGB+NIR)",           sample_fdr_auto_register_user_both},
#ifdef PERIPHERAL_TEST      
    {"PWM function(PWM5) ",                         sample_pwm5_test },
    {"ADC function",                                sample_adc_test },
    {"GPIO function",                               sample_gpio_test },
    {"I2C function",                                sample_i2c_test  },
#endif
//#ifdef DEBUG_POWER_MGR
    {"power mode : reset",                          power_manager_reset },
    {"power mode : retention",                      power_manager_sleep },
    {"power mode : deep retention",                 power_manager_deep_sleep },
    {"power mode : RTC mode",                       power_manager_shutdown },
//#endif
    {"sample_stability_recognition_test",           sample_stability_recognition_test},
#endif
    {"Quit",                                        sample_app_console_quit }
};

void doorlock_console_thread(void *arg)
{
    unsigned int i = 0;
    unsigned int cmd_size = ARRAY_SIZE(console_cmd_array);
    char buf[256];
    console_cmd_func cmd_func;
    uint32_t rc = 1;

    while(1)
    {
        int id = 0;
        int user_idx;
        int remaining_fmap_cnt;

        DSG("\n === Klp520 sample app console Test Kit (%u) === ", cmd_size);
        for (i = 0; i < cmd_size; ++i)
        { 
            sprintf(buf, "(%2d) %s ", i + 1, console_cmd_array[i].desc);
            DSG("%s", buf);
        }

        DSG_NOLF(" command >> ");
        rc = uart_com_log_echo_gets(buf, sizeof(buf));
        if (!rc)
            continue;
        id = atoi(strtok(buf, " \r\n\t"));

        //DSG("id=%d", id);
        if (id > 0 && id < cmd_size)
        {
            cmd_func = console_cmd_array[id - 1].func;
            //DSG("console_cmd_array[%d].func=%p", id - 1, console_cmd_array[id - 1].func);
            if (cmd_func) {
                if(cmd_func == sample_face_register) {
                    user_idx = kdp_fdfr_get_available_db_idx();
                    if(user_idx >= 20)
                    {
                        DSG_NOLF(" No DB space anymore, DB Maximum is 20\n");
                        continue;
                    }
l_rgst_cmd:
                    remaining_fmap_cnt = kdp_fdfr_get_remaining_fmap_cnt(user_idx);
                    DSG_NOLF(" try to register index %d, remaining fmap_cnt %d\n", user_idx, remaining_fmap_cnt);
                    if (kdp_app_db_get_fm0_cnt(user_idx)==65535) {
                        DSG_NOLF(" try to register index %d fm cnt is uninitialized\n", user_idx);
                    }
                    else if(remaining_fmap_cnt > 0)
					{
                        DSG_NOLF(" Assign UID=%d, ready to capture image(%d) to register, Y/N? ",
							user_idx, remaining_fmap_cnt);
	                }
                    else if(remaining_fmap_cnt == 0)
                    {
                        DSG_NOLF(" UID=%d, remaining %d can be registered\n",
							user_idx, remaining_fmap_cnt);
                        continue;
                    }
                    uart_com_log_echo_gets(buf, sizeof(buf));

                    if(strncmp(buf, "Y", 1) == 0 || strncmp(buf, "y", 1) == 0)
                        m_curr_user_idx = user_idx;
                    else if(strncmp(buf, "N", 1) == 0 || strncmp(buf, "n", 1) == 0)
                        continue;
                    else
                        goto l_rgst_cmd;

                    cmd_func();
                    goto l_rgst_cmd;
                }
                else if (cmd_func == sample_face_auto_register) {
                    sample_face_loop_recognition_stop();
                    user_idx = kdp_fdfr_get_available_db_idx();
                    if(user_idx >= 20)
                    {
                        DSG_NOLF(" No DB space anymore, DB Maximum is 20\n");
                        continue;
                    }
                    m_curr_user_idx = user_idx;
                    DSG_NOLF(" Start Auto Register for UID:%d! (Move Your Face Slowly)\n", user_idx);
                    reg_index = 0;
                    for (int i = 0; i < 20; i++) {
                        //DSG_NOLF(" reg_index %d !\n", reg_index);
                        info_msg(".");
                        memcpy((void *) (DUMP_IMG_HEAD+i*IMG_SIZE), (const void *) 0x62FFE000, IMG_SIZE*sizeof(char));
                        cmd_func();
                        reg_index++;
                        delay_us(50000); //adjust according to UX
                    }
                    info_msg(" Done \n");
                    kl520_api_face_register_cfm(m_curr_user_idx);                                      
                }
                else if (cmd_func == sample_fdr_auto_register_user_both) {
                    sample_face_loop_recognition_stop();
                    user_idx = kdp_fdfr_get_available_db_idx();
                    if(user_idx >= 20)
                    {
                        DSG_NOLF(" No DB space anymore, DB Maximum is 20\n");
                        continue;
                    }
                    m_curr_user_idx = user_idx;
                    DSG_NOLF(" Start Auto Register for UID:%d! (Move Your Face Slowly)\n", user_idx);
                    reg_index = 0; 

                    for (int i = 0; i < 20; i++) {
                        info_msg(".");
                        // memcpy((void *) (DUMP_IMG_HEAD+i*IMG_SIZE), (const void *) 0x62FFE000, IMG_SIZE*sizeof(char));
                        // memcpy((void *) (DUMP_IMG_HEAD+(i+20)*IMG_SIZE), (const void *) 0x6357C400, 640*480*sizeof(char));
                        cmd_func();
                        reg_index++;
                        delay_us(50000); //adjust according to UX
                    }
                    DSG_NOLF(" Done Auto Register for user idx %d !\n", m_curr_user_idx);
                    kl520_api_face_register_cfm_both(m_curr_user_idx);                 
                }
                else if(cmd_func == sample_fdr_register_user_with_pose_justify) {
					reg_type = 0;
                    
                    user_idx = kdp_fdfr_get_available_db_idx();
                    if(user_idx >= 20)
                    {
                        DSG_NOLF(" No DB space anymore, DB Maximum is 20\n");
                        continue;
                    }
l_rgst_pose_cmd:
                   dbg_msg("reg_count: %d\n", reg_count);
                   reg_count += 1;
                    if(reg_count >= 200){
                        reg_type += 1;
                        reg_count = 0;
                    }
                    remaining_fmap_cnt = kdp_fdfr_get_remaining_fmap_cnt(user_idx);
                    dbg_msg(" try to register index %d, remaining fmap_cnt %d\n", user_idx, remaining_fmap_cnt);
                    if (kdp_app_db_get_fm0_cnt(user_idx)==65535) {
                        dbg_msg(" try to register index %d fm cnt is uninitialized\n", user_idx);
                    }
                    else if(remaining_fmap_cnt > 0 && reg_type < 5)
					{
                        dbg_msg(" Assign UID=%d, ready to capture image(%d) to register\n",
							user_idx, remaining_fmap_cnt);
                        m_curr_user_idx = user_idx;

                        if(reg_type == 0){
                            DSG_NOLF("CENTER pose, please\n");
                        }else if(reg_type == 1){
                            DSG_NOLF("LEFT pose, please\n");
                        }else if(reg_type == 2){
                            DSG_NOLF("RIGHT pose, please\n");
                        }else if(reg_type == 3){
                            DSG_NOLF("UP pose, please\n");
                        }else if(reg_type == 4){
                            DSG_NOLF("DOWN pose, please\n");
                        }
	                }
                    else if(remaining_fmap_cnt == 5 && reg_type >= 5)
                    {
                        DSG_NOLF("Fail to capture any validate pictures\n");
                        continue;
                    }else if(remaining_fmap_cnt < 5 && reg_type >= 5){
                        DSG_NOLF(" UID=%d, remaining %d can be registered\n",
						user_idx, remaining_fmap_cnt);
                        continue;
                    }
                    //DSG_NOLF(" DB Maximum is 5.[add_user] Please enter 0 ~ 4 : ");
                    // m_show_txt = FALSE;                    
                    // uart_com_log_echo_gets(buf, sizeof(buf));
                    // m_show_txt = TRUE;

                    // if(strncmp(buf, "Y", 1) == 0 || strncmp(buf, "y", 1) == 0)
                    //     m_curr_user_idx = user_idx;
                    // else if(strncmp(buf, "N", 1) == 0 || strncmp(buf, "n", 1) == 0)
                    //     continue;
                    // else
                    //     goto l_rgst_cmd;

                    cmd_func();
                    goto l_rgst_pose_cmd;
                }
                else if(cmd_func == sample_face_del_user) {
                    DSG_NOLF(" [del_user] Please enter 0 ~ 19 : ");
                    uart_com_log_echo_gets(buf, sizeof(buf));
                    m_curr_user_idx = atoi(strtok(buf, " \r\n\t"));
                    if(m_curr_user_idx == 0 && buf[0] != '0')  {
                        DSG(" [del_user] wrong input, abort !");
                        continue;
                    }
                    cmd_func();
                }
                else {
                    cmd_func();
                }
            }
        }
        else if (cmd_size == id) {
            cmd_func = console_cmd_array[id - 1].func;
            cmd_func();
            break;
        }
        else
            continue;
    }
}

void sample_doorlock_entry(void)
{
#ifdef V2K_ENABLE
    /*Configure camera setting*/
    kl520_api_camera_setting(CAMERA_RGB_IDX, CAMERA_RGB_IMG_WIDTH, CAMERA_RGB_IMG_HEIGHT, IMG_PIX_FMT_RGB565);
    kl520_api_camera_setting(CAMERA_NIR_IDX, CAMERA_NIR_IMG_WIDTH, CAMERA_NIR_IMG_HEIGHT, IMG_PIX_FMT_RAW8);
    
	if(0 != kdp_fdfr_inference_thread_create())
        return;

    tid_doorlock_console = osThreadNew(doorlock_console_thread, NULL, NULL);
    osThreadSetPriority (tid_doorlock_console, osPriorityBelowNormal);

#endif
}

