#include "sample_app_touch.h"
#include "touch_panel.h"
#include "base.h"
#include "cmsis_os2.h"
#include "dbg.h"


#ifdef BOARD_CUSTOMER1 

const u32 flag_app_touchpanel_evt = 0x0080;
osThreadId_t tid_touchpanel;
static t_app_touch_data __app_touch_data;
static touch_type_click __click[TOUCH_CLICK_MAX_NUM];
static s32 __click_reg_tbl[TOUCH_EVENT_MAX_NUM][6] =
{
    // { touch_func, x, y, range, init_status, active_layer }
    {UI_RETURN_TO_FRONT_PAGE, TOUCH_RETURN_X,    TOUCH_RETURN_Y,    TOUCH_CLICK_RANGE,  TOUCH_INACTIVE, STATE_MAIN_MENU},
    {UI_RETURN_TO_FRONT_PAGE, TOUCH_RETURN_X,    TOUCH_RETURN_Y,    TOUCH_CLICK_RANGE,  TOUCH_INACTIVE, STATE_OPEN_REGISTER},
    {UI_RETURN_TO_FRONT_PAGE, TOUCH_RETURN_X,    TOUCH_RETURN_Y,    TOUCH_CLICK_RANGE,  TOUCH_INACTIVE, STATE_INFERENCE},
    {UI_OPEN_REGISTER,        TOUCH_REG_OPEN_X,  TOUCH_REG_OPEN_Y,  TOUCH_CLICK_RANGE,  TOUCH_INACTIVE, STATE_MAIN_MENU},
    {UI_START_REGISTER,       TOUCH_REG_START_X, TOUCH_REG_START_Y, TOUCH_CLICK_RANGE_1,TOUCH_INACTIVE, STATE_OPEN_REGISTER},
    {UI_INFERENCE,            TOUCH_INFE_X,      TOUCH_INFE_Y,      TOUCH_CLICK_RANGE,  TOUCH_INACTIVE, STATE_MAIN_MENU},
    {UI_DELETE_ALL,           TOUCH_REG_DELETE_X,TOUCH_REG_DELETE_Y,TOUCH_CLICK_RANGE,  TOUCH_INACTIVE, STATE_MAIN_MENU},
    {TYPE_NULL},
};

void sample_app_init_touch_panel(void);
static void sample_app_touch_motion_register(void);
static void sample_app_touch_event_handler(void);

void touchpanel_thread(void *arg)
{
    sample_app_touch_motion_register();

    while(1)
    {
        int flag = 0;
        int err = 0;

        //loop time : 200ms
        flag = osThreadFlagsWait(flag_app_touchpanel_evt, osFlagsWaitAny, 200);
        osThreadFlagsClear(flag&(osFlagsErrorTimeout|flag_app_touchpanel_evt));
        err = flag&osFlagsErrorTimeout;

        if (osFlagsErrorTimeout == err)//flag&osFlagsErrorTimeout)
            __app_touch_data.count = 0;//reset touch event
        else if (flag_app_touchpanel_evt == (flag&flag_app_touchpanel_evt))
            touch_get_raw_data(1);

        sample_app_touch_event_handler();
    }
}

void sample_app_init_touch_panel(void)
{
    tid_touchpanel = osThreadNew(touchpanel_thread, NULL, NULL);
    osThreadSetPriority (touchpanel_thread, osPriorityBelowNormal);    
}

static void sample_app_touch_motion_register(void)
{
    u8 i = 0;

    //click
    for (i=0; i<TOUCH_EVENT_MAX_NUM; i++)
    {
        if (TYPE_NULL == (user_behavior_type)__click_reg_tbl[i][0])
            return;
        
        __click[i].ui_event = (user_behavior_type)__click_reg_tbl[i][0];
        __click[i].x = __click_reg_tbl[i][1];
        __click[i].y = __click_reg_tbl[i][2];
        __click[i].range = __click_reg_tbl[i][3];
        __click[i].status = (e_touch_status)__click_reg_tbl[i][4];
        __click[i].activate_state = (fsm_state)__click_reg_tbl[i][5];
    }

}

static void sample_app_touch_event_handler(void)
{
    u8 state = ui_get_main_state();
    u8 count = __app_touch_data.count;

    if (!count){
        //reset all touch events and return.
        for (u8 i=0; i<TOUCH_CLICK_MAX_NUM; i++)
            __click[i].status = TOUCH_INACTIVE;
        return;
    }

    for (u8 i=0; i<TOUCH_EVENT_MAX_NUM; i++)
    {
        //each click object should have their active layer(state).
        if (state != __click[i].activate_state) {continue;}

        if (TOUCH_INACTIVE != __click[i].status) {continue;}

        for (u8 k=0; k<count; k++){
            if ( __app_touch_data.input_x[k] > (__click[i].x - __click[i].range) &&
                 __app_touch_data.input_x[k] < (__click[i].x + __click[i].range) &&
                 __app_touch_data.input_y[k] > (__click[i].y - __click[i].range) &&
                 __app_touch_data.input_y[k] < (__click[i].y + __click[i].range) )
            {
                notify_user_behavior_event(__click[i].ui_event, 0);
                __click[i].status = TOUCH_CLICK_ACTIVE;
                return;//only enable single touch func.
            }
            else
                __click[i].status = TOUCH_INACTIVE;
        }
    }

}

extern void sample_app_touch_point_setting(u8 id,  u16 input_x, u16 input_y){
    __app_touch_data.input_x[id] = input_x;
    __app_touch_data.input_y[id] = input_y;
}
extern void sample_app_touch_count_setting(u8 cnt){
    __app_touch_data.count = cnt;
}

#endif
