#if 1//def BOARD_DATANG
#include "config/board_kdp520.h"
#include "user_ui.h"
#include "base.h"
#include "framework/event.h"
#include "cmsis_os2.h"
#include "kdp_memxfer.h"
#include "pinmux.h"
#include "kdp520_gpio.h"
#include "ui_fsm_proto.h"
#include "dbg.h"

#define MEMXFER_USE_DRIVER_STYLE

typedef enum
{
    SYS_FDR_IDLE = 0,
    SYS_FDR_REG = 1,
    SYS_FDR_INFE = 2,
}sys_fdr_state_t;
    
const u32 flag_lcd_display_evt_0 = 0x0100;
const u32 flag_lcd_display_evt_1 = 0x0200;
const u32 flag_lcd_display_evt_2 = 0x0400;
const u32 flag_lcd_display_evt_3 = 0x0800;
const u32 flag_lcd_display_evt_4 = 0x1000;
const u32 flag_lcd_display_evt_5 = 0x2000;
const u32 flag_lcd_display_evt_6 = 0x4000;
const u32 flag_fdr_reg_done_evt = 0x00020000;

u32 sys_fdr_state = 0;
osThreadId_t tid_ui_fsm;

extern const struct s_kdp_memxfer kdp_memxfer_module;

#ifdef BOARD_DATANG
extern void sample_open_nir_led(void);
extern void sample_close_nir_led(void);
#endif
#ifndef MEMXFER_USE_DRIVER_STYLE
int kdp_memxfer_init(u8 flash_mode, u8 mem_mode);
int kdp_memxfer_flash_to_ddr(u32 dst, u32 src, size_t bytes);
#endif

extern void console_set_reg_start_event(void);
extern void console_set_fdr_delete_all_event(void);

extern void sample_app_macro_cam_1_to_lcm_start(void);
extern void sample_app_macro_dummy_cam_to_lcm_start(int cam_idx);
extern void sample_app_macro_cam_0_start(void);
extern void sample_app_macro_cam_1_start(void);
extern void sample_fdr_init(void);
extern void sample_fdr_start_inference_camera_0(void);
extern void sample_fdr_start_inference_camera_1(void);
extern void sample_fdr_stop_inference_camera_0(void);
extern void sample_fdr_stop_inference_camera_1(void);
extern void sample_open_camera_1(void);
extern void sample_start_camera_1(void);
extern void sample_stop_video_renderer_from_camera_0(void);
extern void sample_stop_video_renderer_from_camera_1(void);
extern void sample_stop_video_renderer_from_camera(int cam_idx);
extern void sample_open_nir_led(void);

void ui_op_wait_reg_done_event(void)
{
    //wait_thread_event(flag_fdr_reg_done_evt);
    osThreadFlagsWait(flag_fdr_reg_done_evt, osFlagsWaitAny, 1000);//wait 1s
}

void ui_op_set_reg_done_event(void)
{
    set_thread_event(tid_ui_fsm, flag_fdr_reg_done_evt);
}

void ui_op_main_menu(void)
{
    osDelay(1200);//delay for main menu
    sample_app_macro_dummy_cam_to_lcm_start(2);
}

void ui_op_front_page(void)
{
    sample_fdr_stop_inference_camera_1();//21
    sample_fdr_stop_inference_camera_0();	// 22
#ifdef BOARD_DATANG    
    sample_close_nir_led();		// 31
#endif    
    sample_stop_video_renderer_from_camera_1();
    sample_app_macro_dummy_cam_to_lcm_start(3);
    sys_fdr_state = SYS_FDR_IDLE;
    notify_user_behavior_event(UI_RETURN_TO_MAIN_MENU, 0);

    //sample_app_cam_thread_mipi0_terminate();
    //sample_app_cam_thread_mipi1_terminate();
}

int ui_op_inference(void)
{
    int ret = -1;
    if (SYS_FDR_IDLE != sys_fdr_state)
        return ret;
    sys_fdr_state = SYS_FDR_INFE;

    //sample_app_cam_thread_mipi0_create();
    //sample_app_cam_thread_mipi1_create();

    sample_stop_video_renderer_from_camera(2);
    sample_stop_video_renderer_from_camera(3);
osDelay(100);
    sample_app_macro_cam_0_start();
    sample_app_macro_cam_1_start();
    sample_open_nir_led();		// 30
osDelay(100);
    sample_fdr_start_inference_camera_0();//14
osDelay(100);   
    sample_fdr_start_inference_camera_1();//20
    
    ret = 0;
    return ret;
}

int ui_op_open_registration(void)
{
    int ret = -1;
    if (SYS_FDR_IDLE != sys_fdr_state)
        return ret;
    sys_fdr_state = SYS_FDR_REG;
    
    //sample_app_cam_thread_mipi1_create();

    sample_stop_video_renderer_from_camera(2);
    sample_stop_video_renderer_from_camera(3);
osDelay(100);
    sample_app_macro_cam_0_start();
    sample_app_macro_cam_1_start();
sample_open_nir_led();		// 30
osDelay(100);
    sample_fdr_start_inference_camera_1();//20
    
    ret = 0;
    return ret;
}

int ui_op_start_registration(void)
{       
    int ret = -1;
    if (SYS_FDR_INFE==sys_fdr_state)
        return ret;

    console_set_reg_start_event();
    ui_op_wait_reg_done_event();
    notify_user_behavior_event(UI_RETURN_TO_FRONT_PAGE, 0);

    ret = 0;
    return ret;
}

int  ui_op_delete_all(void)
{
    int ret = -1;
    if (SYS_FDR_INFE==sys_fdr_state)
        return ret;    
    
    console_set_fdr_delete_all_event();
    sample_fdr_stop_inference_camera_1();
    notify_user_behavior_event(UI_RETURN_TO_FRONT_PAGE, 0);
    ret = 0;
    
    return ret;    
}

void user_interface_fsm_thread(void *arg)
{
#ifndef MEMXFER_USE_DRIVER_STYLE
    kdp_memxfer_init(MEMXFER_OPS_CPU, MEMXFER_OPS_CPU);
    kdp_memxfer_flash_to_ddr(KDP_DDR_BASE_IMAGE_FOR_APP, 0x006A1000, KDP_IMAGE_BUF_SIZE_FOR_APP);
    kdp_memxfer_flash_to_ddr(KDP_DDR_BASE_IMAGE_FOR_APP+KDP_IMAGE_BUF_SIZE_FOR_APP, 0x0060B000, KDP_IMAGE_BUF_SIZE_FOR_APP);
#else
    kdp_memxfer_module.init(MEMXFER_OPS_CPU, MEMXFER_OPS_CPU);
    kdp_memxfer_module.flash_to_ddr(KDP_DDR_BASE_IMAGE_FOR_APP, 0x006A1000 , KDP_IMAGE_BUF_SIZE_FOR_APP);
    kdp_memxfer_module.flash_to_ddr(KDP_DDR_BASE_IMAGE_FOR_APP+KDP_IMAGE_BUF_SIZE_FOR_APP, 0x0060B000, KDP_IMAGE_BUF_SIZE_FOR_APP);
#endif

    sample_open_camera_1();//2
    sample_start_camera_1();//4
    sample_fdr_init();//13

    //start the finite state machine.
    int product_type = DEVICE_TYPE_CUSTOMER1;
    create_fsm(product_type);
    ui_fsm->task_run( (void *)0 );//into __run while loop

    DSG("ui fsm error :@got to the end.");
    while(1);/* should never go to here */
}

void ui_op_init(void)
{
    tid_ui_fsm = osThreadNew(user_interface_fsm_thread, NULL, NULL);
    osThreadSetPriority (user_interface_fsm_thread, osPriorityRealtime7);
}



#define MAX_STATE_SIZE      (6)
#define MAX_EVENT_SIZE      (6)
#define MAX_EVENT_QUEUE     (12)
#define MAX_MSG_SIZE        (4)
static osMessageQueueId_t __event_queue = NULL;

static void __push_event(user_behavior_type type,unsigned short data);
static void __run(void *param);
static void __get_event(user_behavior_data* a);

static char __main_state;
static user_behavior_ui_interface __ui =
{
    .task_run = __run,
    .push_event = __push_event,
    .is_run = 0
};

static signed char __a_null(user_behavior_data *event);
static signed char __a_null(user_behavior_data *event){
    DSG("wrong fsm state\n");
    return REMAIN_STATE;
}

static signed char __a_0(user_behavior_data *event);
static signed char __a_0(user_behavior_data *event){
    ui_op_front_page();
    return STATE_FRONT_PAGE;
}

static signed char __a_1(user_behavior_data *event);
static signed char __a_1(user_behavior_data *event){
    ui_op_main_menu();
    return STATE_MAIN_MENU;
}

static signed char __a_2(user_behavior_data *event);
static signed char __a_2(user_behavior_data *event)
{
    ui_op_open_registration();
    return STATE_OPEN_REGISTER;
}

static signed char __a_3(user_behavior_data *event);
static signed char __a_3(user_behavior_data *event)
{
    ui_op_start_registration();
    return STATE_START_REGISTER;
}

static signed char __a_4(user_behavior_data *event);
static signed char __a_4(user_behavior_data *event){
    ui_op_inference();
    return STATE_INFERENCE;
}

static signed char __a_5(user_behavior_data *event);
static signed char __a_5(user_behavior_data *event)
{
    ui_op_delete_all();
    return STATE_DELETE_ALL;
}

static signed char (*__action[MAX_EVENT_SIZE][MAX_STATE_SIZE])(user_behavior_data *event) =
{
    {__a_0,     __a_null, __a_0,    __a_0,    __a_0,    __a_0   }
    ,{__a_1,    __a_null, __a_null, __a_null, __a_null, __a_null}
    ,{__a_null, __a_2,    __a_null, __a_null, __a_null, __a_null}
    ,{__a_null, __a_null, __a_3,    __a_null, __a_null, __a_null}
    ,{__a_null, __a_4,    __a_null, __a_null, __a_null, __a_null}
    ,{__a_null, __a_5,    __a_null, __a_null, __a_null, __a_null}
};

static char __run_main_state_machine(char state,user_behavior_data *event);
static char __run_main_state_machine(char state,user_behavior_data *event)
{
    int i;
    signed char ret;

    i = -1;
    switch (event->type)
    {
        case UI_RETURN_TO_FRONT_PAGE:       i = 0;    break;
        case UI_RETURN_TO_MAIN_MENU:        i = 1;    break;
        case UI_OPEN_REGISTER:              i = 2;    break;
        case UI_START_REGISTER:             i = 3;    break;
        case UI_INFERENCE:                  i = 4;    break;
        case UI_DELETE_ALL:                 i = 5;    break;
        default:
            break;
    }

    if (i != -1)
    {
        ret = (*__action[i][state])(event);
        if (ret != -1)
            return ret;
    }
    return state;
}

static void __push_event(user_behavior_type type,unsigned short data)
{
    unsigned int i;

    i = 0;
    i = (int)type << 24;
    i |= data;

    osMessageQueuePut(__event_queue, &i, NULL, 0);
}

static void __get_event(user_behavior_data* a)
{
    int i;
    //osStatus_t err;

    osMessageQueueGet(__event_queue, &i, NULL, osWaitForever);//100);
    a->data = i & 0x00FFFFFF;
    a->type = (user_behavior_type) (i >> 24);
}

static void __run(void *param)
{
    user_behavior_data event;

    __event_queue = osMessageQueueNew(MAX_EVENT_QUEUE, MAX_MSG_SIZE, NULL);

    __ui.is_run = 1;
    __main_state = 0;
    notify_user_behavior_event(UI_RETURN_TO_FRONT_PAGE, 0);

    while (1)
    {
        __get_event(&event);
        __main_state = __run_main_state_machine(__main_state,&event);
    }
}

extern user_behavior_ui_interface *get_ui_interface(void){
    return &__ui;
}
#else
void ui_op_main_menu(void) {}
void ui_op_front_page(void) {}
int ui_op_inference(void) { return 0; }
int ui_op_open_registration(void) { return 0; }
int ui_op_start_registration(void) { return 0; }
int ui_op_delete_all(void) { return 0; }
#endif
extern u8 ui_get_main_state(void){
    return __main_state;
}

