#ifndef __TIMER_H
#define __TIMER_H

#include "base.h"
#include "kneron_mozart.h"

#define HZ			100				/// how many tick each sec

#define MAX_TIMER					1

#define TIMER_INTSTAT              	0x0
#define TIMER_CR                	0x0
#define TIMER_LOAD					0x4
#define TIMER_COMPARE				0x8
#define TIMER_CNTO					0xc

typedef enum {
    DRVPWMTMR1=1,
    DRVPWMTMR2=2,
    DRVPWMTMR3=3, 
    DRVPWMTMR4=4,
    DRVPWMTMR5=5,
    DRVPWMTMR6=6,
    DRVPWMTMR7=7, 
    DRVPWMTMR8=8
} DRVTIMER;


typedef struct
{	
	u32 TmSrc:1;	/* bit 0 */
	u32 TmStart:1;
	u32 TmUpdate:1;
	u32 TmOutInv:1;
	u32 TmAutoLoad:1;
	u32 TmIntEn:1;
	u32 TmIntMode:1;
	u32 TmDmaEn:1;	
	u32 TmPwmEn:1;	/* bit 8 */
	u32 Reserved:15;	/* bit 9~23 */
	u32 TmDeadZone:8;	/* bit 24~31 */
}fLib_TimerControl;

typedef struct
{	
	u32 Tm1Match1:1;
	u32 Tm1Match2:1;
	u32 Tm1Overflow:1;
	u32 Tm2Match1:1;
	u32 Tm2Match2:1;
	u32 Tm2Overflow:1;
	u32 Tm3Match1:1;
	u32 Tm3Match2:1;
	u32 Tm3Overflow:1;
	u32 Reserved;	
}fLib_TimerMask;


typedef struct 
{
    u32 IntNum;           /* interrupt number */   
    u32 Tick;        /* Tick Per Second */   
    u32 Running;       /* Is timer running */       
}fLib_TimerStruct;


#define APB_CLK APB_CLOCK
//--------------------------------------------------
// Timer tick
//--------------------------------------------------
#define PWMTMR_1000MSEC_PERIOD			(u32)(APB_CLK)
#define PWMTMR_100MSEC_PERIOD			(u32)(APB_CLK/10)
#define PWMTMR_20MSEC_PERIOD			(u32)(APB_CLK/50)
#define PWMTMR_15MSEC_PERIOD			(u32)(((APB_CLK/100)*3)/2)
#define PWMTMR_10MSEC_PERIOD			(u32)(APB_CLK/100)
#define PWMTMR_1MSEC_PERIOD				(u32)(APB_CLK/1000)

typedef enum Timer_IoType
{
	IO_TIMER_RESETALL,
	IO_TIMER_GETTICK,
	IO_TIMER_SETTICK,
	IO_TIMER_SETCLKSRC
}fLib_Timer_IoType;


typedef struct
{
	u32 hour;
	u32 minute;
	u32 second;
}fLib_Time;



/*  -------------------------------------------------------------------------------
 *   API
 *  -------------------------------------------------------------------------------
 */
 
//this routines will export to upper ap or test program
//extern bool fLib_Timer_Init(DRVTIMER timer, u32 tick);

extern bool fLib_Timer_Init(DRVTIMER timer, u32 tick);

extern s32 fLib_Timer_IOCtrl(fLib_Timer_IoType IoType,DRVTIMER timer,u32 tick);
extern s32 fLib_Timer_Counter(DRVTIMER timer);
extern void fLib_Timer_AutoReloadValue(DRVTIMER timer, u32 value);
extern void fLib_Timer_CmpValue(DRVTIMER timer, u32 value);
extern s32 fLib_Timer_Enable(DRVTIMER timer);
extern s32 fLib_Timer_Disable(DRVTIMER timer);
extern s32 fLib_Timer_Close(DRVTIMER timer);
extern s32 fLib_Timer_IntEnable(DRVTIMER timer);
extern s32 fLib_Timer_IntClear(DRVTIMER timer);
extern void Timer_ResetAll(void);
extern u32 fLib_CurrentT1Tick(void);
extern u32 fLib_CurrentT2Tick(void);
extern u32 fLib_CurrentT3Tick(void);
extern u32 fLib_CurrentT4Tick(void);

extern void PWMTMR1_IRQHandler(void);
extern void PWMTMR2_IRQHandler(void);
extern void PWMTMR3_IRQHandler(void);
extern void PWMTMR4_IRQHandler(void);


#endif //__TIMER_H
