/*
 * Kneron Peripheral API
 *
 * Copyright (C) 2019 Kneron, Inc. All rights reserved.
 *
 */

#ifndef __KDP_GPIO_H__
#define __KDP_GPIO_H__

#include <cmsis_os2.h>
#include "kdp_peripheral.h"

typedef enum{
    /* pin direction */
    KDP_GPIO_DIR_INPUT = 0x1,
    KDP_GPIO_DIR_OUTPUT = 0x2,
    /* interrupt setting for input */
    KDP_GPIO_INT_EDGE_RISING = 0x4,
    KDP_GPIO_INT_EDGE_FALLING = 0x8,
    KDP_GPIO_INT_EDGE_BOTH = 0x10,
    KDP_GPIO_INT_LEVEL_HIGH = 0x20,
    KDP_GPIO_INT_LEVEL_LOW = 0x40,
}kdp_gpio_attribute_e;

typedef enum{
    KDP_GPIO_PIN_0 = 0,
    KDP_GPIO_PIN_1,
    KDP_GPIO_PIN_2,
    KDP_GPIO_PIN_3,
    KDP_GPIO_PIN_4,
    KDP_GPIO_PIN_5,
    KDP_GPIO_PIN_6,
    KDP_GPIO_PIN_7,
    KDP_GPIO_PIN_8,
    KDP_GPIO_PIN_9,
    KDP_GPIO_PIN_10,
    KDP_GPIO_PIN_11,
    KDP_GPIO_PIN_12,
    KDP_GPIO_PIN_13,
    KDP_GPIO_PIN_14,
    KDP_GPIO_PIN_15,
    KDP_GPIO_PIN_16,
    KDP_GPIO_PIN_17,
    KDP_GPIO_PIN_18,
    KDP_GPIO_PIN_19,
    KDP_GPIO_PIN_20,
    KDP_GPIO_PIN_21,
    KDP_GPIO_PIN_22,
    KDP_GPIO_PIN_23,
    KDP_GPIO_PIN_24,
    KDP_GPIO_PIN_25,
    KDP_GPIO_PIN_26,
    KDP_GPIO_PIN_27,
    KDP_GPIO_PIN_28,
    KDP_GPIO_PIN_29,
    KDP_GPIO_PIN_30,
    KDP_GPIO_PIN_31
}kdp_gpio_pin_e;

// init gpio controller
kdp_status_e kdp_gpio_init(void);

// de-init gpio controller
kdp_status_e kdp_gpio_deinit(void);

// set pin attributes for one GPIO pin
kdp_status_e kdp_gpio_set_pin_attribute(
    kdp_gpio_pin_e pin,
    uint32_t attributes /* kdp_gpio_attribute_e */);

// register a CMSIS-RTOS2 thread flag to get notified
// when the specified pin is being interrupted.
// in that case, user's thread must wait for a
// specified thread flag as event notifications
kdp_status_e kdp_gpio_set_notify(
    kdp_gpio_pin_e pin,
    osThreadId_t tid,
    uint32_t tflag);

// enable / disable interrupt of one GPIO pin
kdp_status_e kdp_gpio_set_interrupt_enable(
    kdp_gpio_pin_e pin,
    kdp_bool_e isEnable);

// enable / disable debounce with clock setting in Hz
kdp_status_e kdp_gpio_set_debounce_enable(
    kdp_gpio_pin_e pin,
    kdp_bool_e,
    uint32_t debounce_clock /* in Hz */);

// set pin value
kdp_status_e kdp_gpio_set_pin_value(
    kdp_gpio_pin_e pin,
    kdp_bool_e value);

// get pin value
kdp_status_e kdp_gpio_get_pin_value(
    kdp_gpio_pin_e pin,
    kdp_bool_e *pValue);

#endif
