/*
 * KDP Camera driver header
 *
 * Copyright (C) 2019 Kneron, Inc. All rights reserved.
 *
 */

#ifndef __KDP_CAMERA_H__
#define __KDP_CAMERA_H__

enum {
    KDP_CAM_0,
    KDP_CAM_1,
    KDP_CAM_NUM,    // = IMGSRC_NUM
};

enum camera_state {
    CAMERA_STATE_IDLE = 0,
    CAMERA_STATE_INITED,
    CAMERA_STATE_RUNNING,
    CAMERA_STATE_IN_FDR_INFERENCE,
    CAMERA_STATE_IN_FDR_REGISTRATION,
    CAMERA_STATE_IN_FDR_AUTO_REGISTRATION,
    CAMERA_STATE_IN_FDR_REGISTRATION_CONFIRM,
	 CAMERA_STATE_IN_FDR_BOTH_REGISTRATION,
    CAMERA_STATE_IN_FDR_BOTH_REGISTRATION_CONFIRM,
    CAMERA_STATE_IN_FDR_BOTH_INFERENCE,
    CAMERA_STATE_IN_FDR_REGISTRATION_POSE_JUSTIFY,
};

enum display_state {
    DISPLAY_STATE_CLOSED = 0,
    DISPLAY_STATE_OPENED,
    DISPLAY_STATE_RUNNING,
};

struct cam_format {
    unsigned int width;
    unsigned int height;
    unsigned int pixelformat;    /* fourcc */
    unsigned int field;          /* enum v2k_field */
    unsigned int bytesperline;   /* for padding, zero if unused */
    unsigned int sizeimage;
    unsigned int colorspace;     /* enum v2k_colorspace */
};

struct cam_capability {
    char driver[16];
    char desc[16];
    unsigned int version;
    unsigned int capabilities;
};

struct cam_ops {
    int (*open)(int cam_idx);
    void (*close)(int cam_idx);
    int (*set_format)(int cam_idx, struct cam_format *format);
    int (*get_format)(int cam_idx, struct cam_format *format);
    int (*buffer_init)(int cam_idx);		
    int (*start_capture)(int cam_idx);	
    int (*stop_capture)(int cam_idx);
    int (*buffer_prepare)(int cam_idx);
    int (*buffer_capture)(int cam_idx, int *addr, int *size);
    int (*stream_on)(int cam_idx);
    int (*stream_off)(int cam_idx);
    int (*query_capability)(int cam_idx, struct cam_capability *cap);
};

int kdp_camera_init(void);

int kdp_camera_open(int cam_idx);

int kdp_camera_close(int cam_idx);

int kdp_camera_query_capability(int cam_idx, struct cam_capability *cap);
int kdp_camera_get_device_info(int cam_idx, struct cam_capability *cap);

int kdp_camera_set_frame_format(int cam_idx, struct cam_format *format);

int kdp_camera_get_frame_format(int cam_idx, struct cam_format *format);

int kdp_camera_buffer_init(int cam_idx);

int kdp_camera_start(int cam_idx);

int kdp_camera_stop(int cam_idx);

int kdp_camera_buffer_prepare(int cam_idx);

int kdp_camera_buffer_capture(int cam_idx, int *addr, int *size);

int kdp_camera_stream_on(int cam_idx);

int kdp_camera_stream_off(int cam_idx);

int kdp_camera_controller_register(int cam_idx, struct cam_ops *cam_ops_p);

int kdp_camera_controller_unregister(int cam_idx, struct cam_ops *cam_ops_p);

#endif // __KDP_CAMERA_H__
