#ifndef __KDP_INFERENCE_H__
#define __KDP_INFERENCE_H__

#include "config/board_kdp520.h"
#include "base.h"
#include "media/kdp_camera.h"

#define FLAGS_KDP520_OPEN_CAMERA0_EVT               BIT0
#define FLAGS_KDP520_START_CAMERA0_EVT              BIT1
#define FLAGS_KDP520_STOP_CAMERA0_EVT               BIT2
#define FLAGS_KDP520_CLOSE_CAMERA0_EVT              BIT3
#define FLAGS_KDP520_OPEN_CAMERA1_EVT               BIT4
#define FLAGS_KDP520_START_CAMERA1_EVT              BIT5
#define FLAGS_KDP520_STOP_CAMERA1_EVT               BIT6
#define FLAGS_KDP520_CLOSE_CAMERA1_EVT              BIT7
#define FLAGS_KDP520_RESERVED_1                     BIT8
#define FLAGS_KDP520_RESERVED_2                     BIT9
#define FLAGS_KDP520_START_INFERENCE_LOOP_EVT       BIT10
#define FLAGS_KDP520_STOP_INFERENCE_LOOP_EVT        BIT11
#define FLAGS_KDP520_START_INFERENCE_EVT            BIT12
#define FLAGS_KDP520_REGISTER_FDR_EVT               BIT13
#define FLAGS_KDP520_AUTO_REGISTER_FDR_EVT          BIT14
#define FLAGS_KDP520_REGISTER_CFM_FDR_EVT           BIT15
#define FLAGS_KDP520_AUTO_REGISTER_BOTH_FDR_EVT     BIT16
#define FLAGS_KDP520_REGISTER_CFM_BOTH_FDR_EVT      BIT17
#define FLAGS_KDP520_AUTO_REGISTER_5_FACE_EVT       BIT18
#define FLAGS_KDP520_CANCEL_REGISTER_FDR_EVT        BIT19
#define FLAGS_KDP520_ADD_FDR_FACE_EVT               BIT20
#define FLAGS_KDP520_DEL_FDR_FACE_EVT               BIT21
#define FLAGS_KDP520_RESERVED_3                     BIT22
#define FLAGS_KDP520_RESERVED_4                     BIT23


#define FLAGS_KDP520_ALL_EVENTS    ( FLAGS_KDP520_OPEN_CAMERA0_EVT | \
                                    FLAGS_KDP520_START_CAMERA0_EVT | \
                                    FLAGS_KDP520_STOP_CAMERA0_EVT | \
                                    FLAGS_KDP520_CLOSE_CAMERA0_EVT | \
                                    FLAGS_KDP520_OPEN_CAMERA1_EVT | \
                                    FLAGS_KDP520_START_CAMERA1_EVT | \
                                    FLAGS_KDP520_STOP_CAMERA1_EVT | \
                                    FLAGS_KDP520_CLOSE_CAMERA1_EVT | \
                                    FLAGS_KDP520_START_INFERENCE_LOOP_EVT | \
                                    FLAGS_KDP520_STOP_INFERENCE_LOOP_EVT | \
                                    FLAGS_KDP520_START_INFERENCE_EVT | \
                                    FLAGS_KDP520_REGISTER_FDR_EVT | \
                                    FLAGS_KDP520_AUTO_REGISTER_FDR_EVT | \
                                    FLAGS_KDP520_REGISTER_CFM_FDR_EVT | \
                                    FLAGS_KDP520_AUTO_REGISTER_BOTH_FDR_EVT | \
                                    FLAGS_KDP520_REGISTER_CFM_BOTH_FDR_EVT | \
                                    FLAGS_KDP520_AUTO_REGISTER_5_FACE_EVT | \
                                    FLAGS_KDP520_CANCEL_REGISTER_FDR_EVT | \
                                    FLAGS_KDP520_ADD_FDR_FACE_EVT | \
                                    FLAGS_KDP520_DEL_FDR_FACE_EVT)


#define DRAW_HINT_BOUNDINGBOX          (0x01)
#define DRAW_FDR_RESULT_BOUNDINGBOX    (0x02)
#define ALL_DRAW_BOUNDINGBOX_MODE      (DRAW_HINT_BOUNDINGBOX | DRAW_FDR_RESULT_BOUNDINGBOX)

#define DB_SLOT_SIZE                   (20)

typedef struct kdp_camera_ctx {
    bool init;
    enum camera_state state;
    unsigned int width;
    unsigned int height;
    unsigned int pixelformat;
} kdp_camera_ctx;


/*----------------------------------------------------------------------------------
 *    Communication struct parameters between Application <-> Moddleware
 *----------------------------------------------------------------------------------*/
/**
* Structure to represent face register content which be commnuicated between Application <-> Moddleware
*/
struct kdp_fdfr_rgst_context {
    osThreadId_t caller_tid;
    int rgst_type; /**< FLAGS_KDP520_REGISTER_FDR_EVT ~ FLAGS_KDP520_REGISTER_CFM_FDR_EVT */
    int user_idx;
    int rgst_idx;  /**< For auto register usage only */
    int num_rgst;  /**< Number of face register */ 
    int ret;
};

/**
* Structure to represent 5 face register content which be commnuicated between Application <-> Moddleware
*/
struct kdp_fdfr_rgst_5_context {
    osThreadId_t caller_tid;
    int user_idx;
    int pose_type;
    int ret;
};

/**
* Structure to represent face add content which be commnuicated between Application <-> Moddleware
*/
struct kdp_fdfr_add_context {
    osThreadId_t caller_tid;
    int user_idx;
    int ret;
};

/**
* Structure to represent face delete content which be commnuicated between Application <-> Moddleware
*/
struct kdp_fdfr_del_context {
    osThreadId_t caller_tid;
    int user_idx;
    bool all;
    int ret;
};

/**
* Structure to represent minimum requirment content which be commnuicated between Application <-> Moddleware
*/
struct kdp_fdfr_context {
    osThreadId_t caller_tid;
    int ret;
};

/**
* typedef structure to represent camera commnucation parameter which be commnuicated between Application <-> Moddleware
*/
typedef struct kdp_camera_comm_params {
    osThreadId_t caller_tid;
    int cam_idx;
    int ret;
} kdp_camera_comm_params;

/**
* typedef structure to represent fdfr commnucation parameter which be commnuicated between Application <-> Moddleware
*/
typedef struct kdp_fdfr_comm_params {
    union {
        struct kdp_fdfr_context         fdfr_ctx;
        struct kdp_fdfr_rgst_context    rgst_ctx;
        struct kdp_fdfr_rgst_5_context  rgst_5_ctx;
        struct kdp_fdfr_context         rgst_cancel_ctx;
        struct kdp_fdfr_add_context     add_ctx;
        struct kdp_fdfr_del_context     del_ctx;
    };
} kdp_fdfr_comm_params;

/**
* typedef structure to represent all commnucation parameter which be commnuicated between Application <-> Moddleware
*/
typedef struct kdp_all_comm_params {
    union {
        kdp_camera_comm_params cam_params;
        kdp_fdfr_comm_params   fdfr_params;
    };
} kdp_all_comm_params;


extern kdp_camera_ctx m_kdp_cam_ctx[IMGSRC_NUM + 2];
extern kdp_all_comm_params m_kdp_all_comm_params;
extern osThreadId_t tid_inference;
extern int g_frame_margin_length;
extern int reg_type;
extern int reg_count;
extern int m_face_add_timeout;
extern int m_face_recognition_timeout;
extern u32 m_fdfr_tick;

/*-----------------------------------------------------------------------
 *                          Puclic FDFR API 
 *-----------------------------------------------------------------------*/
bool kdp_fdfr_model_init(void);
 
int kdp_fdfr_inference_thread_create(void);

void kdp_fdfr_set_boundingbox_mode(int mode);

void kdp_fdfr_inference_start(void);

void kdp_fdfr_inference_stop(void);

bool kdp_fdfr_inference_is_running(void);

void kdp_fdfr_set_hint_boundingbox(void);

int kdp_fdfr_get_available_db_idx(void);

int kdp_fdfr_get_remaining_fmap_cnt(int user_idx);

void kdp_fdfr_recognition_variable_reset(void);

/*-----------------------------------------------------------------------
 *                          Puclic camera API 
 *-----------------------------------------------------------------------*/
int kdp_camera_setting(uint8_t cam_idx, unsigned int width, unsigned int height, unsigned int pixelformat);

int kdp_get_camera_ctx_state(uint8_t cam_idx);

osThreadId_t kdp_get_inference_thread_id(void);


/*-----------------------------------------------------------------------
 *                         extern  API 
 *-----------------------------------------------------------------------*/
extern void nir_set_exp_time(u8 ana_gn1, u8 ana_gn2);
extern int32_t kdp_app_lw3d_fdr(int distance_flag);

#endif

