#ifdef V2K_ENABLE
#include "config/board_kdp520.h"
#include <cmsis_os2.h>
#include "kneron_mozart.h"
#include "clock.h"
#include "framework/framework_driver.h"
#include "framework/v2k_image.h"
#include "media/display/lcdc.h"
#include "media/display/kdp_display.h"
#include "media/kdp_inference.h"
#include "dbg.h"

struct display_driver *m_display_driver = NULL;
#if DISPLAY_DEVICE == DISPLAY_DEVICE_LCDC
extern struct display_driver kdp520_lcdc_driver;
extern struct panel_driver mzt_480x272_driver;
#elif DISPLAY_DEVICE == DISPLAY_DEVICE_LCM
extern struct display_driver kdp520_lcm_driver;
extern struct panel_driver st7789_240x320_driver;
#endif

int kdp_display_pip_test(void)
{
    if (m_display_driver)
        m_display_driver->pip_test(m_display_driver->core_dev);

    return 0;
}

int kdp_display_set_pen_rgb565(unsigned short color, unsigned int pen_width)
{
    if (m_display_driver)
        m_display_driver->set_pen(m_display_driver->core_dev, color, pen_width);
        //m_display_driver->set_pen(m_display_driver->core_dev, r, g, b, pen_width);
    
    return 0;
}

int kdp_display_set_pen_rgb24(unsigned char r, unsigned char g, unsigned char b, unsigned int pen_width)
{
    return 0;

}

int kdp_display_draw_rect(unsigned int fb_addr, int x, int y, int width, int height, int draw_mode)
{
    if (m_display_driver)
    {
        if(draw_mode == DRAW_HINT_BOUNDINGBOX)
            m_display_driver->draw_static_rect(m_display_driver->core_dev, fb_addr, x, y, width, height);
        else if(draw_mode == DRAW_FDR_RESULT_BOUNDINGBOX)
            m_display_driver->draw_rect(m_display_driver->core_dev, fb_addr, x, y, width, height);
    }
    return 0;
}

int kdp_display_draw_line(u32 xs, u32 ys, u32 xe, u32 ye)
{
    if (m_display_driver)
    {
        m_display_driver->draw_line(m_display_driver->core_dev, xs, ys, xe, ye);
    }
    
    return 0;
}

int kdp_display_fill_rect(u32 org_x, u32 org_y, u32 width, u32 height)
{
    if (m_display_driver)
    {
        m_display_driver->fill_rect(m_display_driver->core_dev, org_x, org_y, width, height);
    }
    
    return 0;
}

int kdp_display_draw_bitmap(u32 org_x, u32 org_y, u32 width, u32 height, void *buf)
{
    if (m_display_driver)
    {
        m_display_driver->draw_bitmap(m_display_driver->core_dev, org_x, org_y, width, height, buf);
    }
    
    return 0;
}

int kdp_display_get_device_id(void)
{
    if (m_display_driver)
        return m_display_driver->display_id;
    else
        return -1;
}

static void _board_assign_display_device()
{
#if DISPLAY_DEVICE == DISPLAY_DEVICE_LCDC
    m_display_driver  = &kdp520_lcdc_driver;
#elif DISPLAY_DEVICE == DISPLAY_DEVICE_LCM
    m_display_driver  = &kdp520_lcm_driver;
#endif
}

static void _board_connect_display_panel()
{
    if (m_display_driver) {
    #if PANEL_TYPE == PANEL_MZT_480X272
        m_display_driver->attach_panel(m_display_driver->core_dev, &mzt_480x272_driver);
    #elif PANEL_TYPE == PANEL_ST7789_240X320
        m_display_driver->attach_panel(m_display_driver->core_dev, &st7789_240x320_driver);
    #endif
    }
}

int kdp_video_renderer_open(struct video_input_params *params)
{
    int ret = -1;

    _board_assign_display_device();
    _board_connect_display_panel();

    if (m_display_driver) {
        if ((0 == m_display_driver->set_params(m_display_driver->core_dev, params)) &&
            (0 == m_display_driver->init(m_display_driver->core_dev))) {
            ret = 0;
        }
    }

    return ret;
}

int kdp_video_renderer_set_camera(unsigned int cam_idx)
{
    return m_display_driver->set_camera(m_display_driver->core_dev, cam_idx);
}

int kdp_video_renderer_start(void)
{
//    DSG("m_display_driver=%x", m_display_driver);
//    DSG("m_display_driver->start=%x", m_display_driver->start);
//    DSG("m_display_driver->core_dev=%x", m_display_driver->core_dev);
    if (m_display_driver)
        m_display_driver->start(m_display_driver->core_dev);

    return 0;
}

int kdp_video_renderer_stop(void)
{
    if (m_display_driver)
        m_display_driver->stop(m_display_driver->core_dev);

    return 0;
}

#endif
