#ifdef V2K_ENABLE
#include "config/board_kdp520.h"
#if PANEL_TYPE == PANEL_MZT_480X272
#include "base.h"
#include "framework/framework_driver.h"
#include "framework/v2k_image.h"
#include "media/display/lcdc.h"
#include "dbg.h"

#define MZT_PANEL_WIDTH     TFT43_WIDTH
#define MZT_PANEL_HEIGHT    TFT43_HEIGHT
/* if enable DOWN_SCALE_WITH_CROP, 
   need to modify result gui boundary due to changed display boundary*/
//#define DOWN_SCALE_WITH_CROP

extern struct display_driver kdp520_lcdc_driver;

int mzt_480x272_init(struct core_device *core_d)
{
    int ret = 0;
    struct display_driver *display_drv = &kdp520_lcdc_driver;

    u16 hor_no_in = display_drv->vi_params.input_xres;
    u16 ver_no_in = display_drv->vi_params.input_yres;
    u16 hor_no_out = MZT_PANEL_WIDTH;
    u16 ver_no_out = MZT_PANEL_HEIGHT;

#ifdef DOWN_SCALE_WITH_CROP
    //pick 640x360 from 640x480
    ver_no_in = (u16)((float)hor_no_in * (float)ver_no_out / (float)hor_no_out );
#endif

    dbg_msg("[%s] hor_no_in=%u ver_no_in=%u hor_no_out=%u ver_no_out=%u",
    __func__, hor_no_in, ver_no_in, hor_no_out, ver_no_out);
    LCDC_REG_FUNC_ENABLE_SET_LCDon(0);
    LCDC_REG_FUNC_ENABLE_SET_LCDen(0);

    //for DOWN_SCALING
    if ((hor_no_in != hor_no_out) || (hor_no_out != ver_no_out))
    {
        LCDC_REG_FUNC_ENABLE_SET_TVEn(0);
        LCDC_REG_FUNC_ENABLE_SET_ScalerEn(1);    
    }

    //LCDC_REG_FUNC_ENABLE_SET_ScalerEn(1);
    LCDC_REG_PANEL_PIXEL_SET_UpdateSrc(0); //image0

    kdp520_lcdc_set_panel_type(lcdc_6bits_per_channel);
    kdp520_lcdc_set_bgrsw(lcdc_output_fmt_sel_rgb); //RGB
    //LCDC_REG_PATGEN_SET_Img0PatGen(0);

    //Pixel Clock = H total * V total * Frame Rate
    //horizontal timing control
    LCDC_REG_HORIZONTAL_TIMING_CTRL_SET_HBP(1);
    LCDC_REG_HORIZONTAL_TIMING_CTRL_SET_HFP(1);
    LCDC_REG_HORIZONTAL_TIMING_CTRL_SET_HW(40);  
    LCDC_REG_HORIZONTAL_TIMING_CTRL_SET_PL(((hor_no_out >> 4) - 1));
    //LCDC_REG_HORIZONTAL_TIMING_CTRL_SET_PL(((MZT_PANEL_WIDTH >> 4) - 1));      
    
    //vertical timing control
    LCDC_REG_VERTICAL_TIMING_CTRL_SET_VFP(2);
    LCDC_REG_VERTICAL_TIMING_CTRL_SET_VW(9);
    LCDC_REG_VERTICAL_TIMING_CTRL_SET_LF((ver_no_out - 1));
    //LCDC_REG_VERTICAL_TIMING_CTRL_SET_LF((MZT_PANEL_HEIGHT - 1));    

    //vertical back porch
    LCDC_REG_VERTICAL_BACK_PORCH_SET_VBP(2);

    LCDC_REG_POLARITY_CTRL_SET_DivNo(5);
    LCDC_REG_POLARITY_CTRL_SET_IDE(1);
    LCDC_REG_POLARITY_CTRL_SET_ICK(1);
    LCDC_REG_POLARITY_CTRL_SET_IHS(1);
    LCDC_REG_POLARITY_CTRL_SET_IVS(1);

    //serial panel pixel
    LCDC_REG_SERIAL_PANEL_PIXEL_SET_AUO052(0);
    LCDC_REG_SERIAL_PANEL_PIXEL_SET_LSR(0);
    LCDC_REG_SERIAL_PANEL_PIXEL_SET_ColorSeq(2);
    LCDC_REG_SERIAL_PANEL_PIXEL_SET_DeltaType(0);
    LCDC_REG_SERIAL_PANEL_PIXEL_SET_SerialMode(0);

    //TV control
    LCDC_REG_TV_SET_ImgFormat(0);
    LCDC_REG_TV_SET_PHASE(0);
    LCDC_REG_TV_SET_OutFormat(0);

    kdp520_lcdc_set_framerate(60, hor_no_out, ver_no_out);

    //LCD Image Color Management
    outw(LCDC_REG_COLOR_MGR_0, 0x2000);
    outw(LCDC_REG_COLOR_MGR_1, 0x2000);
    outw(LCDC_REG_COLOR_MGR_2, 0x0);
    outw(LCDC_REG_COLOR_MGR_3, 0x40000);
    //LCDC_REG_COLOR_MGR_3_SET_Contr_slope(4);

    //FrameBuffer Parameter
    outw(LCDC_REG_FRAME_BUFFER, 0x0);

    //Ignore some transformation
    outw(LCDC_REG_BANDWIDTH_CTRL, 0x0);

    //for DOWN_SCALING
    if ((hor_no_in != hor_no_out) || (ver_no_in != ver_no_out)) {
        LCDC_REG_SCALER_HOR_RES_IN_SET_hor_no_in((hor_no_in - 1)); 
        LCDC_REG_SCALER_VER_RES_IN_SET_ver_no_in((ver_no_in - 1));
        LCDC_REG_SCALER_HOR_RES_OUT_SET_hor_no_out(hor_no_out);
        LCDC_REG_SCALER_VER_RES_OUT_SET_ver_no_out(ver_no_out);
        LCDC_REG_SCALER_MISC_SET_fir_sel(0);
        LCDC_REG_SCALER_MISC_SET_hor_inter_mode(0);
        LCDC_REG_SCALER_MISC_SET_ver_inter_mode(0);
        LCDC_REG_SCALER_MISC_SET_bypass_mode(0);
        //u32 scal_hor_num = mod((hor_no_in + 1) / hor_no_out) * 256 / hor_no_out;
        //u32 scal_ver_num = mod((ver_no_in + 1) / ver_no_out) * 256 / ver_no_out;
        LCDC_REG_SCALER_RES_SET_scal_hor_num(85);
#ifdef DOWN_SCALE_WITH_CROP
        //keep ratio but with cropping
        LCDC_REG_SCALER_RES_SET_scal_ver_num(85);
#else
        //all are display but ratio is changed
        LCDC_REG_SCALER_RES_SET_scal_ver_num(195);
#endif
    }
    else {
        //reset the scalar control register	
        outw(LCDC_REG_SCALER_MISC, 0x0);
    }

    return ret;
}

int mzt_480x272_clear(struct core_device *core_d, u32 color)
{
    return 0;
}

u16 mzt_480x272_read_display_id(struct core_device *core_d)
{
    return 0;
}

struct panel_driver mzt_480x272_driver = {
    .init       = mzt_480x272_init,
    .clear      = mzt_480x272_clear,
    .read_did   = mzt_480x272_read_display_id,
};

#endif
#endif
