#ifdef V2K_ENABLE
#include "config/board_kdp520.h"
#if PANEL_TYPE == PANEL_ST7789_240X320
#include "base.h"
#include "io.h"
#include "framework/framework_driver.h"
#include "framework/v2k_image.h"
#include "media/display/display.h"
#include "kdp520_pwm_timer.h"
#include "kdp520_dma.h"
#include "kdp520_gpio.h"
#include "dbg.h"
//#include "memory.h"
#include "delay.h"


#define TE_ENABLE
#define PORTRAIT_DMA_ENABLE

#define FRAME_RATE_60HZ 0
#define FRAME_RATE_39HZ 1
#define FRAME_RATE_CTRL FRAME_RATE_60HZ

#define RAM_ENDIAN_MSB  0
#define RAM_ENDIAN_LSB  1
#define RAM_ENDIAN_TYPE RAM_ENDIAN_MSB


/* 
This command is used to transfer data from MCU to frame memory.
-When this command is accepted, the column register and the page register are reset to the start column/start
page positions.
-The start column/start page positions are different in accordance with MADCTL setting.
-Sending any other command can stop frame write.
*/
#define PANEL_REG_RAMWR             0x2C //Memory write
#define PANEL_REG_WRMEMC            0x3C //Write memory continue

//#define ST7789_LANDSCAPE
#define ST7789_PORTRAIT

#ifdef ST7789_PORTRAIT
	//#define	ST7789_PORTRAIT_PARTIAL
	//#define ZOOM_IN_RATIO				3/2
#endif

#ifdef ST7789_LANDSCAPE
#define ST7789_PANEL_WIDTH          QVGA_LANDSCAPE_WIDTH
#define ST7789_PANEL_HEIGHT         QVGA_LANDSCAPE_HEIGHT
#else
#define ST7789_PANEL_WIDTH          QVGA_PORTRAIT_WIDTH
#define ST7789_PANEL_HEIGHT         QVGA_PORTRAIT_HEIGHT
#endif
#define ST7789_PANEL_TOTAL_PIXEL    ST7789_PANEL_WIDTH * ST7789_PANEL_HEIGHT
#define OPEN_DOWN_SCALING
#define BYTES_PER_PIXEL				2

#ifdef PORTRAIT_DMA_ENABLE
#undef TE_ENABLE
#undef FRAME_RATE_CTRL
#undef RAM_ENDIAN_TYPE

#define FRAME_RATE_CTRL FRAME_RATE_39HZ
#define RAM_ENDIAN_TYPE RAM_ENDIAN_LSB
#define TE_ENABLE

#endif


volatile dma_lld LinkAddr0[320];
static int _st7789_240x320_clear(struct core_device *core_d, u32 color);

int _st7789_240x320_init(struct core_device *core_d)
{
    int ret;

    struct display_driver *display_drv = (struct display_driver *)framework_drv_get_drv(&core_d->pin_ctx);
    struct display_driver_ops *ops = (struct display_driver_ops*)display_drv->ctrller_ops;    

    u32 base = display_drv->base;
    u16 hor_no_in = display_drv->vi_params.input_xres;
    u16 ver_no_in = display_drv->vi_params.input_yres;
    u16 hor_no_out = ST7789_PANEL_WIDTH;//MZT_PANEL_WIDTH;
    u16 ver_no_out = ST7789_PANEL_HEIGHT;//MZT_PANEL_HEIGHT;
    DSG("[%s] hor_no_in=%u ver_no_in=%u hor_no_out=%u ver_no_out=%u",
    __func__, hor_no_in, ver_no_in, hor_no_out, ver_no_out);
 
    if (!ops)
        return -1;

    ops->write_cmd(base, 0x11); //Exit Sleep
    delay_ms_enable();
    delay_ms(200);
    delay_ms_disable();

    //-----------Display and color format setting-------------//
    // Column Address Set
    ops->write_cmd(base, 0x2A); 
    // XS
    ops->write_data(base, 0x00);
    ops->write_data(base, 0x00);
    // XE
#ifdef ST7789_LANDSCAPE
    ops->write_data(base, 0x01);
    ops->write_data(base, 0x3F);
#else
    ops->write_data(base, 0x00);
    ops->write_data(base, 0xEF);
#endif

    // Row Address Set
    ops->write_cmd(base, 0x2B);
#ifdef ST7789_LANDSCAPE
    // YS
    ops->write_data(base, 0x00);
    ops->write_data(base, 0x00);
    // YE
    ops->write_data(base, 0x00);
    ops->write_data(base, 0xEF);
#else
	#ifdef ST7789_PORTRAIT_PARTIAL
	  // YS
    ops->write_data(base, 0x00);
    ops->write_data(base, 0x40);
    // YE
    ops->write_data(base, 0x01);
    ops->write_data(base, 0x00);
	#else
	  // YS
    ops->write_data(base, 0x00);
    ops->write_data(base, 0x00);
    // YE
    ops->write_data(base, 0x01);
    ops->write_data(base, 0x3F);
	#endif
#endif
    
#ifdef ST7789_PORTRAIT_PARTIAL
    ops->write_cmd(base, 0x13);
    ops->write_data(base, 0x00);
    ops->write_cmd(base, 0x12);
    ops->write_data(base, 0xFF);
    ops->write_cmd(base, 0x30);
    ops->write_data(base, 0x00);
    ops->write_data(base, 0x40);
    ops->write_data(base, 0x01);
    ops->write_data(base, 0x00);
#endif

#ifdef TE_ENABLE
    ops->write_cmd(base, 0x35); // Enable Tearing effect
    ops->write_data(base, 0x00);
#endif

    ops->write_cmd(base, 0x36); // Memory Access Control
#ifdef ST7789_LANDSCAPE
    // xy_exchange / y_inverse
    //(Rotate positive 90 degrees)
    //MY = 1, MX = 0, MV = 1, ML = 0
    ops->write_data(base, 0xA0); 
#else
    //MY = 1, MX = 1, MV = 0, ML = 0 
    ops->write_data(base, 0x40); 
#endif

    // Interface Pixel Format
    ops->write_cmd(base, 0x3a);//DPI=101  16bit RGB
    ops->write_data(base, 0x55);
    //ops->write_data(base, 0x05);
    //-------------ST7789V Porch setting-----------//
    ops->write_cmd(base, 0xb2);
    ops->write_data(base, 0x0C); // Back porch (normal)
    ops->write_data(base, 0x0C); // Front porch (normal)
    ops->write_data(base, 0x00); // 
    ops->write_data(base, 0X33); // Back porch (idle) + Front porch (idle)
    ops->write_data(base, 0X33); // Back porch (partial) + Front porch (partial)
    
    // Gate Control. 13.26, -10.43
    ops->write_cmd(base, 0xb7);
    ops->write_data(base, 0x35);//(0x71);
    
    //Setting limitation: VCOMS+VCOMS offset+VDV=0.1V~1.675V.
    // VCOMS Setting. used for feed through voltage compensation
    // 1.175V
    ops->write_cmd(base, 0xBB);
    ops->write_data(base, 0x2b);//(0x25);
    
    // LCM Control ???
    ops->write_cmd(base, 0xC0); 
    ops->write_data(base, 0x2c); 
    
    // VDV and VRH command enable
    ops->write_cmd(base, 0xC2); 
    ops->write_data(base, 0x01);
    
    // VRH Set
    ops->write_cmd(base, 0xC3); 
    ops->write_data(base, 0x11);//(0X14);//
    
    // VDVS Set
    ops->write_cmd(base, 0xC4); 
    ops->write_data(base, 0x20); // 0
    
    // Frame rate control in normal mode
#if FRAME_RATE_CTRL == FRAME_RATE_60HZ
    ops->write_cmd(base, 0xC6); 
    ops->write_data(base, 0x0F);
#elif FRAME_RATE_CTRL == FRAME_RATE_39HZ
    ops->write_cmd(base, 0xC6); 
    ops->write_data(base, 0x1F);
#endif

    // Power Control 1
    ops->write_cmd(base, 0xd0); 
    ops->write_data(base, 0xa4);
    ops->write_data(base, 0xa1); // AVDD : 6.8V, AVCL : -4.6V, VDDS : 2.3 V

    //---------------ST7789V gamma setting-------------//
    { // Positive Voltage gamma control
        ops->write_cmd(base, 0xE0); 
        ops->write_data(base, 0xd0);
        ops->write_data(base, 0x08);
        ops->write_data(base, 0x0e);
        ops->write_data(base, 0x0a);
        ops->write_data(base, 0x0a);
        ops->write_data(base, 0x06);
        ops->write_data(base, 0x38);
        ops->write_data(base, 0x44);
        ops->write_data(base, 0x50);
        ops->write_data(base, 0x29);
        ops->write_data(base, 0x15);
        ops->write_data(base, 0x16);
        ops->write_data(base, 0x33);
        ops->write_data(base, 0x36);
    }
    { // Negative Boltage gamma control
        ops->write_cmd(base, 0XE1); 
        ops->write_data(base, 0xd0);
        ops->write_data(base, 0x07);
        ops->write_data(base, 0x0d);
        ops->write_data(base, 0x09);
        ops->write_data(base, 0x08);
        ops->write_data(base, 0x06);
        ops->write_data(base, 0x33);
        ops->write_data(base, 0x33);
        ops->write_data(base, 0x4d);
        ops->write_data(base, 0x28); 
        ops->write_data(base, 0x16);
        ops->write_data(base, 0x15);
        ops->write_data(base, 0x33);
        ops->write_data(base, 0x35);   
    }

    ops->write_cmd(base, 0x21); // Display Inversion On ??
    
    // RAM Control
#if RAM_ENDIAN_TYPE == RAM_ENDIAN_MSB
    ops->write_cmd(base, 0xB0);
    ops->write_data(base, 0x00); // Ram access from MCU interface
    ops->write_data(base, 0xd0); // Normal Endian (MSB first), 18 bit bus width
#elif RAM_ENDIAN_TYPE == RAM_ENDIAN_LSB
    ops->write_cmd(base, 0xB0);
    ops->write_data(base, 0x00); // Ram access from MCU interface
    ops->write_data(base, 0xd8); // Normal Endian (MSB first), 18 bit bus width    
#endif
    
    ops->write_cmd(base, 0x29); //Display on
    //ops->write_cmd(base, 0x28); //Display off

    //_st7789_240x320_clear(core_d, YELLOW);//WHITE);

    ret = 0;

    return ret;
}

static int _st7789_240x320_clear(struct core_device *core_d, u32 color)
{
    int ret = 0;
    int i;

    struct display_driver *display_drv = (struct display_driver *)framework_drv_get_drv(&core_d->pin_ctx);    
    struct display_driver_ops *ops = (struct display_driver_ops*)display_drv->ctrller_ops;
    
    if (ops) {
        ops->write_cmd(display_drv->base, PANEL_REG_WRMEMC);

        for(i = 0; i < ST7789_PANEL_TOTAL_PIXEL; ++i)
        {
            ops->write_data(display_drv->base, (u8)((((color & 0xFF00) >>8 ) & 0xFF)));
            ops->write_data(display_drv->base, (u8)(color & 0x00FF));
        }
    }

    return ret;
}

u16 _st7789_240x320_read_display_id(struct core_device *core_d)
{
    u16 id = 0;
    struct display_driver *display_drv = (struct display_driver *)framework_drv_get_drv(&core_d->pin_ctx); 
    struct display_driver_ops *ops = (struct display_driver_ops*)display_drv->ctrller_ops;
    if (ops) {
        u32 base = display_drv->base;
        
        ops->write_cmd(base, 0x04);
        id = ops->read_data(base);  //dummy read 
        id = ops->read_data(base); 
        id = ((ops->read_data(base) & 0xBF) << 8);
        id |= ops->read_data(base);
    }
    
    return id;
}

void _st7789_240x320_start(struct core_device *core_d)
{
    struct display_driver *display_drv = (struct display_driver *)framework_drv_get_drv(&core_d->pin_ctx); 
    struct display_driver_ops *ops = (struct display_driver_ops*)display_drv->ctrller_ops;
    if (ops) {
        ops->write_cmd(display_drv->base, 0x29);
    }
}

void _st7789_240x320_stop(struct core_device *core_d)
{
    struct display_driver *display_drv = (struct display_driver *)framework_drv_get_drv(&core_d->pin_ctx); 
    struct display_driver_ops *ops = (struct display_driver_ops*)display_drv->ctrller_ops;
    if (ops) {
        ops->write_cmd(display_drv->base, 0x28);
    }
}

void _st7789_240x320_update_image(struct core_device *core_d, u32 addr, u16 color, u16 pen_width, struct kdp_rect *rect)
{
    struct fb_platform_data *pdata = (struct fb_platform_data *)(core_d->pin_ctx.platform_data);    
    struct display_driver *display_drv = (struct display_driver *)framework_drv_get_drv(&core_d->pin_ctx);     
    struct display_driver_ops *ops = (struct display_driver_ops*)display_drv->ctrller_ops;
    u32 base = display_drv->base;    
    u32 row_idx = 0;
    u32 col_idx = 0;
	u32 index = 0;
	u32 data_buf;
    u32 readAddr = addr; 
    u32 totalpoint;
    u16 clr;
    u16 pen_w;
    u16 s_x;
    u16 s_y;
    u16 e_x;
    u16 e_y;
    u16 s_x_e;
    u16 s_y_e;
    u16 e_x_e;
    u16 e_y_e;
    const u16 input_xres = display_drv->vi_params.input_xres;
    const u16 input_yres = display_drv->vi_params.input_yres;
    const u32 input_fmt  = display_drv->vi_params.input_fmt;
    const BOOL is_raw8 = (V2K_PIX_FMT_RAW8 == input_fmt) ? (TRUE) : (FALSE);

    totalpoint = ((is_raw8) ? (input_xres * input_yres) : ((input_xres * input_yres) << 1));


#ifdef PORTRAIT_DMA_ENABLE
		readAddr = KDP_DDR_BASE_IMAGE_FOR_LCM;//0x6341CB00
		memcpy(readAddr,addr,600*800*2);
		addr = KDP_DDR_BASE_IMAGE_FOR_LCM;
#endif



    if (rect) {
        clr = color;
        pen_w = pen_width;
        s_x = rect->start_x;
        s_y = rect->start_y;
        e_x = rect->end_x;
        e_y = rect->end_y;
        s_x_e = s_x + pen_w;
        s_y_e = s_y + pen_w;
        e_x_e = e_x + pen_w;
        e_y_e = e_y + pen_w;
    }

//DSG(" %u %u %u", input_xres, input_yres, is_raw8);
    //DSG("www %u %u %u %u %u %u %u %u", s_x, s_y, e_x, e_y, s_x_e, s_y_e, e_x_e, e_y_e);

    //DSG("input_fmt =%x", input_fmt);
    // rect.start_x = 80;
    // rect.start_y = 60;
    // rect.end_x = 240;
    // rect.end_y = 180;
    if (!ops) return;

#ifndef ST7789_PORTRAIT_PARTIAL
    //home point : (0, 0)
    ops->write_cmd(base, 0x2A);
    ops->write_data(base, 0x00);
    ops->write_data(base, 0x00);
    ops->write_cmd(base, 0x2B);
    ops->write_data(base, 0x00);
    ops->write_data(base, 0x00);
#endif
		
    ops->write_cmd(base, PANEL_REG_RAMWR);

#ifdef ST7789_LANDSCAPE
    //640 downscale to 320
    for(index = 0; index < totalpoint;) {
        
        if ( (rect) && (((((row_idx > s_y) && (row_idx < s_y_e)) || ((row_idx > e_y) && (row_idx < e_y_e))) && ((col_idx > s_x) && (col_idx < e_x_e))) ||
           ((((col_idx > s_x) && (col_idx < s_x_e)) || ((col_idx > e_x) && (col_idx < e_x_e))) && ((row_idx > s_y) && (row_idx < e_y_e)))))
            data_buf = clr;
        else
            data_buf = inw(readAddr);

        ops->write_data(base, (u8)(((data_buf & 0x0000FF00) >> 8) & 0xFF));
        ops->write_data(base, (u8)(data_buf & 0x000000FF));

        if (0 == ((index + 4) % (input_xres << 1)))
        {
            index += (input_xres << 1);
            readAddr += (input_xres << 1);
            row_idx += 2;
            col_idx = 0;
        }
        else
            col_idx += 2;
        
        index += 4;
        readAddr += 4;
    }
#elif defined(ST7789_PORTRAIT)

    //readAddr += ((640*80+200)/2);

    if (is_raw8) {
        for(index = 0; index < totalpoint;) {
            if ( (rect) && (((((row_idx > s_y) && (row_idx < s_y_e)) || ((row_idx > e_y) && (row_idx < e_y_e))) && ((col_idx > s_x) && (col_idx < e_x_e))) ||
            ((((col_idx > s_x) && (col_idx < s_x_e)) || ((col_idx > e_x) && (col_idx < e_x_e))) && ((row_idx > s_y) && (row_idx < e_y_e)))))
					data_buf = clr;
			else
				data_buf = (u32)inb(readAddr);
			
			data_buf = ((data_buf>>3)<<11) | ((data_buf>>2)<<5) | ((data_buf>>3)<<0);
			
			ops->write_data(base, (u8)(((data_buf & 0x0000FF00) >> 8) & 0xFF));
			ops->write_data(base, (u8)(data_buf & 0xFF));
			
			if (0 == ((index + 2) % (input_xres)))
			{
				index += (input_xres);
				readAddr += (input_xres);
				row_idx += 2;
				col_idx = 0;
			}
			else
				col_idx += 2;
			
			index += 2;
			readAddr += 2;
        }
    }
    else
    {
		u16 lcm_w = ST7789_PANEL_WIDTH;
#ifdef ST7789_PORTRAIT_PARTIAL
		u16 lcm_h = 192;//ST7789_PANEL_HEIGHT;
#else
		u16 lcm_h = ST7789_PANEL_HEIGHT;
#endif
		u16 cam_w = input_xres;
		u16 cam_h = input_yres;
		u8 bpp = BYTES_PER_PIXEL;
#ifdef ZOOM_IN_RATIO
		u16 step = (2*bpp)*ZOOM_IN_RATIO*4/4;
#else
    	u16 step = (2*bpp);
    	readAddr += ((cam_w*(cam_h-lcm_h)/2+(cam_w-lcm_w)/2)*bpp);//(200, 80)
#endif
        col_idx = (cam_w-lcm_w)/2;
        row_idx = (cam_h-lcm_h)/2;
        
#ifdef PORTRAIT_DMA_ENABLE
			  
        //kdp520_gpio_setedgemode( 1 << GPIO_16, 0);
				#if 1
				if(rect)
				{
					for(index = 0; index < lcm_w*lcm_h/bpp; index++)
					{
						if ( (rect) && (((((row_idx > s_y) && (row_idx < s_y_e)) || ((row_idx > e_y) && (row_idx < e_y_e))) && ((col_idx > s_x) && (col_idx < e_x_e))) ||
						((((col_idx > s_x) && (col_idx < s_x_e)) || ((col_idx > e_x) && (col_idx < e_x_e))) && ((row_idx > s_y) && (row_idx < e_y_e)))))                
						{
							outw(readAddr, ((clr<<16) | clr));
						}

						if (0 == ((index + 1) % (lcm_w/bpp)))
						{
							#ifdef ZOOM_IN_RATIO
								readAddr += ((cam_w-lcm_w*ZOOM_IN_RATIO)*bpp+step)*4/4;
							if ( 0==(index+1)/(lcm_w/bpp)%2 )
							{
								readAddr += cam_w*bpp*4/4;
							}
							#else
								readAddr += ((cam_w-lcm_w)*bpp+2*bpp);
							#endif
							row_idx += 1;
							col_idx = (cam_w-lcm_w)/2;
						}
						else
						{
							readAddr += step;
							col_idx += 2;
						}
					}
				}

				#endif
        //kdp520_gpio_enableint( GPIO_16, GPIO_EDGE, TRUE);
				//kdp520_wait_gpio_int(1<<GPIO_16);    
        //kdp520_gpio_disableint(GPIO_16);
        
				kdp_dma_reset_ch(AHBDMA_Channel1);
				kdp_dma_clear_all_interrupt();
				kdp_dma_enable_dma_int();
				kdp_dma_init(0,0,0);

				readAddr = ((cam_w*(cam_h-lcm_h)/2+(cam_w-lcm_w)/2)*bpp)+addr;//(200, 80)
		
	//LinkAddr0 = malloc(sizeof(dma_lld) * 320);
#if 1
        for(index =0;index < lcm_h;index++)
        {

          kdp_dma_normal_mode(AHBDMA_Channel1,(UINT32)(readAddr),(UINT32)0xC340020c, lcm_w*2 ,0,0,1,0,2,0,0,0); //init DMA and wait TE signal
					//kdp_ddr_to_slcd_dma_copy((UINT32*)readAddr,(UINT32*)0xC340020c, lcm_w*2);
						
					kdp_dma_enable_ch(AHBDMA_Channel1);
					kdp_dma_wait_dma_int(AHBDMA_Channel1);					
					readAddr += (lcm_h << 2 );
        }
#else				
				kdp_dma_link_transfer(AHBDMA_Channel1,(UINT32)LinkAddr0,320,(UINT32)(readAddr),(UINT32)0xC340020C, lcm_w*lcm_h*2 ,0,0,1,0,2,0,0,0);
				kdp_dma_enable_ch(AHBDMA_Channel1);
				kdp_dma_wait_dma_int(AHBDMA_Channel1);	
				kdp_dma_disable_ch(AHBDMA_Channel1);
#endif
					

#else        
        for(index = 0; index < lcm_w*lcm_h/bpp; index++)
        {
            if ( (rect) && (((((row_idx > s_y) && (row_idx < s_y_e)) || ((row_idx > e_y) && (row_idx < e_y_e))) && ((col_idx > s_x) && (col_idx < e_x_e))) ||
            ((((col_idx > s_x) && (col_idx < s_x_e)) || ((col_idx > e_x) && (col_idx < e_x_e))) && ((row_idx > s_y) && (row_idx < e_y_e)))))
                data_buf = clr;
            else
                data_buf = inw(readAddr);

            ops->write_data(base, (u8)(((data_buf & 0x0000FF00) >> 8) & 0xFF));
            ops->write_data(base, (u8)(data_buf & 0xFF));
            ops->write_data(base, (u8)(((data_buf & 0xFF000000) >> 24) & 0xFF));
            ops->write_data(base, (u8)(((data_buf & 0x00FF0000) >> 16) & 0xFF));

            if (0 == ((index + 1) % (lcm_w/bpp)))
            {
            #ifdef ZOOM_IN_RATIO
                readAddr += ((cam_w-lcm_w*ZOOM_IN_RATIO)*bpp+step)*4/4;
                if ( 0==(index+1)/(lcm_w/bpp)%2 )
                	readAddr += cam_w*bpp*4/4;
            #else
                readAddr += ((cam_w-lcm_w)*bpp+2*bpp);
            #endif
                row_idx += 1;
                col_idx = (cam_w-lcm_w)/2;
            }
            else
			{
            	readAddr += step;
            	col_idx += 2;
			}
        }
  
#endif    
			
			
			
		
		
		}
				
#endif
}

struct panel_driver st7789_240x320_driver = {
    .init           = _st7789_240x320_init,
    .clear          = _st7789_240x320_clear,
    .read_did       = _st7789_240x320_read_display_id,
    .start          = _st7789_240x320_start,
    .stop           = _st7789_240x320_stop,
    .update_image   = _st7789_240x320_update_image,
};


#endif
#endif
