#include "config/board_kdp520.h"
#ifdef V2K_ENABLE
#if (DISPLAY_DEVICE == DISPLAY_DEVICE_LCDC)
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <cmsis_os2.h>
#include "kneron_mozart.h"
#include "base.h"
#include "io.h"
#include "scu_extreg.h"
#include "clock.h"
#include "pinmux.h"
#include "framework/init.h"
#include "framework/ioport.h"
#include "framework/framework_errno.h"
#include "framework/mm.h"
#include "framework/framework_driver.h"
#include "framework/v2k_image.h"
#include "framework/event.h"
#include "media/display/lcdc.h"
#include "media/display/kdp_display.h"
#include "media/display/kdp_2d.h"
#include "dbg.h"
#include "framework/result.h"
#include "framework/v2k_color.h"
#include "media/kdp_fb_mgr.h"
#include "kdp_app.h"
#include "media/kdp_inference.h"
/*=============================== TEST #define MCRO Control ===============================*/
//#define TEST_LCDC_PATTERN

/*===================================== #define MACRO =====================================*/
#define MAX_FRAME_NUM                   (1) //2 for PIP
#define MIXING_MODE_OPEN_RENDERER
#define FRAME_SIZE_RGB(xres,yres,mbpp)  ((xres) * (yres) * (mbpp) / 8)

#define GET_RGB565(r, g, b) \
    (((((unsigned char)r)>>3)<<11) | ((((unsigned char)g)>>2)<<5) | (((unsigned char)b)>>3))

#define to_kdp520_lcdc_drvdata(_info) \
    container_of(_info, struct kdp520_lcdc_context, info)

#define USE_INFERENCE_THREAD_TO_DRAW

#ifdef PIP_FEATURE
struct pip_window
{
    int x;
    int y;
    int width;
    int height;
};
#endif

// WR added - not finding example anywhere
struct display_pen_info
{
    int width;
    int color;
};

struct kdp520_lcdc_context
{
    u16 cam_src_idx;
    u16 input_xres;
    u16 input_yres;

    //u32 base;
#ifdef PIP_FEATURE
    u32 frame_num;
    u32 pip_num;
    u32 pip_width[MAX_FRAME_NUM];
    u32 pip_height[MAX_FRAME_NUM];
#endif
    u32 frame_width[MAX_FRAME_NUM];
    u32 frame_height[MAX_FRAME_NUM];
    u32 frame_size[MAX_FRAME_NUM];
    u32 frame_buffer_addr[MAX_FRAME_NUM];

    struct fb_info info; /* FB driver info record */
    struct display_pen_info pen_info;
} lcdc_ctx;


static u32 _kdp520_lcdc_get_bpp_from_img_pixfmt(enum lcdc_img_pixfmt pixfmt);


struct display_driver kdp520_lcdc_driver;
int lcdc_frame_addr = 0;

extern struct kdp_rect h_boundingbox;
extern struct kdp_rect s_boundingbox;

/*=============================== extern  ===============================*/
extern struct clock_node clock_node_lcdc;
extern struct core_device kdp520_csi2rx0;
extern struct core_device kdp520_csi2rx1;


static __inline int _kdp520_lcdc_draw_pixel(
    u32 frame_buffer, int x, int y, u32 width, u16 color)
{
    *(u16 *)(frame_buffer + ((y * width + x) << 1) + 0) = color;
    
    return 0;
}

/********************************************************************************
 *
 * Frame buffer data endianness control. (ref to 2.4 Pixel Data Unpack)
 *
 * 00: Little-endian byte little-endian pixel
 * 01: Big-endian byte big-endian pixel
 * 10: Little-endian byte big-endian pixel (WinCE)
 * 11: Reserved
 *
 * @param type: @ref lcdc_fb_data_endianness_type
 *
 ********************************************************************************/
static void _kdp520_lcdc_set_endian(enum lcdc_fb_data_endianness_type type)
{
    switch (type) {
        case lcdc_fb_data_endianness_lblp:
            LCDC_REG_PANEL_PIXEL_SET_Endian(0);
            break;
        case lcdc_fb_data_endianness_bbbp:
            LCDC_REG_PANEL_PIXEL_SET_Endian(1);
            break;
        case lcdc_fb_data_endianness_lbbp:
            LCDC_REG_PANEL_PIXEL_SET_Endian(2);
            break;
        default:
            break;
    }
}

/********************************************************************************
 *
 * LCD timing and polarity parameters.
 * This register is for LCD panel clock divisor control (LC_PCLK).
 *
 * LC_PCLK is divided from LC_CLK.
 * The actual divisor value is equal to (DivNo + 1)
 *
 * @param div: @ref lcdc_fb_data_endianness_type
 *
 ********************************************************************************/
static void _kdp520_lcdc_set_divno(u32 div)
{
    LCDC_REG_POLARITY_CTRL_SET_DivNo(div);
}

/********************************************************************************
 *
 * Image0 Pixel format
 *
 * The value indicates the data format of image0 (Bit per pixel).
 * 000: 1 bpp
 * 001: 2 bpp
 * 010: 4 bpp
 * 011: 8 bpp
 * 100: 16 bpp
 * 101: 24 bpp
 * 110: ARGB8888
 * 111: ARGB1555
 *
 * @param div: @ref lcdc_fb_data_endianness_type
 *
 ********************************************************************************/
static void _kdp520_lcdc_set_img_fmt(struct lcdc_img_pixfmt_pxp *fmt)
{
    u32 val = inw(LCDC_REG_PIPPOP_FMT_1);
    u32 bpp0 = _kdp520_lcdc_get_bpp_from_img_pixfmt(fmt->pixfmt_img0);
    u32 bpp1 = _kdp520_lcdc_get_bpp_from_img_pixfmt(fmt->pixfmt_img1) << 4;
    u32 bpp2 = _kdp520_lcdc_get_bpp_from_img_pixfmt(fmt->pixfmt_img2) << 8;
    u32 bpp3 = _kdp520_lcdc_get_bpp_from_img_pixfmt(fmt->pixfmt_img3) << 12;

    //val |= bpp0 | bpp1 | bpp2 | bpp3;
    val = bpp0 | bpp1 | bpp2 | bpp3;
    LCDC_REG_PIPPOP_FMT_1_SET_BppFifo0(val);
}

static void _kdp520_lcdc_set_pinmux()
{
    PINMUX_LC_PCLK_SET(PINMUX_LC_PCLK);
    PINMUX_LC_VS_SET(PINMUX_LC_VS);
    PINMUX_LC_HS_SET(PINMUX_LC_HS);
    PINMUX_LC_DE_SET(PINMUX_LC_DE);
    PINMUX_LC_DATA0_SET(PINMUX_LC_DATA0);
    PINMUX_LC_DATA1_SET(PINMUX_LC_DATA1);
    PINMUX_LC_DATA2_SET(PINMUX_LC_DATA2);
    PINMUX_LC_DATA3_SET(PINMUX_LC_DATA3);
    PINMUX_LC_DATA4_SET(PINMUX_LC_DATA4);
    PINMUX_LC_DATA5_SET(PINMUX_LC_DATA5);
    PINMUX_LC_DATA6_SET(PINMUX_LC_DATA6);
    PINMUX_LC_DATA7_SET(PINMUX_LC_DATA7);
    PINMUX_LC_DATA8_SET(PINMUX_LC_DATA8);
    PINMUX_LC_DATA9_SET(PINMUX_LC_DATA9);
    PINMUX_LC_DATA10_SET(PINMUX_LC_DATA10);
    PINMUX_LC_DATA11_SET(PINMUX_LC_DATA11);
    PINMUX_LC_DATA12_SET(PINMUX_LC_DATA12);
    PINMUX_LC_DATA13_SET(PINMUX_LC_DATA13);
    PINMUX_LC_DATA14_SET(PINMUX_LC_DATA14);
    PINMUX_LC_DATA15_SET(PINMUX_LC_DATA15);
    PINMUX_SD_CLK_SET(PINMUX_SD_CLK_LC_DATA16);
    PINMUX_SD_CMD_SET(PINMUX_SD_CMD_LC_DATA17);
}

static u32 _kdp520_lcdc_get_bpp_from_img_pixfmt(enum lcdc_img_pixfmt pixfmt)
{
    u32 bpp = 0;
    switch (pixfmt) {
        case lcdc_img_pixfmt_1bpp:
            bpp = 0;
            break;
        case lcdc_img_pixfmt_2bpp:
            bpp = 1;
            break;
        case lcdc_img_pixfmt_4bpp:
            bpp = 2;
            break;
        case lcdc_img_pixfmt_8bpp:
            bpp = 3;
            break;
        case lcdc_img_pixfmt_16bpp:
            bpp = 4;
            break;
        case lcdc_img_pixfmt_24bpp:
            bpp = 5;
            break;
        case lcdc_img_pixfmt_argb888:
            bpp = 6;
            break;
        case lcdc_img_pixfmt_argb1555:
            bpp = 7;
            break;
        default:
            bpp = 4;
            break;
    }
    return bpp;
}

static void _kdp520_lcdc_init_rgb_value()
{
    int i;
    for (i = 0; i < 0x100; i++) {
        writeb(i, LCDC_REG_LT_OF_GAMMA_RED + i);
        writeb(i, LCDC_REG_LT_OF_GAMMA_GREEN + i);
        writeb(i, LCDC_REG_LT_OF_GAMMA_BLUE + i);
    }
}

static void _kdp520_lcdc_init_palette()
{
    int i;
    u16 color;

    for (i = 0; i < 256; i++) {
        color = ((i >> 3) << 11) | ((i >> 2) << 5) | ((i >> 3) << 0);
        writew(color, LCDC_REG_PALETTE + i * 2);
    }
}

static void _kdp520_lcdc_set_img_input_fmt(struct core_device *core_d)
{
    struct display_driver *display_drv = &kdp520_lcdc_driver; // (struct display_driver *)framework_drv_get_drv(&core_d->pin_ctx);
    struct lcdc_img_pixfmt_pxp img_pixfmt;

    u32 pix_param = inw(LCDC_REG_PANEL_PIXEL);
    if (V2K_PIX_FMT_RGB565 == display_drv->vi_params.input_fmt) {
        img_pixfmt.pixfmt_img0 = lcdc_img_pixfmt_16bpp;
        img_pixfmt.pixfmt_img1 = lcdc_img_pixfmt_16bpp;
        img_pixfmt.pixfmt_img2 = lcdc_img_pixfmt_16bpp;
        img_pixfmt.pixfmt_img3 = lcdc_img_pixfmt_16bpp;
        _kdp520_lcdc_init_rgb_value();
        _kdp520_lcdc_set_img_fmt(&img_pixfmt);

        LCDC_REG_FUNC_ENABLE_SET_EnYCbCr(0);
        pix_param = (pix_param & ~(LCDC_REG_PANEL_PIXEL_RGBTYPE_MASK |
                                   LCDC_REG_PANEL_PIXEL_BppFifo_MASK) ) |
                    LCDC_REG_PANEL_PIXEL_BppFifo_16bpp |
                    LCDC_REG_PANEL_PIXEL_RGBTYPE_565;

        outw(LCDC_REG_PANEL_PIXEL, pix_param);
    } else if (V2K_PIX_FMT_RAW8 == display_drv->vi_params.input_fmt) {
        img_pixfmt.pixfmt_img0 = lcdc_img_pixfmt_8bpp;
        img_pixfmt.pixfmt_img1 = lcdc_img_pixfmt_8bpp;
        img_pixfmt.pixfmt_img2 = lcdc_img_pixfmt_8bpp;
        img_pixfmt.pixfmt_img3 = lcdc_img_pixfmt_8bpp;
        _kdp520_lcdc_init_rgb_value();
        _kdp520_lcdc_init_palette();
        _kdp520_lcdc_set_img_fmt(&img_pixfmt);

        LCDC_REG_FUNC_ENABLE_SET_EnYCbCr(0);
        pix_param = (pix_param & ~(LCDC_REG_PANEL_PIXEL_RGBTYPE_MASK |
                                   LCDC_REG_PANEL_PIXEL_BppFifo_MASK) ) |
                    LCDC_REG_PANEL_PIXEL_BppFifo_8bpp;
        outw(LCDC_REG_PANEL_PIXEL, pix_param);
    }
}

static void _kdp520_lcdc_alloc_framebuffer(struct kdp520_lcdc_context *context)
{
#ifdef PIP_FEATURE
    _kdp520_lcdc_decide_frame_num(ctx);
    _kdp520_lcdc_decide_frame_dim(ctx);
#endif

    LCDC_REG_PANEL_PIXEL_SET_Vcomp(0);
}


/********************************************************************************
 *
 * TFT panel color depth selection
 *
 * This value chooses the color depth of the output format.
 * 0: 6 bits per channel with a 18-bit panel interface
 * 1: 8 bits per channel with a 24-bit panel interface
 *
 * @param div: @ref lcdc_panel_type
 *
 ********************************************************************************/
void kdp520_lcdc_set_panel_type(enum lcdc_panel_type type)
{
    switch (type) {
        case lcdc_6bits_per_channel:
            LCDC_REG_PANEL_PIXEL_SET_PanelType(0);
            break;
        case lcdc_8bits_per_channel:
            LCDC_REG_PANEL_PIXEL_SET_PanelType(1);
            break;
        default:
            break;
    }
}

/********************************************************************************
 *
 * @brief: RGB or BGR output format selection
 *         0: RGB normal output
 *         1: BGR red and blue swapped
 *
 * @param div: @ref enum lcdc_output_fmt_sel
 *
 ********************************************************************************/
void kdp520_lcdc_set_bgrsw(enum lcdc_output_fmt_sel type)
{
    switch (type) {
        case lcdc_output_fmt_sel_rgb:
            LCDC_REG_PANEL_PIXEL_SET_BGRSW(0);
            break;
        case lcdc_output_fmt_sel_bgr:
            LCDC_REG_PANEL_PIXEL_SET_BGRSW(1);
            break;
        default:
            break;
    }
}


void lcdc_vstatus_isr(void)
{
    int status;
    struct kdp520_lcdc_context *ctx = &lcdc_ctx; //framework_drv_get_drvdata(&core_d->pin_ctx);

    status = inw(LCDC_REG_INTR_STATUS);
    dbg_msg("lcdc_vstatus_isr status=%x", status);

    if (status & LCDC_REG_INTR_STATUS_IntVstatus) {
        u32 buf_addr = kdp_fb_mgr_next_read(ctx->cam_src_idx, NULL);

        // Update next address
        outw(LCDC_REG_PANEL_IMAGE0_FRAME0, buf_addr);
        kdp_fb_mgr_free_read_buf(ctx->cam_src_idx);
        LCDC_REG_INTR_CLEAR_SET_ClrVstatus();
    }

    outw(LCDC_REG_INTR_CLEAR, status);
    NVIC_ClearPendingIRQ((IRQn_Type)LCDC_FTLCDC210_VSTATUS_IRQ);
}

void kdp520_lcdc_set_framerate(int frame_rate, u32 width, u32 height)
{
#define LCD_PLL 148500000
    u32 div;
    u32 serial_mode = LCDC_REG_SERIAL_PANEL_PIXEL_GET_SerialMode();
    u32 upper_margin = LCDC_REG_VERTICAL_BACK_PORCH_GET_VBP() + 1;
    u32 lower_margin = LCDC_REG_VERTICAL_TIMING_CTRL_GET_VFP();
    u32 vsync_len = LCDC_REG_VERTICAL_TIMING_CTRL_GET_VW() + 1;
    u32 left_margin = LCDC_REG_HORIZONTAL_TIMING_CTRL_GET_HBP() + 1;
    u32 right_margin = LCDC_REG_HORIZONTAL_TIMING_CTRL_GET_HFP() + 1;
    u32 hsync_len = LCDC_REG_HORIZONTAL_TIMING_CTRL_GET_HW() + 1;
    u32 pixel_clock = (serial_mode ? 3 : 1 ) * frame_rate *
                      (height + upper_margin + lower_margin + vsync_len) *
                      (width + left_margin + right_margin + hsync_len);

    //dbg_msg("[lcdc] kdp520_lcdc_set_framerate width=%d", width);
    //dbg_msg("[lcdc] kdp520_lcdc_set_framerate height=%d", height);
    dbg_msg("[lcdc] kdp520_lcdc_set_framerate left_margin=%d, right_margin=%d, hsync_len = %d", left_margin, right_margin, hsync_len);

    div = LCD_PLL / pixel_clock;
    dbg_msg("[lcdc] kdp520_lcdc_set_framerate div=0x%x, fbi->input_clock=%d, pclk = %d", div, LCD_PLL, pixel_clock);
    if (div < 1) {
        dbg_msg("invalid input clock for LCD");
        for (;;) ;
    }
    if ((height == 1080) && (width == 1920)) {
        div = 0;
    } else if ((height == 480) && (width == 640)) {
        div = 1;
    } else {
        div = 5;
    }

    _kdp520_lcdc_set_divno(div);
}

#if 0
static int kdp520_fb_blank(int blank_mode, struct fb_info *fbi)
{
    //struct kdp520fb_drvdata *drvdata = to_kdp520_lcdc_drvdata(fbi);

    /* to Do */

    return 0;
}
#endif

void kdp520_lcdc_get_dp_size(unsigned short *dp_xres, unsigned short *dp_yres)
{
    struct display_driver *display_drv = &kdp520_lcdc_driver;
    *dp_xres = display_drv->vi_params.input_xres;
    *dp_yres = display_drv->vi_params.input_yres;
}

#if 0
static int kdp520_lcdc_reset(struct core_device *core_d)
{

    SCU_EXTREG_SWRST_SET_LCDC_resetn(1);

    // outw(SCU_EXTREG_SWRST_MASK1,
    //      SCU_EXTREG_SWRST_MASK1_AResetn_u_FTLCDC210 |
    //      SCU_EXTREG_SWRST_MASK1_PRESETn_u_FTLCDC210 |
    //      SCU_EXTREG_SWRST_MASK1_TV_RSTn_FTLCDC210 |
    //      SCU_EXTREG_SWRST_MASK1_LC_SCALER_RSTn_FTLCDC210 |
    //      SCU_EXTREG_SWRST_MASK1_LC_RSTn_FTLCDC210);

    return 0;
}
#endif

static int kdp520_lcdc_attach_panel(struct core_device *core_d, struct panel_driver *panel)
{
    struct display_driver *display_drv = &kdp520_lcdc_driver; // (struct display_driver *)framework_drv_get_drv(&core_d->pin_ctx);

    display_drv->panel = panel;

    return 0;
}

static int kdp520_lcdc_set_params(struct core_device *core_d, struct video_input_params *params)
{
    struct display_driver *display_drv = &kdp520_lcdc_driver; // (struct display_driver *)framework_drv_get_drv(&core_d->pin_ctx);
    struct video_input_params *p = &display_drv->vi_params;
//    struct kdp520_lcdc_context *ctx = framework_drv_get_drvdata(&core_d->pin_ctx);

    memcpy(p, params, sizeof(*params));
    display_drv->fb_size = calc_framesize(p->input_xres, p->input_yres, p->input_fmt);

    return 0;
}

static int kdp520_lcdc_set_camera(struct core_device *core_d, unsigned int cam_idx)
{
    struct kdp520_lcdc_context *ctx = &lcdc_ctx; //framework_drv_get_drvdata(&core_d->pin_ctx);

    ctx->cam_src_idx = cam_idx;
    dbg_msg("[%s] ctx->cam_src_idx=%d", __func__, ctx->cam_src_idx);
    _kdp520_lcdc_alloc_framebuffer(ctx);

    return 0;
}

static int kdp520_lcdc_init(struct core_device *core_d)
{
//    struct kdp520_lcdc_context *ctx = framework_drv_get_drvdata(&core_d->pin_ctx);
   // struct display_driver *display_drv = &kdp520_lcdc_driver; // (struct display_driver *)framework_drv_get_drv(&core_d->pin_ctx);

#if 1 // test new flow
    SCU_EXTREG_SWRST_SET_LCDC_resetn(0); //software reset

    SCU_EXTREG_CLK_EN1_SET_LC_SCALER(1);
    SCU_EXTREG_CLK_EN1_SET_LC_CLK(1);

    SCU_EXTREG_SWRST_SET_LCDC_resetn(1); //reset release
    clock_mgr_change_pll5_clock(1, 0x54, 5); //***** MUST SET Clock
#else
    SCU_EXTREG_SWRST_SET_LCDC_resetn(0); //software reset

    SCU_EXTREG_CLK_EN1_SET_LC_SCALER(1);
    SCU_EXTREG_CLK_EN1_SET_LC_CLK(1);

    clock_mgr_change_pll5_clock(1, 0x54, 5);

    SCU_EXTREG_SWRST_SET_LCDC_resetn(1); //reset release
#endif
    _kdp520_lcdc_set_pinmux();

    //clear all interrupt status
    outw(LCDC_REG_INTR_CLEAR, 0x0f);

    //enable buserr / vstatus / next frame / fifo underrun interrupt
    outw(LCDC_REG_INTR_ENABLE_MASK, 0x0f);

    _kdp520_lcdc_set_img_input_fmt(core_d);

    // set FIFO thread
    outw(LCDC_REG_FIFO_THRESHOLD, 0x04040404);

    _kdp520_lcdc_set_endian(lcdc_fb_data_endianness_lblp);

    NVIC_SetVector((IRQn_Type)LCDC_FTLCDC210_VSTATUS_IRQ, (uint32_t)lcdc_vstatus_isr);
    NVIC_EnableIRQ(LCDC_FTLCDC210_VSTATUS_IRQ);

#ifdef TEST_LCDC_PATTERN
    LCDC_REG_FUNC_ENABLE_SET_PenGen(1);
#endif

    if (kdp520_lcdc_driver.panel) {
        kdp520_lcdc_driver.display_id = kdp520_lcdc_driver.panel->read_did(core_d);
        dbg_msg("kdp520_lcdc_driver.display_id=%x", kdp520_lcdc_driver.display_id);
        kdp520_lcdc_driver.panel->init(core_d);
    }

    return 0;
}

static int kdp520_lcdc_power(struct core_device *core_d, bool on)
{
    return 0;
}

static int kdp520_lcdc_start(struct core_device *core_d)
{
    int ret = -KDP_FRAMEWORK_ERRNO_INVALA;
//    struct kdp520_lcdc_context *ctx = framework_drv_get_drvdata(&core_d->pin_ctx);

    dbg_msg("%s", __func__);

    if (core_d) {
        LCDC_REG_FUNC_ENABLE_SET_LCDen(1);
        LCDC_REG_FUNC_ENABLE_SET_LCDon(1);
        ret = 0;
    }

    return ret;
}

static int kdp520_lcdc_stop(struct core_device *core_d)
{
    int ret = -KDP_FRAMEWORK_ERRNO_INVALA;

    dbg_msg("%s", __func__);

    if (core_d) {
        LCDC_REG_FUNC_ENABLE_SET_LCDon(0);
        LCDC_REG_FUNC_ENABLE_SET_LCDen(0);
        ret = 0;
    }

    return ret;
}
//static int kdp520_lcdc_set_pen(struct core_device *core_d, u8 r, u8 g, u8 b, unsigned int width)
static int kdp520_lcdc_set_pen(struct core_device *core_d, u16 color, unsigned int width)
{
    int ret = -KDP_FRAMEWORK_ERRNO_INVALA;
    struct kdp520_lcdc_context *ctx = &lcdc_ctx; // framework_drv_get_drvdata(&core_d->pin_ctx);

    //dbg_msg("%s", __func__);

    if (core_d) {
        // ctx->pen_info.r = r;
        // ctx->pen_info.g = g;
        // ctx->pen_info.b = b;
        ctx->pen_info.color = color;
        ctx->pen_info.width = width;
        ret = 0;
    }

    return ret;
}

/*TODO: merge kdp520_lcdc_draw_static_rect() and kdp520_lcdc_draw_rect() into a API*/
static int kdp520_lcdc_draw_static_rect(struct core_device *core_d,
                                 u32 fb_addr, u32 org_x, u32 org_y, u32 width, u32 height)
{
    int ret = -KDP_FRAMEWORK_ERRNO_INVALA;
    struct kdp520_lcdc_context *ctx = &lcdc_ctx; //framework_drv_get_drvdata(&core_d->pin_ctx);
	struct display_driver *display_drv = &kdp520_lcdc_driver; // (struct display_driver *)framework_drv_get_drv(&core_d->pin_ctx);

    if (core_d) {

        u32 x_pos, y_pos;
        u32 x_unit = 1;
        u32 y_unit = 1;
        //u32 frame_no = 0;
        u32 top = org_y;
        u32 left = org_x;
        u32 right = org_x + width;
        u32 bottom = top + height;
        u32 frame_width =  display_drv->vi_params.input_xres;//ctx->frame_width[frame_no];
        //u32 frame_height = display_drv->vi_params.input_yres;//ctx->frame_height[frame_no];
        u32 frame_buffer_addr = fb_addr;//ctx->frame_buffer[frame_no];
        u32 border_size = ctx->pen_info.width;
        u32 left_border = left + border_size;
        u32 right_border = right - border_size;
        u32 top_lower = top + border_size;
        u32 bottom_upper = bottom - border_size;
        u16 color = ctx->pen_info.color;

        //dbg_msg("ctx->frame_buffer[frame_no]=%d %d %x",
        //ctx->frame_width[frame_no], ctx->frame_height[frame_no], ctx->frame_buffer[frame_no]);

        for (y_pos = top; y_pos < bottom; y_pos += y_unit)
        {
            if ((y_pos >= top && y_pos < top_lower) || 
                (y_pos >= bottom_upper))
            {
                for (x_pos = left; x_pos < right; x_pos += x_unit)
                {
                    if ((x_pos >= left && x_pos < (g_frame_margin_length+border_size)*3) || 
                        (x_pos >= (right_border - (g_frame_margin_length+border_size)*2)))
                    {
                        //dbg_msg("x_pos=%d y_pos=%d", x_pos, y_pos);
                        _kdp520_lcdc_draw_pixel(frame_buffer_addr, x_pos, y_pos, frame_width, color);
                    }
                }
            }
            else
            {
                if ((y_pos >= top && y_pos < (g_frame_margin_length+border_size)*3) || 
                    (y_pos >= (bottom_upper - (g_frame_margin_length+border_size)*2)))
                {
                    for (x_pos = left; x_pos < left_border; x_pos += x_unit)
                    {
                        //dbg_msg("x_pos=%d y_pos=%d", x_pos, y_pos);
                        _kdp520_lcdc_draw_pixel(frame_buffer_addr, x_pos, y_pos, frame_width, color);
                    }
           
                    for (x_pos = right_border; x_pos < right; x_pos += x_unit)
                    {
                        //dbg_msg("x_pos=%d y_pos=%d", x_pos, y_pos);
                        _kdp520_lcdc_draw_pixel(frame_buffer_addr, x_pos, y_pos, frame_width, color);
                    }
                }
            }
        }

        ret = 0;
    }

    return ret;
}

static int kdp520_lcdc_draw_rect(struct core_device *core_d,
                                 u32 fb_addr, u32 org_x, u32 org_y, u32 width, u32 height)
{
    int ret = -KDP_FRAMEWORK_ERRNO_INVALA;
    struct kdp520_lcdc_context *ctx = &lcdc_ctx; // framework_drv_get_drvdata(&core_d->pin_ctx);
	struct display_driver *display_drv = &kdp520_lcdc_driver; // (struct display_driver *)framework_drv_get_drv(&core_d->pin_ctx);

    if (core_d) {

        u32 x_pos, y_pos;
        u32 x_unit = 1;
        u32 y_unit = 1;
        //u32 frame_no = 0;
        u32 top = org_y;
        u32 left = org_x;
        u32 right = org_x + width;
        u32 bottom = top + height;
        u32 frame_width =  display_drv->vi_params.input_xres;//ctx->frame_width[frame_no];
        //u32 frame_height = display_drv->vi_params.input_yres;//ctx->frame_height[frame_no];
        u32 frame_buffer_addr = fb_addr;//ctx->frame_buffer[frame_no];
        u32 border_size = ctx->pen_info.width;
        u32 left_border = left + border_size;
        u32 right_border = right - border_size;
        u32 top_lower = top + border_size;
        u32 bottom_upper = bottom - border_size;
        u16 color = ctx->pen_info.color;
        

        //dbg_msg("ctx->frame_buffer[frame_no]=%d %d %x",
        //ctx->frame_width[frame_no], ctx->frame_height[frame_no], ctx->frame_buffer[frame_no]);

        for (y_pos = top; y_pos < bottom; y_pos += y_unit)
        {
            if ((y_pos >= top && y_pos < top_lower) || 
                (y_pos >= bottom_upper))
            {
                for (x_pos = left; x_pos < right; x_pos += x_unit)
                {
                     _kdp520_lcdc_draw_pixel(frame_buffer_addr, x_pos, y_pos, frame_width, color);
                }
            }
            else
            {
                for (x_pos = left; x_pos < left_border; x_pos += x_unit)
                {
                    //dbg_msg("x_pos=%d y_pos=%d", x_pos, y_pos);
                    _kdp520_lcdc_draw_pixel(frame_buffer_addr, x_pos, y_pos, frame_width, color);
                }
       
                for (x_pos = right_border; x_pos < right; x_pos += x_unit)
                {
                    //dbg_msg("x_pos=%d y_pos=%d", x_pos, y_pos);
                    _kdp520_lcdc_draw_pixel(frame_buffer_addr, x_pos, y_pos, frame_width, color);
                }
            }
        }

        ret = 0;
    }

    return ret;
}

#ifdef PIP_FEATURE
static int kdp520_lcdc_pip_test(struct core_device *core_d)
{
    int ret = -KDP_FRAMEWORK_ERRNO_INVALA;
    /*
        dbg_msg("%s", __func__);

        if (core_d) {
            //struct kdp520_lcdc_context *ctx = framework_drv_get_drvdata(&core_d->pin_ctx);
            struct display_driver *display_drv = (struct display_driver *)framework_drv_get_drv(&core_d->pin_ctx);
            struct video_input_params *p = &display_drv->vi_params;

            int i;
            int pip_num=2;
            int hstep1=1;
            int vstep1=1;
            int hstep2=1;
            int vstep2=1;
            u32 panel_width = p->input_xres;
            u32 panel_height = p->input_yres;
            struct pip_window osd_win1 = {150, 150, 8, 8};
            struct pip_window osd_win2 = {50, 50, 8, 8};


            _lcdc_set_pip_pos(ctx, 0, 50, 50);
            _lcdc_set_pip_pos(ctx, 1, 50+50, 50+50);

            _lcdc_set_pip_dim(ctx, 0, panel_width >> 2, panel_height >> 2);
            _lcdc_set_pip_dim(ctx, 1, panel_width >> 2, panel_height >> 2);

            _lcdc_set_pip_enable(ctx, pip_num);
            _lcdc_set_pip_alpha_blending(ctx, alpha_blending_none, 0, 0);


            while(1){
                osd_win1.x += hstep1;
                osd_win1.y += vstep1;
                _lcdc_set_pip_pos(ctx, 0, osd_win1.x, osd_win1.y);
                if((osd_win1.x +(panel_width >> 2)) > (panel_width - 1)){
                    hstep1 = -1;
                }
                if(osd_win1.y + (panel_height >> 2) > (panel_height - 1)){
                    vstep1 = -1;
                }
                if(osd_win1.x == 0){
                    hstep1 = 1;
                }
                if(osd_win1.y == 0){
                    vstep1 = 1;
                }

                osd_win2.x += hstep2;
                osd_win2.y += vstep2;
                _lcdc_set_pip_pos(ctx, 1, osd_win2.x, osd_win2.y);
                if((osd_win2.x +(panel_width >> 2)) > (panel_width - 1)){
                    _lcdc_set_pip_alpha_blending(ctx, alpha_blending_none, 0x3, 0x1);
                    hstep2 = -1;
                }
                if(osd_win2.y + (panel_height >> 2) > (panel_height - 1)){
                    _lcdc_set_pip_alpha_blending(ctx, alpha_blending_none, 0, 0x6);
                    vstep2 = -1;
                }
                if(osd_win2.x == 0){
                    _lcdc_set_pip_alpha_blending(ctx, alpha_blending_none, 0, 0x7);
                    hstep2 = 1;
                }
                if(osd_win2.y == 0){
                    _lcdc_set_pip_alpha_blending(ctx, alpha_blending_none, 6, 0x0);
                    vstep2 = 1;
                }

                _lcdc_pip_update(ctx);
            }

            ret = 0;
        }
        */
    return ret;
}


static void _kdp520_lcdc_decide_frame_num(struct kdp520_lcdc_context *context)
{
    struct kdp520_lcdc_context *ctx = context;

    if (lcd_img_input_fmt_rgb565 == ctx->input_fmt ||
            lcd_img_input_fmt_rgb555 == ctx->input_fmt ||
            lcd_img_input_fmt_rgb444 == ctx->input_fmt ||
            lcd_img_input_fmt_rgb24 == ctx->input_fmt ||
            lcd_img_input_fmt_ycbcr422 == ctx->input_fmt) {
        if (ctx->pip_num > 0) {
            ctx->frame_num = ctx->pip_num + 1;
        } else {
            ctx->frame_num = 1;
        }
    } else {
        ctx->frame_num = 1;
    }
}

static void _kdp520_lcdc_decide_frame_dim(struct kdp520_lcdc_context *context)
{
    struct kdp520_lcdc_context *ctx = context;
    int i;

    dbg_msg("<%s> frame_num=%d", __func__, ctx->frame_num);
    for (i = 0; i < ctx->frame_num; ++i) {
        ctx->frame_width[i] = ctx->input_xres;
        ctx->frame_height[i] = ctx->input_yres;
        dbg_msg("[%s] ctx->frame_width[%d]=%d ctx->frame_height[%d]=%d", __func__, i, ctx->frame_width[i], i, ctx->frame_height[i]);

        if (ctx->frame_num > 1) {
            if (i > 0) {
                //fbi->frame_width[i] = fbi->pip_width[i-1]*(float)fbi->frame_width[i]/(float)fbi->panel_width;
                ctx->frame_width[i] = ctx->pip_width[i];
                //fbi->frame_height[i] = fbi->pip_height[i-1]*(float)fbi->frame_height[i]/(float)fbi->panel_height;
                ctx->frame_height[i] = ctx->pip_height[i];
            }
        }
    }
}

static  void _lcdc_pip_update(struct kdp520_lcdc_context *context)
{
    LCDC_REG_PIP_SUBPIC1_POS_SET_PiP_Update(1);
}

static void _lcdc_set_pip_pos(struct kdp520_lcdc_context *context, int pip_idx, u32  h_pos, u32 v_pos)
{
//    struct kdp520_lcdc_context *ctx = context;

    switch (pip_idx) {
        case 0:
            LCDC_REG_PIP_SUBPIC1_POS_SET_PiP1Hpos(0);
            LCDC_REG_PIP_SUBPIC1_POS_SET_PiP1Vpos(0);
            LCDC_REG_PIP_SUBPIC1_POS_SET_PiP1Hpos(h_pos);
            LCDC_REG_PIP_SUBPIC1_POS_SET_PiP1Vpos(v_pos);
            break;

        case 1:
            LCDC_REG_PIP_SUBPIC2_POS_SET_PiP1Hpos(0);
            LCDC_REG_PIP_SUBPIC2_POS_SET_PiP1Vpos(0);
            LCDC_REG_PIP_SUBPIC2_POS_SET_PiP1Hpos(h_pos);
            LCDC_REG_PIP_SUBPIC2_POS_SET_PiP1Vpos(v_pos);
            break;
        case 2:
            break;
        default:
            ;
    }

    LCDC_REG_PIP_SUBPIC1_POS_SET_PiP_Update(1);
}

static void _lcdc_set_pip_dim(struct kdp520_lcdc_context *context, int pip_idx, u32 h_dim, u32 v_dim)
{
    struct kdp520_lcdc_context *ctx = context;

    ctx->pip_width[pip_idx] = h_dim;
    ctx->pip_height[pip_idx] = v_dim;

    switch (pip_idx) {
        case 0:
            LCDC_REG_PIP_SUBPIC1_DIM_SET_PiP1Hdim(0);
            LCDC_REG_PIP_SUBPIC1_DIM_SET_PiP1Vdim(0);
            LCDC_REG_PIP_SUBPIC1_DIM_SET_PiP1Hdim(h_dim);
            LCDC_REG_PIP_SUBPIC1_DIM_SET_PiP1Vdim(v_dim);
            break;

        case 1:
            LCDC_REG_PIP_SUBPIC1_DIM_SET_PiP2Hdim(0);
            LCDC_REG_PIP_SUBPIC1_DIM_SET_PiP2Vdim(0);

            LCDC_REG_PIP_SUBPIC1_DIM_SET_PiP2Hdim(h_dim);
            LCDC_REG_PIP_SUBPIC1_DIM_SET_PiP2Vdim(v_dim);
            break;
        case 2:
            break;
        default:
            ;
    }
}

static int _lcdc_set_pip_enable(struct kdp520_lcdc_context *context, u32 pip_num)
{
    struct kdp520_lcdc_context *ctx = context;

    ctx->pip_num = pip_num;
    ctx->frame_num = pip_num + 1;

    LCDC_REG_FUNC_ENABLE_SET_PiPEn(0);
    LCDC_REG_FUNC_ENABLE_SET_PiPEn(pip_num);

    //_kdp520_lcdc_alloc_framebuffer(ctx);

    return 0;
}

void _lcdc_set_pip_alpha_blending(
    struct kdp520_lcdc_context *context,
    enum alpha_blending_type type, int blend_l, int blend_h)
{
//    struct kdp520_lcdc_context *ctx = context;

    switch (type) {
        case alpha_blending_none:
            LCDC_REG_FUNC_ENABLE_SET_BlendEn(0);
            break;
        case alpha_blending_global:
            LCDC_REG_PIP_BLENDING_SET_PiPBlend_l(blend_l);
            LCDC_REG_PIP_BLENDING_SET_PiPBlend_h(blend_h);
            LCDC_REG_FUNC_ENABLE_SET_BlendEn(1);
            break;
        case alpha_blending_pixel:
            LCDC_REG_FUNC_ENABLE_SET_BlendEn(2);
            break;
    }
}

#endif

extern struct core_device kdp520_lcdc;
struct display_driver kdp520_lcdc_driver = {
    .driver       = {
        .name     = "kdp520_lcdc",
    },
    .core_dev     = &kdp520_lcdc,

    .power_mgr    = {
        .power    = kdp520_lcdc_power,
        .suspend  = NULL,
        .resume   = NULL,
    },

    .ctrller_ops  = NULL,
    .attach_panel = kdp520_lcdc_attach_panel,
    .set_params   = kdp520_lcdc_set_params,
    .set_camera   = kdp520_lcdc_set_camera,
    .init         = kdp520_lcdc_init,
    .start        = kdp520_lcdc_start,
    .stop         = kdp520_lcdc_stop,
#ifdef PIP_FEATURE
    .pip_test     = kdp520_lcdc_pip_test,
#endif
    .set_pen      = kdp520_lcdc_set_pen,
    .draw_static_rect    = kdp520_lcdc_draw_static_rect,
    .draw_rect   = kdp520_lcdc_draw_rect
};

#endif
#endif
