#include "config/board_kdp520.h"
#ifdef V2K_ENABLE
#if (DISPLAY_DEVICE == DISPLAY_DEVICE_LCM)
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <cmsis_os2.h>
#include "kneron_mozart.h"
#include "base.h"
#include "io.h"
#include "clock.h"
#include "pinmux.h"
#include "scu_extreg.h"
#include "framework/mm.h"
#include "framework/event.h"
#include "framework/init.h"
#include "framework/ioport.h"
#include "framework/framework_errno.h"
#include "framework/framework_driver.h"
#include "framework/v2k_color.h"
#include "framework/v2k_image.h"
#include "media/display/lcm.h"
#include "media/display/display.h"
#include "kdp520_pwm_timer.h"
#include "dbg.h"
//#include "memory.h"
#include "framework/result.h"
#include "delay.h"


static u32 _prof_start, _prof_end;
static u32 _prof_offset, _prof_total = 0, _prof_frame_cnt = 0;
#define LCM_PROFILE_ENABLE
#ifdef LCM_PROFILE_ENABLE 
#define LCM_PROFILE_START() _prof_start = osKernelGetTickCount(); 
#define LCM_PROFILE_STOP() { \
                _prof_end = osKernelGetTickCount(); \
                _prof_offset = _prof_end - _prof_start; \
                _prof_total += _prof_offset; \
                ++_prof_frame_cnt; \
                /*DSG("[%s] offset=%u average=%u", __func__, _prof_offset, _prof_total / _prof_frame_cnt);*/ \
                }
#else
#define LCM_PROFILE_START() 
#define LCM_PROFILE_STOP() 
#endif

//#define INIT_WHEN_PROBE
        
#define LCM_BASE                SLCD_FTLCDC210_PA_BASE
#define FLAGS_LCM_START_EVT     0x0020
#define FLAGS_LCM_STOP_EVT      0x0040
#define FLAGS_LCM_QUIT_EVT      0x0008

enum lcm_signal_cycle {
    LCM_SIGNAL_WIDTH_LC_PCLK_X1  = 0x00,
    LCM_SIGNAL_WIDTH_LC_PCLK_X2  = 0x01,
    LCM_SIGNAL_WIDTH_LC_PCLK_X3  = 0x02,
    LCM_SIGNAL_WIDTH_LC_PCLK_X4  = 0x03,
    LCM_SIGNAL_WIDTH_LC_PCLK_X5  = 0x04,
    LCM_SIGNAL_WIDTH_LC_PCLK_X6  = 0x05,
    LCM_SIGNAL_WIDTH_LC_PCLK_X7  = 0x06,
    LCM_SIGNAL_WIDTH_LC_PCLK_X8  = 0x07,
    LCM_SIGNAL_WIDTH_LC_PCLK_X9  = 0x08,
    LCM_SIGNAL_WIDTH_LC_PCLK_X10 = 0x09,
    LCM_SIGNAL_WIDTH_LC_PCLK_X11 = 0x0A,
    LCM_SIGNAL_WIDTH_LC_PCLK_X12 = 0x0B,
    LCM_SIGNAL_WIDTH_LC_PCLK_X13 = 0x0C,
    LCM_SIGNAL_WIDTH_LC_PCLK_X14 = 0x0D,
    LCM_SIGNAL_WIDTH_LC_PCLK_X15 = 0x0E,
    LCM_SIGNAL_WIDTH_LC_PCLK_X16 = 0x0F,
};

enum lcm_16bitmode_selection {
    LCM_16BITMODE_ONESHOT   = 0x00,
    LCM_16BITMODE_16_AT_MSB = 0x01,
    LCM_16BITMODE_16_AT_LSB = 0x01,
};

enum lcm_data_bus_type {
    LCM_DATA_BUS_8_BITS  = 0x00,
    LCM_DATA_BUS_9_BITS  = 0x01,
    LCM_DATA_BUS_16_BITS = 0x02,
    LCM_DATA_BUS_18_BITS = 0x03,
};

enum lcm_panel_interface_type {
    LCM_PANEL_INTEFACE_MONO    = 0x00,
    LCM_PANEL_INTEFACE_16_BITS = 0x00,
    LCM_PANEL_INTEFACE_18_BITS = 0x00,
};

enum lcm_mcu_interface_mode {
    LCM_MCU_INTERFACE_8080 = 0x00,
    LCM_MCU_INTERFACE_6800 = 0x01,
};

struct pip_window
{
    int x;
    int y;
    int width;
    int height;
};

struct lcm_pen_info
{
    u16 color;
    // u8 r;
    // u8 g;
    // u8 b;
    unsigned int width;
};

struct kdp520_lcm_context {
    u16 cam_src_idx;
    u32 base;
    struct lcm_pen_info pen_info;
    u32 frame_buffer_addr;
    struct core_device *camera_ctrller;
};

extern void small_delay(int us);

enum lcm_state {
    LCM_STATE_IDLE = 0,
    LCM_STATE_STOPPED,
    LCM_STATE_STARTED,
};

struct display_driver kdp520_lcm_driver;

enum lcm_state m_lcm_state;
static osThreadId_t m_tid_lcm;
static osEventFlagsId_t evt_lcm;

extern BOOL m_fdr_done;
extern struct kdp_rect m_boundingbox;
extern fdr_result m_fdr_result;
#ifdef BOARD_CUSTOMER1
extern u32 sys_fdr_state;
#endif
static void lcm_thread(void *argument)
{
    u32 flags;
    u32 status;

    struct core_device *core_d = (struct core_device*)argument;
    struct display_driver *display_drv = (struct display_driver *)framework_drv_get_drv(&core_d->pin_ctx);
    struct kdp520_lcm_context *ctx = framework_drv_get_drvdata(&core_d->pin_ctx);
    struct panel_driver *panel = display_drv->panel;

    memset(&m_boundingbox, 0, sizeof(m_boundingbox));

    //DSG("xxx color=%u pen_width= %u", color, pen_width);

    while (1) {
        flags = osThreadFlagsWait(FLAGS_LCM_START_EVT /*| FLAGS_LCM_STOP_EVT | FLAGS_LCM_QUIT_EVT  */, 
                                  osFlagsWaitAny, osWaitForever);
        if (FLAGS_LCM_START_EVT == flags) {
        }
        else if (FLAGS_LCM_STOP_EVT == flags) {
        }
        else if (FLAGS_LCM_QUIT_EVT == flags) {
        }
        osThreadFlagsClear(FLAGS_LCM_START_EVT /* | FLAGS_LCM_STOP_EVT | FLAGS_LCM_QUIT_EVT */);

        while(1) {
            if (LCM_STATE_STARTED == m_lcm_state) {
                LCM_PROFILE_START();
            #ifdef BOARD_CUSTOMER1                
                if (2 == sys_fdr_state)
                {
                    if (FDR_RET_DB_COMP_OK == m_fdr_result)
                        panel->update_image(core_d, ctx->frame_buffer_addr, GREEN, ctx->pen_info.width, &m_boundingbox);
                    else if (FDR_RET_OK == m_fdr_result)
                        panel->update_image(core_d, ctx->frame_buffer_addr, RED, ctx->pen_info.width, &m_boundingbox);
					else
                        panel->update_image(core_d, ctx->frame_buffer_addr, ctx->pen_info.color, ctx->pen_info.width, NULL);
                }
				else if (0 == sys_fdr_state)
                {
					panel->update_image(core_d, ctx->frame_buffer_addr, ctx->pen_info.color, ctx->pen_info.width, NULL);
                }
                else
            #endif
                {
                    if (FDR_RET_OK == m_fdr_result)
                        panel->update_image(core_d, ctx->frame_buffer_addr, ctx->pen_info.color, ctx->pen_info.width, &m_boundingbox);
                    else if (FDR_RET_DB_COMP_OK == m_fdr_result)
                        panel->update_image(core_d, ctx->frame_buffer_addr, GREEN, ctx->pen_info.width, &m_boundingbox);
                    else
                        panel->update_image(core_d, ctx->frame_buffer_addr, ctx->pen_info.color, ctx->pen_info.width, NULL);
                }
                LCM_PROFILE_STOP();
                //__WFI();
                osDelay(5);
            }
            else 
                break;  
        }           
        if (LCM_STATE_IDLE == m_lcm_state)
            break;
    }  
}

static void _lcm_wait_ready(u32 base)
{
    //DSG("[%s]", __func__);
    unsigned int rdata;
    do{ 
        rdata = inw(LCM_REG_RDY_P(base));
    }while((rdata & LCM_REG_RDY_READY_FOR_ACCESS) != LCM_REG_RDY_READY_FOR_ACCESS);
}

static void _lcm_write_cmd(u32 base, unsigned char data)
{
    _lcm_wait_ready(base);
    outb(LCM_REG_CMD_P(base), data);
}

static void _lcm_write_data(u32 base, unsigned char data)
{
    _lcm_wait_ready(base);    
    outb(LCM_REG_DATA_P(base), data);
}

unsigned int _lcm_read_data(u32 base)
{
    _lcm_wait_ready(base);
    outb(LCM_REG_RS_P(base), LCM_REG_RS_DMYRD_RS);

    _lcm_wait_ready(base);
    return inw(LCM_REG_DATA_P(base));
}

static void _lcm_set_16bitmode_selection(enum lcm_16bitmode_selection sel)
{
    LCM_REG_OP_MODE_SET_16bit_mode(sel);
}

static void _lcm_set_data_bus_type(enum lcm_data_bus_type type) 
{
    LCM_REG_OP_MODE_SET_Bus_IF(type);
}

static void _lcm_set_panel_interface_type(enum lcm_panel_interface_type type) 
{
    LCM_REG_OP_MODE_SET_Bus_IF(type);
}

static void _lcm_set_mcu_interface_mode(enum lcm_mcu_interface_mode mode) 
{
    LCM_REG_OP_MODE_SET_C68(mode);
}

static void _lcm_set_pwh_r(enum lcm_signal_cycle cycle)
{
    //Tpwh width for the read cycles
    LCM_REG_TIMING_SET_Tpwh_r((u32)cycle);
}

static void _lcm_set_as(enum lcm_signal_cycle cycle)
{
    LCM_REG_TIMING_SET_Tas((u32)cycle); 
}

static void _lcm_set_ah(enum lcm_signal_cycle cycle)
{
    LCM_REG_TIMING_SET_Tah((u32)cycle); 
}

static void _lcm_set_pwl(enum lcm_signal_cycle cycle)
{
    LCM_REG_TIMING_SET_Tpwl((u32)cycle); 
}

static void _lcm_set_pwh_w(enum lcm_signal_cycle cycle)
{
    //Tpwh width for the write cycles
    LCM_REG_TIMING_SET_Tpwh_w((u32)cycle); 
}


extern struct core_device kdp520_csi2rx0;
extern struct core_device kdp520_csi2rx1;
extern struct kdp520_csi2rx_context_lite *g_csi2rx0_ctx;
extern struct kdp520_csi2rx_context_lite *g_csi2rx1_ctx;
static void _lcm_alloc_framebuffer(
        struct kdp520_lcm_context *context)
{
    struct kdp520_lcm_context *ctx = context;

    if (0 == ctx->cam_src_idx) {// == g_csi2rx0_ctx->inst_idx) {
        ctx->frame_buffer_addr = g_csi2rx0_ctx->vw_mem_addr;
    }
    else if (1 == ctx->cam_src_idx) {// == g_csi2rx1_ctx->inst_idx) {
        ctx->frame_buffer_addr = g_csi2rx1_ctx->vw_mem_addr;            
    }
    else if (2 == ctx->cam_src_idx)
       ctx->frame_buffer_addr = KDP_DDR_BASE_IMAGE_FOR_APP;
    else if (3 == ctx->cam_src_idx)
       ctx->frame_buffer_addr = KDP_DDR_BASE_IMAGE_FOR_APP + KDP_IMAGE_BUF_SIZE_FOR_APP;

    DSG("[%s] ctx->frame_buffer_addr=%x", __func__, ctx->frame_buffer_addr);

    int color[]={0x00ffff, 0x00008888, 0x00004444, 0x00002222};
}

static __inline int _kdp520_lcm_draw_pixel(
		u32 frame_buffer, int x, int y, u32 width, u16 color)
{
	*(u16 *)(frame_buffer + ((y * width + x) << 1) + 0) = color; 
}

static int kdp520_lcm_probe(struct core_device *core_d)
{
    struct kdp520_lcm_context *ctx;
    struct display_driver *display_drv = (struct display_driver *)framework_drv_get_drv(&core_d->pin_ctx);

    DSG("<%s>", __func__);

    ctx = calloc(1, sizeof(*ctx));
    if (!ctx) {
        return -KDP_FRAMEWORK_ERRNO_NOMEM;
    }

    display_drv->base = driver_get_ioport_setting(core_d, IORESOURCE_BUS)->start;
    ctx->base = driver_get_ioport_setting(core_d, IORESOURCE_BUS)->start;
    
    DSG("[%s] core_d->name=%s", __func__, core_d->name);
    DSG("[%s] ctx->base=%x", __func__, ctx->base);
    DSG("[%s] display_drv->base=%x", __func__, display_drv->base);

    framework_drv_set_drvdata(&core_d->pin_ctx, (void*)ctx);

    //evt_lcm = create_event();
    if (0 != m_tid_lcm)
        osThreadTerminate(m_tid_lcm);
    m_tid_lcm = osThreadNew(lcm_thread, (void*)core_d, NULL);
    int num = osThreadGetCount();
    DSG("osThreadGetCount = %d", num);
	
#ifdef THREAD_TUNING
    osThreadSetPriority (m_tid_lcm, osPriorityRealtime7);
#endif
    return 0;
}

static int kdp520_lcm_remove(struct core_device *core_d)
{
    struct kdp520_lcm_context *ctx = framework_drv_get_drvdata(&core_d->pin_ctx);

    m_lcm_state = LCM_STATE_IDLE;

    //set_thread_event(m_tid_lcm, FLAGS_LCM_QUIT_EVT);

    return 0;
}

static int kdp520_lcm_reset(struct core_device *core_d)
{
    return 0;
}

static int kdp520_lcm_attach_panel(struct core_device *core_d, struct panel_driver *panel)
{
    struct display_driver *display_drv = (struct display_driver *)framework_drv_get_drv(&core_d->pin_ctx);

    display_drv->panel = panel;
    
    return 0;
}

static int kdp520_lcm_set_params(struct core_device *core_d, struct video_input_params *params)
{
    struct display_driver *display_drv = (struct display_driver *)framework_drv_get_drv(&core_d->pin_ctx);
    struct video_input_params *p = &display_drv->vi_params;
    
    memcpy(p, params, sizeof(*params));
    DSG(" *params=%u i=%u", sizeof(*params), params->input_xres);
  
    display_drv->fb_size = calc_framesize(p->input_xres, p->input_yres, p->input_fmt);
    DSG("[%s] video_input_params->input_xres=%u", __func__, p->input_xres);
    DSG("[%s] video_input_params->input_yres=%u", __func__, p->input_yres);
    DSG("[%s] video_input_params->input_fmt=%x", __func__, p->input_fmt);  
    DSG("[%s] display_driver->fb_size=%d", __func__, display_drv->fb_size);
    
    return 0;
}

static int kdp520_lcm_set_camera(struct core_device *core_d, unsigned int cam_idx)
{
    struct kdp520_lcm_context *ctx = framework_drv_get_drvdata(&core_d->pin_ctx);
    
    ctx->cam_src_idx = cam_idx;
    
    _lcm_alloc_framebuffer(ctx);

    return 0;
}

static int kdp520_lcm_init(struct core_device *core_d)
{
    struct kdp520_lcm_context *ctx = framework_drv_get_drvdata(&core_d->pin_ctx);
    u16 id = 0;
    
    DSG("[%s]", __func__);

    SCU_EXTREG_SWRST_MASK1_SET_lcm_reset_n(1);
    SCU_EXTREG_MISC_SET_lcm_cken(1);
    
    _lcm_set_as(LCM_SIGNAL_WIDTH_LC_PCLK_X2);
    _lcm_set_ah(LCM_SIGNAL_WIDTH_LC_PCLK_X2);
    _lcm_set_pwl(LCM_SIGNAL_WIDTH_LC_PCLK_X2);
    _lcm_set_pwh_r(LCM_SIGNAL_WIDTH_LC_PCLK_X4);
    _lcm_set_pwh_w(LCM_SIGNAL_WIDTH_LC_PCLK_X4);

    _lcm_set_16bitmode_selection(LCM_16BITMODE_ONESHOT);
    _lcm_set_data_bus_type(LCM_DATA_BUS_16_BITS);
    //_lcm_set_data_bus_type(LCM_DATA_BUS_8_BITS);
    
    _lcm_set_panel_interface_type(LCM_PANEL_INTEFACE_18_BITS);
    _lcm_set_mcu_interface_mode(LCM_MCU_INTERFACE_8080);

    // reset lcm
    {
        LCM_REG_ENABLE_SET_RST_SET(); // power on reset   
        delay_ms_enable();
        delay_ms(100);
        delay_ms_disable();
        LCM_REG_ENABLE_SET_RST_CLR();
        LCM_REG_ENABLE_SET_LCM_En(1);
        LCM_REG_ENABLE_SET_BLCTRL_SET(1);
    }
 
    struct display_driver *display_drv = (struct display_driver *)framework_drv_get_drv(&core_d->pin_ctx);
//    DSG("display_drv->core_dev->name=%s", display_drv->core_dev->name);
    
    // read display ID
    if ((display_drv) && (display_drv->panel))
    {
        display_drv->display_id = display_drv->panel->read_did(core_d);
        DSG("display_drv->display_id=%x", display_drv->display_id);
        display_drv->panel->init(core_d);
    }
 
    return 0;
}

static int kdp520_lcm_power(struct core_device *core_d, BOOL on)
{
    return 0;
}

static int kdp520_lcm_start(struct core_device *core_d)
{
    int ret = -KDP_FRAMEWORK_ERRNO_INVALA;

    struct display_driver *display_drv = (struct display_driver *)framework_drv_get_drv(&core_d->pin_ctx);
    
    //DSG("[%s]", __func__);
    //DSG("display_drv=%x", display_drv);
    //DSG("display_drv->panel=%x", display_drv->panel);
    
    if ((display_drv) && (display_drv->panel))
    {
        display_drv->panel->start(core_d);

        m_lcm_state = LCM_STATE_STARTED;

        set_thread_event(m_tid_lcm, FLAGS_LCM_START_EVT);
        
        ret = 0;
    }

    return ret;
}

static int kdp520_lcm_stop(struct core_device *core_d)
{
    int ret = -KDP_FRAMEWORK_ERRNO_INVALA;

    struct display_driver *display_drv = (struct display_driver *)framework_drv_get_drv(&core_d->pin_ctx);
    
    DSG("[%s]", __func__);
    
    if ((display_drv) && (display_drv->panel))
    {
        display_drv->panel->stop(core_d);

        m_lcm_state = LCM_STATE_STOPPED;

        ret = 0;
    }

    return ret;
}

//static int kdp520_lcm_set_pen(struct core_device *core_d, u8 r, u8 g, u8 b, unsigned int width)
static int kdp520_lcm_set_pen(struct core_device *core_d, u16 color, unsigned int width)
{
    int ret = -KDP_FRAMEWORK_ERRNO_INVALA;
    struct kdp520_lcm_context *ctx = framework_drv_get_drvdata(&core_d->pin_ctx);
    
    DSG("%s", __func__);

    if (core_d) {
        // ctx->pen_info.r = r;
        // ctx->pen_info.g = g;
        // ctx->pen_info.b = b;
        ctx->pen_info.color = color;        
        ctx->pen_info.width = width;
        ret = 0;
    } 

    return ret;
}

static int kdp520_lcm_draw_line(struct core_device *core_d, 
        u32 xs, u32 ys, u32 xe, u32 ye)
{
    int ret = -KDP_FRAMEWORK_ERRNO_INVALA;
    struct kdp520_lcm_context *ctx = framework_drv_get_drvdata(&core_d->pin_ctx);  
    struct display_driver *display_drv = (struct display_driver *)framework_drv_get_drv(&core_d->pin_ctx);
	
    if (core_d)
	{
        u32 x_pos, y_pos;
        u32 x_unit = 1;
        u32 y_unit = 1;
        u32 frame_buffer_addr = ctx->frame_buffer_addr;
        u32 frame_width = display_drv->vi_params.input_xres;
        u32 frame_height = display_drv->vi_params.input_yres;
		u16 color = ctx->pen_info.color;
		u16 pen_width = ctx->pen_info.width;
		
		if(xs == xe)
		{
			if(ys > ye)
			{
				ys ^= ye;
				ye ^= ys;
				ys ^= ye;
			}
			
			for(y_pos = ys; y_pos <= ye; y_pos += y_unit)
			{
				for(x_pos = xs; (x_pos < xs+pen_width) && (x_pos < frame_width); x_pos += x_unit)
				{
					_kdp520_lcm_draw_pixel(frame_buffer_addr, x_pos, y_pos, frame_width, color);
				}
			}
			ret = 0;
		}
		else if(ys == ye)
		{
			if(xs > xe)
			{
				xs ^= xe;
				xe ^= xs;
				xs ^= xe;
			}
			
			for(x_pos = xs; x_pos <= xe; x_pos += x_unit)
			{
				for(y_pos = ys; (y_pos < ys+pen_width) && (y_pos < frame_height); y_pos += y_unit)
				{
					_kdp520_lcm_draw_pixel(frame_buffer_addr, x_pos, y_pos, frame_width, color);
				}
			}
			ret = 0;
		}
    } 
    
    return ret;
}
		
static int kdp520_lcm_draw_rect(struct core_device *core_d, 
		u32 org_x, u32 org_y, u32 width, u32 height)
{
	int ret = -KDP_FRAMEWORK_ERRNO_INVALA;
	struct kdp520_lcm_context *ctx = framework_drv_get_drvdata(&core_d->pin_ctx);  
    struct display_driver *display_drv = (struct display_driver *)framework_drv_get_drv(&core_d->pin_ctx);
	
	if (core_d) {

		u32 x_pos, y_pos;		 
		u32 x_unit = 1;
		u32 y_unit = 1;
		u32 frame_no = 0;
		u32 top = org_y;
		u32 left = org_x;
		u32 right = org_x + width;
		u32 bottom = top + height;
		u32 frame_width = display_drv->vi_params.input_xres;
		u32 frame_height = display_drv->vi_params.input_yres;
		u32 frame_buffer_addr = ctx->frame_buffer_addr;
		u32 border_size = ctx->pen_info.width;
		u32 left_border = left + border_size;
		u32 right_border = right - border_size;
		u32 top_lower = top + border_size;
		u32 bottom_upper = bottom - border_size;  
		u16 color = ctx->pen_info.color;

		//DSG("ctx->frame_buffer[frame_no]=%d %d %x", 
		//ctx->frame_width[frame_no], ctx->frame_height[frame_no], ctx->frame_buffer[frame_no]);
		
		for (y_pos = top; y_pos < bottom; y_pos += y_unit)
		{
			if ((y_pos >= top && y_pos < top_lower) || 
				(y_pos >= bottom_upper))
			{
				for (x_pos = org_x; x_pos < right; x_pos += x_unit)
				{
					//DSG("x_pos=%d y_pos=%d", x_pos, y_pos);					 
					_kdp520_lcm_draw_pixel(frame_buffer_addr, x_pos, y_pos, frame_width, color);
				}
			}
			else
			{
				for (x_pos = left; x_pos < left_border; x_pos += x_unit)
				{
					//DSG("x_pos=%d y_pos=%d", x_pos, y_pos);					 
					_kdp520_lcm_draw_pixel(frame_buffer_addr, x_pos, y_pos, frame_width, color);
				}
	   
				for (x_pos = right_border; x_pos < right; x_pos += x_unit)
				{
					//DSG("x_pos=%d y_pos=%d", x_pos, y_pos);					 
					_kdp520_lcm_draw_pixel(frame_buffer_addr, x_pos, y_pos, frame_width, color);
				}
			}
		}

		ret = 0;
	} 
	
	return ret;
}

static int kdp520_lcm_fill_rect(struct core_device *core_d, 
		u32 org_x, u32 org_y, u32 width, u32 height)
{
	int ret = -KDP_FRAMEWORK_ERRNO_INVALA;
	struct kdp520_lcm_context *ctx = framework_drv_get_drvdata(&core_d->pin_ctx);  
	struct display_driver *display_drv = (struct display_driver *)framework_drv_get_drv(&core_d->pin_ctx);
	
	if (core_d)
	{
		u32 x_pos, y_pos;
		u32 frame_buffer_addr = ctx->frame_buffer_addr;
		u32 frame_width = display_drv->vi_params.input_xres;
		u32 frame_height = display_drv->vi_params.input_yres;
		u16 color = ctx->pen_info.color;

		if((org_x > frame_width)
			|| (org_y > frame_height)
			|| ((org_x + width) > frame_width)
			|| ((org_y + height) > frame_height))
		{
			return ret;
		}
		
		for(y_pos=org_y; y_pos<(org_y+height); y_pos++)
		{
			for(x_pos=org_x; x_pos<(org_x+width); x_pos++)
			{
				_kdp520_lcm_draw_pixel(frame_buffer_addr, x_pos, y_pos, frame_width, color);
			}
		}
	} 
	
	return ret;
}

static int kdp520_lcm_draw_bitmap(struct core_device *core_d, 
		u32 org_x, u32 org_y, u32 width, u32 height, void *buf)
{
	int ret = -KDP_FRAMEWORK_ERRNO_INVALA;
	struct kdp520_lcm_context *ctx = framework_drv_get_drvdata(&core_d->pin_ctx);  
	struct display_driver *display_drv = (struct display_driver *)framework_drv_get_drv(&core_d->pin_ctx);
	
	if (core_d)
	{
		u32 x_pos, y_pos;
		u32 x_unit = 1;
		u32 y_unit = 1;
		u32 frame_buffer_addr = ctx->frame_buffer_addr;
		u32 frame_width = display_drv->vi_params.input_xres;
		u32 frame_height = display_drv->vi_params.input_yres;
		u8 bpp = 2;

		if((org_x > frame_width)
			|| (org_y > frame_height)
			|| ((org_x + width) > frame_width)
			|| ((org_y + height) > frame_height))
		{
			return ret;
		}
		
		for(y_pos=org_y; y_pos<(org_y+height); y_pos++)
		{
			memcpy((void *)(frame_buffer_addr+((y_pos*frame_width+org_x)<<1)), (void *)buf, width*bpp);
		}
	} 
	
	return ret;
}

static struct display_driver_ops kdp520_lcm_ops = {
    
    .wait_ready = _lcm_wait_ready,
    .write_cmd  = _lcm_write_cmd,
    .write_data = _lcm_write_data,
    .read_data  = _lcm_read_data,
};

extern struct core_device kdp520_lcm;
struct display_driver kdp520_lcm_driver = {
    .driver       = {
        .name     = "kdp520_lcm",
    },
    .probe        = kdp520_lcm_probe,
    .remove       = kdp520_lcm_remove,
    .core_dev     = &kdp520_lcm,
    
    .power_mgr    = {
        .power    = kdp520_lcm_power,
        .suspend  = NULL,
        .resume   = NULL,
    },
    
    .ctrller_ops  = (void*)&kdp520_lcm_ops,
    .attach_panel = kdp520_lcm_attach_panel,
    .set_params   = kdp520_lcm_set_params,
    .set_camera   = kdp520_lcm_set_camera,
    .init         = kdp520_lcm_init,
    .start        = kdp520_lcm_start,
    .stop         = kdp520_lcm_stop,
    .pip_test     = NULL,
    .set_pen      = kdp520_lcm_set_pen,
    .draw_rect    = kdp520_lcm_draw_rect,
    .draw_line    = kdp520_lcm_draw_line,
    .fill_rect    = kdp520_lcm_fill_rect,
    .draw_bitmap  = kdp520_lcm_draw_bitmap,
}; 
KDP_CORE_DRIVER_SETUP_2(kdp520_lcm_driver, 5);

#endif
#endif
