/*
 * KDP Camera driver
 *
 * Copyright (C) 2019 Kneron, Inc. All rights reserved.
 *
 */

#include "config/board_kdp520.h"
#include <string.h> 
#include <stdio.h> 
#include <stdlib.h>
#include <cmsis_os2.h>
#include "media/kdp_camera.h"

struct kdp_camera_s {
    int             inuse;
    struct cam_ops  *ops;
} camera_s[IMGSRC_NUM];

#ifdef KL520
extern int kdp_camera_kl520_init(void);
#endif

int kdp_camera_init(void)
{
#ifdef KL520
    kdp_camera_kl520_init();
#endif
    return 0;
}

int kdp_camera_open(int cam_idx)
{
    if (cam_idx >= IMGSRC_NUM)
        return -1;
    
    if (camera_s[cam_idx].ops->open == NULL)
        return -1;
        
    return camera_s[cam_idx].ops->open(cam_idx);
}

int kdp_camera_close(int cam_idx)
{
    if (cam_idx >= IMGSRC_NUM)
        return -1;
    
    if (camera_s[cam_idx].ops->close == NULL)
        return -1;
        
    camera_s[cam_idx].ops->close(cam_idx);
    return 0;
}

int kdp_camera_query_capability(int cam_idx, struct cam_capability *cap)
{
    if (cam_idx >= IMGSRC_NUM)
        return -1;
    
    if (camera_s[cam_idx].ops->query_capability == NULL)
        return -1;
        
    return camera_s[cam_idx].ops->query_capability(cam_idx, cap);
}

int kdp_camera_get_device_info(int cam_idx, struct cam_capability *cap)
{
    return kdp_camera_query_capability(cam_idx, cap);
}

int kdp_camera_set_frame_format(int cam_idx, struct cam_format *format)
{
    if (cam_idx >= IMGSRC_NUM)
        return -1;
    
    if (camera_s[cam_idx].ops->set_format == NULL)
        return -1;
        
    return camera_s[cam_idx].ops->set_format(cam_idx, format);
}

int kdp_camera_get_frame_format(int cam_idx, struct cam_format *format)
{
    if (cam_idx >= IMGSRC_NUM)
        return -1;
    
    if (camera_s[cam_idx].ops->get_format == NULL)
        return -1;
        
    return camera_s[cam_idx].ops->get_format(cam_idx, format);
}

int kdp_camera_buffer_init(int cam_idx)
{
    if (cam_idx >= IMGSRC_NUM)
        return -1;
    
    if (camera_s[cam_idx].ops->buffer_init == NULL)
        return -1;
        
    return camera_s[cam_idx].ops->buffer_init(cam_idx);
}

int kdp_camera_start(int cam_idx)
{
    if (cam_idx >= IMGSRC_NUM)
        return -1;
    
    if (camera_s[cam_idx].ops->start_capture == NULL)
        return -1;
        
    return camera_s[cam_idx].ops->start_capture(cam_idx);
}

int kdp_camera_stop(int cam_idx)
{
    if (cam_idx >= IMGSRC_NUM)
        return -1;
    
    if (camera_s[cam_idx].ops->stop_capture == NULL)
        return -1;
        
    return camera_s[cam_idx].ops->stop_capture(cam_idx);
}

int kdp_camera_buffer_prepare(int cam_idx)
{
    if (cam_idx >= IMGSRC_NUM)
        return -1;
    
    if (camera_s[cam_idx].ops->buffer_prepare == NULL)
        return -1;
        
    return camera_s[cam_idx].ops->buffer_prepare(cam_idx);
}

int kdp_camera_buffer_capture(int cam_idx, int *addr, int *size)
{
    if (cam_idx >= IMGSRC_NUM)
        return -1;
    
    if (camera_s[cam_idx].ops->buffer_capture == NULL)
        return -1;
        
    return camera_s[cam_idx].ops->buffer_capture(cam_idx, addr, size);
}

int kdp_camera_stream_on(int cam_idx)
{
    if (cam_idx >= IMGSRC_NUM)
        return -1;
    
    if (camera_s[cam_idx].ops->stream_on == NULL)
        return -1;
        
    return camera_s[cam_idx].ops->stream_on(cam_idx);
}

int kdp_camera_stream_off(int cam_idx)
{
    if (cam_idx >= IMGSRC_NUM)
        return -1;
    
    if (camera_s[cam_idx].ops->stream_off == NULL)
        return -1;
        
    return camera_s[cam_idx].ops->stream_off(cam_idx);
}

int kdp_camera_controller_register(int cam_idx, struct cam_ops *cam_ops_p)
{
    if (cam_idx >= IMGSRC_NUM || cam_ops_p == NULL)
        return -1;

    if (camera_s[cam_idx].inuse)
        return -1;

    camera_s[cam_idx].ops = cam_ops_p;
    camera_s[cam_idx].inuse = 1;

    return 0;
}

int kdp_camera_controller_unregister(int cam_idx, struct cam_ops *cam_ops_p)
{
    if (cam_idx >= IMGSRC_NUM || cam_ops_p == NULL)
        return -1;

    if (!camera_s[cam_idx].inuse)
        return -1;

    camera_s[cam_idx].ops = NULL;
    camera_s[cam_idx].inuse = 0;

    return 0;
}
