/*
 * KDP Sensor driver
 *
 * Copyright (C) 2019 Kneron, Inc. All rights reserved.
 *
 */

#include "config/board_kdp520.h"
#include <string.h> 
#include <stdio.h> 
#include <stdlib.h>
#include <cmsis_os2.h>
#include "media/kdp_sensor.h"

#define SENSOR_TYPE_MAX         (SENSOR_TYPE_SC132GS + 1)

struct kdp_sensor_s {
    int                 inuse;
    struct sensor_ops   *ops;
} sensor_s[SENSOR_TYPE_MAX];


int kdp_sensor_s_power(int sensor_idx, int on)
{
    if (sensor_idx >= SENSOR_TYPE_MAX)
        return -1;
    
    if (sensor_s[sensor_idx].ops->s_power == NULL)
        return -1;
        
    return sensor_s[sensor_idx].ops->s_power(sensor_idx, on);
}

int kdp_sensor_reset(int sensor_idx)
{
    if (sensor_idx >= SENSOR_TYPE_MAX)
        return -1;
    
    if (sensor_s[sensor_idx].ops->reset == NULL)
        return -1;
        
    return sensor_s[sensor_idx].ops->reset(sensor_idx);
}

int kdp_sensor_s_stream(int sensor_idx, int enable)
{
    if (sensor_idx >= SENSOR_TYPE_MAX)
        return -1;
    
    if (sensor_s[sensor_idx].ops->s_stream == NULL)
        return -1;
        
    return sensor_s[sensor_idx].ops->s_stream(sensor_idx, enable);
}

int kdp_sensor_enum_fmt(int sensor_idx, unsigned int index, unsigned int *fourcc)
{
    if (sensor_idx >= SENSOR_TYPE_MAX)
        return -1;
    
    if (sensor_s[sensor_idx].ops->enum_fmt == NULL)
        return -1;
        
    return sensor_s[sensor_idx].ops->enum_fmt(sensor_idx, index, fourcc);
}

int kdp_sensor_set_fmt(int sensor_idx, struct cam_format *format)
{
    if (sensor_idx >= SENSOR_TYPE_MAX)
        return -1;
    
    if (sensor_s[sensor_idx].ops->set_fmt == NULL)
        return -1;
        
    return sensor_s[sensor_idx].ops->set_fmt(sensor_idx, format);
}

int kdp_sensor_get_fmt(int sensor_idx, struct cam_format *format)
{
    if (sensor_idx >= SENSOR_TYPE_MAX)
        return -1;
    
    if (sensor_s[sensor_idx].ops->get_fmt == NULL)
        return -1;
        
    return sensor_s[sensor_idx].ops->get_fmt(sensor_idx, format);
}

int kdp_sensor_register(int sensor_idx, struct sensor_ops *sensor_ops_p)
{
    if (sensor_idx >= SENSOR_TYPE_MAX || sensor_ops_p == NULL)
        return -1;

    if (sensor_s[sensor_idx].inuse)
        return -1;

    sensor_s[sensor_idx].ops = sensor_ops_p;
    sensor_s[sensor_idx].inuse = 1;

    return 0;
}

int kdp_sensor_unregister(int sensor_idx, struct sensor_ops *sensor_ops_p)
{
    if (sensor_idx >= SENSOR_TYPE_MAX || sensor_ops_p == NULL)
        return -1;

    if (!sensor_s[sensor_idx].inuse)
        return -1;

    sensor_s[sensor_idx].ops = NULL;
    sensor_s[sensor_idx].inuse = 0;

    return 0;
}
