/*
 * @name : sensor.c
 * @brief : sensor driver interface for Mozart
 *
 * Copyright (C) 2019 Kneron, Inc. All rights reserved.
 *
 */
#ifndef USE_KDP_CAMERA

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "framework/utils.h"
#include "framework/init.h"
#include "framework/framework_driver.h"
#include "framework/framework_errno.h"
#include "media/camera/sensor.h"
#include "dbg.h"


static int _sensor_drv_add(struct driver_context *dev_drv, struct pin_context *pin_ctx)
{
    struct sensor_driver *drv = to_sensor_driver(dev_drv);
    struct sensor_device *dev = to_sensor_device(pin_ctx);
    int ret = -KDP_FRAMEWORK_ERRNO_CTX;

    if (drv->probe)
        ret = drv->probe(dev);

    return ret;
}

static int _sensor_drv_del(struct driver_context *dev_drv, struct pin_context *pin_ctx)
{
    struct sensor_driver *drv = to_sensor_driver(dev_drv);
    struct sensor_device *dev = to_sensor_device(pin_ctx);
    int ret = -KDP_FRAMEWORK_ERRNO_CTX;

    if (drv->remove)
        ret = drv->remove(dev);

    return ret;
}

int sensor_driver_register(struct sensor_driver *drv)
{
    drv->driver.add = _sensor_drv_add;
    drv->driver.del = _sensor_drv_del;

    return driver_register(&drv->driver, &(drv->core_dev->pin_ctx));
}

void sensor_driver_unregister(struct sensor_driver *driver)
{
    driver_unregister(&driver->driver);
}
#endif
