/* -------------------------------------------------------------------------- 
 * @file ddr.c 
 *      DDR init function for different module
 *---------------------------------------------------------------------------*/

#include "kneron_mozart.h" 
#include "kdp_uart.h"
#include "io.h"
#include "ddr.h"
#include "scu_extreg.h"
#include "delay.h"
#include "DrvPWMTMR010.h"
#include "scu_reg.h"
#include "dbg.h"

#define DDR_BASE                                        DDRC_FTDDR3030_PA_BASE
#define DDR_REG_MCSR                                    (DDR_BASE + 0x0004)

/* Memory Controller State Control (Offset: 0x0004) */
#define DDR_REG_MCSR_EXIT_SELF_REFRESH                  BIT3
#define DDR_REG_MCSR_SELF_REFRESH                       BIT2
#define DDR_REG_MCSR_INIT_CMD                           BIT0

#ifdef KL520

#define DDR_DORMANT_BOOTUP      (SCU_REG_BTUP_STS_RTC_BTUPTS | SCU_REG_BTUP_STS_SMR)

extern u32 bootup_status;

void ddr_self_refresh_enter(void)
{
    masked_outw(DDR_REG_MCSR, DDR_REG_MCSR_SELF_REFRESH, DDR_REG_MCSR_SELF_REFRESH);
}

void ddr_self_refresh_exit(void)
{
    masked_outw(DDR_REG_MCSR, DDR_REG_MCSR_EXIT_SELF_REFRESH, DDR_REG_MCSR_EXIT_SELF_REFRESH);
}

void ddr_wakeup(void)
{   
    delay_us(10);
    masked_outw(SCU_EXTREG_DDR_CTRL,
                (SCU_EXTREG_DDR_CTRL_wakeup | SCU_EXTREG_DDR_CTRL_SELFBIAS),
                (SCU_EXTREG_DDR_CTRL_wakeup | SCU_EXTREG_DDR_CTRL_SELFBIAS));    
    delay_us(10);
}

void ddr_init(enum ddr_init_mode mode)
{       
    int val;

	if((bootup_status & DDR_DORMANT_BOOTUP) == DDR_DORMANT_BOOTUP)
		mode = DDR_INIT_ALL_EXIT_SELF_REFRESH;

    ddr_wakeup();
    if (DDR_INIT_WAKEUP_ONLY == mode)
        return;

    outw(DDR_BASE + 0x100 , 0x000206C3);
    outw(DDR_BASE + 0x104 , 0x0000ff00);
    outw(DDR_BASE + 0x108 , 0x000000ff);

    outw(DDR_BASE + 0x04 , 0x00000000);
    outw(DDR_BASE + 0x08 , 0x00041c70);
    outw(DDR_BASE + 0x0C , 0x00000018);

    outw(DDR_BASE + 0x10 , 0x60007272);
    outw(DDR_BASE + 0x14 , 0x170D0F0B);
    outw(DDR_BASE + 0x18 , 0x21611323);
    outw(DDR_BASE + 0x1C , 0x00002041);

    outw(DDR_BASE + 0x20 , 0x00000f02);
    outw(DDR_BASE + 0x24 , 0x77777777);
    outw(DDR_BASE + 0x130 , 0x77777777);
	outw(DDR_BASE + 0x28 , /*0x00000a29*/1);

    outw(DDR_BASE + 0x30 , 0x90000000);
    outw(DDR_BASE + 0x34 , 0x03030303);
    outw(DDR_BASE + 0x38 , 0x03030303);
    outw(DDR_BASE + 0x3C , 0x00030011);

    outw(DDR_BASE + 0x48 , 0x00001002);

	outw(DDR_BASE + 0x70 , /*0x000000ff*/0x0);
    //outw(DDR_BASE + 0x74 , 0x44444444);
    outw(DDR_BASE + 0x78 , 0xEEEEEEEE);

    outw(DDR_BASE + 0xA0, 0x03080303);

    outw(DDR_BASE + 0xA8 , 0x0000D000);
    outw(DDR_BASE + 0xAC , 0x00000100);

    outw(DDR_BASE + 0x10C , 0x00000000);
    outw(DDR_BASE + 0x110 , 0x00000000);
    outw(DDR_BASE + 0x114 , 0x00000000);
    outw(DDR_BASE + 0x11C , 0x00000000);
    outw(DDR_BASE + 0x138 , 0x00000005);
    
    delay_us(50);

    outw(DDR_BASE + 0x00 , 0x080b9d03);
    
    if (DDR_INIT_ALL == mode)
        outw(DDR_REG_MCSR, DDR_REG_MCSR_INIT_CMD);
    else if (DDR_INIT_ALL_EXIT_SELF_REFRESH == mode)
        outw(DDR_REG_MCSR, DDR_REG_MCSR_INIT_CMD | DDR_REG_MCSR_EXIT_SELF_REFRESH);
    
    do {
        val = inw(DDR_BASE + 0x04);
    } while ((val & 0x100) == 0);
    
	// Reset Dphy after FTDDR3030 init OK
    SCU_EXTREG_DDR_CTRL_SET_Dphy_resetn(1);
    delay_us(30);
    SCU_EXTREG_DDR_CTRL_SET_Dphy_resetn(0);
    delay_us(500);
}

void ddr_init_chip_0429(void)
{   
    //DDR test code, fail
    int val;
    int i= 50000;
    outw(DDR_BASE + 0x100 , 0x000206C3);
    outw(DDR_BASE + 0x104 , 0x0000ff00);
    outw(DDR_BASE + 0x108 , 0x000000ff);

    outw(DDR_BASE + 0x04 , 0x00000000);
    outw(DDR_BASE + 0x08 , 0x00041c70);
    outw(DDR_BASE + 0x0C , 0x00000018);

    outw(DDR_BASE + 0x10 , 0x60007272);
    outw(DDR_BASE + 0x14 , 0x170D0F0B);
    outw(DDR_BASE + 0x18 , 0x21611323);
    outw(DDR_BASE + 0x1C , 0x00002041);

    outw(DDR_BASE + 0x20 , 0x00000f02);
    outw(DDR_BASE + 0x24 , 0x77777777);
    outw(DDR_BASE + 0x130 , 0x77777777);
    outw(DDR_BASE + 0x28 , 0x00000001);

    outw(DDR_BASE + 0x30 , 0x90000000);
    outw(DDR_BASE + 0x34 , 0x03030303);
    outw(DDR_BASE + 0x38 , 0x03030303);
    outw(DDR_BASE + 0x3C , 0x00030011);

    outw(DDR_BASE + 0x48 , 0x00000002);

    outw(DDR_BASE + 0x70 , 0x00000000);
    //outw(DDR_BASE + 0x74 , 0x44444444);
    outw(DDR_BASE + 0x78 , 0xEEEEEEEE);

    outw(DDR_BASE + 0xA8 , 0x0000D000);
    outw(DDR_BASE + 0xAC , 0x00000100);

    outw(DDR_BASE + 0x10C , 0x00000000);
    outw(DDR_BASE + 0x110 , 0x00000000);
    outw(DDR_BASE + 0x114 , 0x00000000);
    outw(DDR_BASE + 0x11C , 0x00000000);
    outw(DDR_BASE + 0x138 , 0x00000005);

    i = 5000;
    while(i--);

    outw(DDR_BASE + 0x00 , 0x080b9d03);
    outw(DDR_BASE + 0x04 , 0x00000001);
    do {
        val = inw(DDR_BASE + 0x04);
    } while ((val & 0x100) == 0);
}


#endif



void ddr_init_v35_mipi(unsigned char ncpu_wakeup)
{
    int val;
    outw(SCU_FTSCU100_PA_BASE + 0x00,0x00010000);
    outw(SCU_FTSCU100_PA_BASE + 0x04,0x00030001);

    //SCU PWRCTRL checking
    outw(SCU_FTSCU100_PA_BASE + 0x08,(inw(SCU_FTSCU100_PA_BASE + 0x08)|0x01000700));
    do{// wait vcc_status
        val = inw(SCU_FTSCU100_PA_BASE + 0x48);
    }while((val & 0xFF) != 0x7);
    do{// wait smr_por_n
        val = inw(SCU_EXTREG_PA_BASE + 0xB0);
    }while((val & 0xF0) != 0x70);
    
    //20181218 Add
    // Initial INTC
    outw(0xE000E284,0x00000001 << 8);
    outw(0xE000E104,0x00000001 << 8);
    // SCU Interrupt enable register, clear scu state
    outw(SCU_FTSCU100_PA_BASE + 0x24 , 0xffffffff);

    outw(0xC2380090,0x13000003); // MIPI
    outw(0xc238009c, 0xff); //DPI2AHB	 
    outw(SCU_EXTREG_PA_BASE + 0x80, (inw(SCU_EXTREG_PA_BASE + 0x80) & ~0x10000000)); //DDRC 
    int i= 50000;
    while(i--);

    outw(SCU_EXTREG_PA_BASE + 0x14, (inw(SCU_EXTREG_PA_BASE + 0x14) | 0x00400000)); // NCPU fclk src en setting
    outw(SCU_EXTREG_PA_BASE + 0x18, (inw(SCU_EXTREG_PA_BASE + 0x18) | 0x00800000)); // Enable npu clk_en
    outw(SCU_EXTREG_PA_BASE + 0x4c, (inw(SCU_EXTREG_PA_BASE + 0x4c) | 0x00800002)); // Enable PD_NPU_reset_n
    outw(SCU_EXTREG_PA_BASE + 0x4c, (inw(SCU_EXTREG_PA_BASE + 0x4c) | 0x00800004)); // Enable NPU_reset_n
    outw(SCU_EXTREG_PA_BASE + 0x4c, (inw(SCU_EXTREG_PA_BASE + 0x4c) | 0x00800001)); // Enable LCD reset release

    outw(SCU_EXTREG_PA_BASE + 0x80, (inw(SCU_EXTREG_PA_BASE + 0x80) | 0x10000000)); // wakeup DDRC 
    
    // NCPU code copy from SPI flash
    fLib_printf("Wake up NCPU\n");
    outw(SCU_EXTREG_PA_BASE + 0x68, (inw(SCU_EXTREG_PA_BASE + 0x68) | 0x00001000)); // wakeup NCPU
    outw(0xC2380044, 0x1F0000);

    int j= 50000;
    while(j--);
    
    // kneron DDR controller base: 0xC3200000 (v9)
#if (HAPS_ID == 3)
    outw(DDR_BASE + 0x00 , 0x08005e01);
#else    
    outw(DDR_BASE + 0x00 , 0x08005e02);//1207->0x08005e01
#endif    
    outw(DDR_BASE + 0x04 , 0x00000000);
    outw(DDR_BASE + 0x08 , 0x00020230);
    outw(DDR_BASE + 0x0C , 0x00000010);
    outw(DDR_BASE + 0x10 , 0x80006464);
    outw(DDR_BASE + 0x14 , 0x11030302);
    outw(DDR_BASE + 0x18 , 0x22352121);
    outw(DDR_BASE + 0x1C , 0x01002690);
    outw(DDR_BASE + 0x20 , 0x00002F02);
    outw(DDR_BASE + 0x24 , 0x0);
    outw(DDR_BASE + 0x28 , 0x0);
    outw(DDR_BASE + 0x30 , 0x160005E5);
    outw(DDR_BASE + 0x34 , 0x0F050506);
    outw(DDR_BASE + 0x38 , 0x05080305);
    outw(DDR_BASE + 0x3C , 0x00520012);
    outw(DDR_BASE + 0x40 , 0x0000AA00);
    outw(DDR_BASE + 0x48 , 0x0);
    outw(DDR_BASE + 0xA0 , 0x02830000);
    outw(DDR_BASE + 0xA4 , 0x0);
    outw(DDR_BASE + 0xA8 , 0x00003E80);
    outw(DDR_BASE + 0xAC , 0x00061A80);
    outw(DDR_BASE + 0x74 , 0x0);
    outw(DDR_BASE + 0x78 , 0xAA);
    outw(DDR_BASE + 0x04 , 0x00000001);

    do {// wait smr_por_n
        val = inw(DDR_BASE + 0x4);
    }while((val & 0x100) != 0x100);
    j = 5000;
    while (j--);
    fLib_printf("DDR Initial ok\n");
}
