#include <string.h>
#include <stdlib.h>

#include "kneron_mozart.h"
#include "io.h"

#include "kdp520_dma.h"
#include "kdp520_spi.h"
#include "kdp_uart.h"

#define kdp_printf      fLib_printf

#define DMA_CHANNEL_COUNT 8 
#define GET_MIN(x,y) ( x < y ? x: y )

#define DATA_BUF_SIZE		0x100//0x2000  

extern u32 rd_bl_len, wr_bl_len;

#ifdef WR_TEST
static char dma_buf1[DATA_BUF_SIZE/*+0x3*/];
static char dma_buf2[DATA_BUF_SIZE/*+0x3*/];
static char dma_buf3[DATA_BUF_SIZE/*+0x3*/];
static char dma_buf4[DATA_BUF_SIZE/*+0x3*/];
#endif // WR_TEST

u32 m_dma_channel = 0;

int dma_mem_2_mem_test(u32 *SrcAddr, u32 *DstAddr, u32 Size);


void kdp_ddr_dma_copy(unsigned int *src_ptr, unsigned int *dst_ptr, int size)
{
	int channel_num;
    kdp_dma_init(FALSE, FALSE, 0x0);	
	kdp_dma_clear_all_interrupt(); 
	kdp_dma_enable_dma_int(); //Enable DMA Interrupt					
	channel_num = kdp_dma_get_ch_num();
	kdp_printf("channel nums: %d\n", channel_num);
	
 
     m_dma_channel = 0;
     kdp_dma_reset_ch(m_dma_channel);
		 if(!dma_mem_2_mem_test(src_ptr, dst_ptr, size))
     {
        //fail("\rFail!\n");
			  kdp_printf("\r!!!!Fail!\n"); //legend
        return;
     }else{
         kdp_printf("dma_mem_2_mem_test pass for channel% d\n", m_dma_channel); //legend
     }
	

}


 int dma_mem_2_mem_test(u32 *SrcAddr, u32 *DstAddr, u32 Size)
{
    u32 i, SrcWidth, DstWidth, SrcSize;
//    u32 *TempAddr;

		//kdp_printf("1\n");
	kdp_printf("dma_mem_2_mem_test for DMA Channel %d\n", m_dma_channel);  
 

                
    SrcWidth = 2;
    DstWidth = 2;
    SrcSize = 4;
    
    memset(DstAddr, 0, Size);

    kdp_dma_normal_mode(m_dma_channel, (u32)SrcAddr, (u32)DstAddr, Size, SrcWidth, DstWidth, SrcSize, 0, 0, 0, 0, 0);             
                    kdp_dma_enable_ch(m_dma_channel);
    kdp_dma_wait_dma_int(m_dma_channel);
    kdp_dma_disable_ch(m_dma_channel);  

    if(memcmp(SrcAddr, DstAddr, Size) != 0)
    {
        for(i = 0; i < (Size/4); i++)
        {
          
            if(DstAddr[i] != SrcAddr[i])
            {
                              kdp_printf("======!!!!!!==============>DstAddr[%0.8X] = %0.8X, should be %0.8X\n", i, DstAddr[i], SrcAddr[i]);
                return FALSE;
            }
        }
    }
                

             
    return TRUE;
}

void kdp_flash_to_ddr_dma_copy(u32 *src_addr, u32 *dst_addr, u32 size)
{
   
    dma_ch_csr csr;
	dma_ch_cfg	cfg;
	memset(&csr, 0x0, sizeof(dma_ch_csr));
	memset(&cfg, 0x0, sizeof(dma_ch_cfg));
    kdp_dma_init(FALSE, FALSE, 0x0);				
    kdp_dma_clear_all_interrupt();
	kdp_dma_enable_dma_int(); //Enable DMA Interrupt					
	kdp_dma_reset_ch(0);
    csr.dst_ctrl = AHBDMA_DstInc;
	csr.src_ctrl = AHBDMA_SrcFix;
	csr.mode = AHBDMA_HwHandShakeMode;
	csr.dst_width = AHBDMA_DstWidth_DWord;
	csr.src_width = AHBDMA_SrcWidth_DWord;		
	cfg.dst_rs = 0;
	cfg.dst_he = 0;
	cfg.src_rs = 11; //SPI_DMA_REQ
	cfg.src_he = 1;
	cfg.int_abt_msk = 0;	
	cfg.int_err_msk = 0;
	cfg.int_tc_msk = 0;
    
    kdp_dma_set_ch_cfg(0, csr);
	kdp_dma_set_ch_cn_cfg(0, cfg);
	kdp_dma_ch_data_ctrl(0, SPI020REG_DATAPORT, (u32)dst_addr, size>>2);		
	kdp_dma_enable_ch(0);
	kdp_dma_wait_dma_int(0);		
	kdp_dma_disable_ch(0);  
	spi020_dma_read_stop();
    kdp_dma_disable_dma_int(); //Disable DMA Interrupt
    //kdp_printf("====== finish dma_flash_to_ddr_test ========\n");
    

}

void kdp_ddr_to_flash_dma_copy(u32 src_addr, u32 dst_addr, u32 size){
    
    kdp_printf("=======kdp_ddr_to_flash_dma_copy===========\n");
//    u32 i;

	u32 write_len=size;
    
	u32 access_byte;
	u32 offset=0;
	dma_ch_csr csr;
	dma_ch_cfg	cfg;
	
	
    memset(&csr, 0x0, sizeof(dma_ch_csr));
	memset(&cfg, 0x0, sizeof(dma_ch_cfg));
	
 
    kdp_printf("before spi020_flash_64kErase\n");
    spi020_flash_64kErase((u32)dst_addr+offset);
    kdp_printf("after spi020_flash_64kErase\n");
	kdp_dma_init(FALSE, FALSE, 0x0);				
    kdp_dma_clear_all_interrupt();
	kdp_dma_enable_dma_int(); //Enable DMA Interrupt		
	kdp_dma_reset_ch(0);	
	csr.dst_ctrl = AHBDMA_DstFix;
	csr.src_ctrl = AHBDMA_SrcInc;
	csr.mode = AHBDMA_HwHandShakeMode;
	csr.dst_width = AHBDMA_DstWidth_DWord;
	csr.src_width = AHBDMA_SrcWidth_DWord;		
	cfg.dst_rs = SPI_DMA_REQ;
	cfg.dst_he = 1;
    cfg.src_rs = 0;
    cfg.src_he = 0;
    cfg.int_abt_msk = 0;	
	cfg.int_err_msk = 0;
	cfg.int_tc_msk = 0;
		
	kdp_dma_set_ch_cfg(0, csr);
	kdp_dma_set_ch_cn_cfg(0, cfg);
	
	while(write_len>0)
	{	
		access_byte=GET_MIN(write_len, spi020_txfifo_depth());
		spi020_flash_write(FLASH_DMA_WRITE, (u32)dst_addr+offset, access_byte, NULL);		
		kdp_dma_ch_data_ctrl(0, ((u32)src_addr)+offset, SPI020REG_DATAPORT, access_byte>>2);		
		kdp_dma_enable_ch(0);
		kdp_dma_wait_dma_int(0);		
		kdp_dma_disable_ch(0);  
		spi020_dma_write_stop();
		write_len-=access_byte;
		offset+=access_byte;
	}	
	
	kdp_dma_disable_dma_int(); //Disable DMA Interrupt
}
