#include <stdio.h>
#include <string.h>
#include "Driver_Common.h"
#include "io.h"
#include "kneron_mozart.h"
#include "Driver_USART.h"
#include "kdp520_adc.h"
#include "dbg.h"
#include "uart_com.h"

#define NUM_TDC         4
#define SCANMODE        SCANMODE_CONT

kdp_adc_resource ADC_Resources = {
    ADC_FTTSC010_0_PA_BASE,
    ADC_FTADCC010_IRQ, 
};

void _kdp_adc_config(void)
{ 
	struct kdp_adc_regs *regs = (struct kdp_adc_regs *)ADC_Resources.io_base;

	u32 val;
	
//set ADC clock pre-scaler control register (offset 0x11C)		
	//outw(regs->prescal, val);	

//set ADC timing parameter register (offset 0x110)		
	//outw(regs->tparam, val);	

//set ADC sampling timing parameter control register (offset 0x114)		
	//outw(regs->smpr, val);		
	
//set ADC scan sequence register (offset 0x120)		
	//outw(regs->sqr, val);			

//set ADC interrupt enable register (offset 0x108)		
	//outw(regs->inten, val);
	
//set ADC control register (offset 0x100)	
		val = 0;
		val = ADC_EN | SWSTART | SCANMODE_CONT | SCAN_NUM(4);
		//DSG("val = 0x%x", val);
		outw(&regs->ctrl, val);
		//_print_adc_register(&ADC_Resources);
}
void kdp_adc_init(int mode)
{
    u32 i;
    u32 val = 0;
    char Ch;
    struct kdp_adc_regs *regs = (struct kdp_adc_regs *)ADC_Resources.io_base;
    
    //Enable ADCCLK in SCU base 0xC2380000
    //Set the 22th bit of the "Clock Enable Register1"(offset = 0x018) to 1
    val = inw(SCU_EXTREG_PA_BASE + 0x018);
    outw(SCU_EXTREG_PA_BASE + 0x018, (val | (1 << 22 )));
    
    DSG("[ADC Test]");
    _kdp_adc_config();
    
    // Wait for scanning done.
    for(i = 0; i < NUM_TDC; i++) {
        while(!(readl(&regs->intst) & CH_INTRSTS(i))) {}	
    }

    while(1)
    {
        DSG("AIN0: %d \r\nAIN1: %d \r\nAIN2: %d \r\nAIN3: %d \r\n ===\r\n",regs->data[0],regs->data[1],regs->data[2],regs->data[3]);
        DSG("press anykey to measureing, and 'q' to exit");
        //_print_adc_register(&ADC_Resources);
        Ch = uart_com_log_getc();
        if (Ch == 'q')
            break;
    }
		
}

int32_t kdp_adc_uninit(kdp_adc_resource *res)
{
	return 0;
}

void kdp_adc_rest(kdp_adc_resource *res)
{
	int val;
	struct kdp_adc_regs *regs = (struct kdp_adc_regs *)res->io_base;
	
	//disable 
	val = readl(&regs->ctrl);
	val &= ~ADC_EN; //clear enable bit 
	writel(val, &regs->ctrl);		
	
	//enable
	val |= ADC_EN; //clear enable bit 
	writel(val, &regs->ctrl);		
	
}

void kdp_adc_enable(kdp_adc_resource *res, int mode)
{
	int val;
	struct kdp_adc_regs *regs = (struct kdp_adc_regs *)res->io_base;
	
	val = readl(&regs->ctrl);
	val &= ~ADC_EN; //clear enable bit 
	val |= mode;
	writel(val, &regs->ctrl);		
}

static void kdp_adc_force_update_temp(kdp_adc_resource *res, int id)
{
		kdp_adc_enable(res, ADC_EN);
}

uint32_t kdp_adc_read(kdp_adc_resource *res, int id , unsigned long *temp)
{
    u32 val = 0;
    struct kdp_adc_regs *regs = (struct kdp_adc_regs *)res->io_base;
    
    if(SCANMODE == SCANMODE_SGL)
        kdp_adc_force_update_temp(res, id);
    
    val = readl(&regs->data[id]);
    
    DSG("print raw data: %d\n", val);
    /*
    if(val >= 374)
        *temp = (val - 374)/4;
    else
        *temp = (374 - val)/4;
    */
    
    return 0;
}

void _print_adc_register(kdp_adc_resource *res)
{
	u32 val = 0;
	struct kdp_adc_regs *regs = (struct kdp_adc_regs *)res->io_base;
	
	val = inw(&regs->prescal);	//0x11C
	DSG("0x%x = 0x%x", &regs->prescal, val);
	val = inw(&regs->tparam);	//0x110
	DSG("0x%x = 0x%x", &regs->tparam, val);
	val = inw(&regs->smpr);		//0x114
	DSG("0x%x = 0x%x", &regs->smpr, val);
	val = inw(&regs->sqr);			//0x120
	DSG("0x%x = 0x%x", &regs->sqr, val);
	val = inw(&regs->inten);		//0x108
	DSG("0x%x = 0x%x", &regs->inten, val);
	val = inw(&regs->ctrl);		//0x100
	DSG("0x%x = 0x%x", &regs->ctrl, val);
	
}
