#include "utility.h"
#include "kneron_mozart.h"
#include "io.h"
#include "kdp520_dma.h"

#define ASSERT(x)   do { \
                        if (!(x)) 	\
                            for (;;)	\
                                ; 		\
                    } while (0)
//#define CN_SIZE			(0x400000-1)
#define CN_SIZE			4080		//==> for test only

#define CPU_TO_AHB_ADDRSPACE 0x23
#define DMAC_FTDMAC020_PA_BASE_SCPU DMAC_FTDMAC020_PA_BASE
int burst_size_selection[] = {1, 4, 8, 16, 32, 64, 128, 256};

volatile kdp520_dma_reg *m_dma_reg = (kdp520_dma_reg *)DMAC_FTDMAC020_PA_BASE;
static volatile u32	__dma_int_occurred = 0, __dma_tc_int_occurred = 0;

void AHB_DMA_IRQHandler(void)
{
    //kdp_printf("NCPU_AHB_DMA_IRQHandler\n");
    u32 status;
    
    
    status = m_dma_reg->dma_int;
    
    if(status)
        __dma_int_occurred |= status;
  else
        return;
    
    outw(DMAC_FTDMAC020_PA_BASE_SCPU+DMA_INT_TC_CLR, status);
    outw(DMAC_FTDMAC020_PA_BASE_SCPU+DMA_INT_ERRABT_CLR, status);		 
}


void AHB_DMA_TC_IRQHandler(void)
{
    //kdp_printf("NCPU_AHB_DMA_TC_IRQHandler\n");
    u32 status;
    
    
    status = m_dma_reg->dma_int_tc;
    
    if(status)
        __dma_tc_int_occurred |= status;
    else
        return;
    
    outw(DMAC_FTDMAC020_PA_BASE_SCPU+DMA_INT_TC_CLR, status);	
}

int kdp_dma_is_ch_busy(s32 Channel)
{
    return ((m_dma_reg->dma_ch_busy >> Channel) & 0x1);
}

int kdp_dma_is_ch_enable(s32 Channel)
{
    return ((m_dma_reg->dma_ch_enable >> Channel) & 0x1);
}

int kdp_dma_get_busy_status(void)
{
    return m_dma_reg->dma_ch_busy;
}

int kdp_dma_get_enable_status(void)
{
    return m_dma_reg->dma_ch_enable;
}

u32 kdp_dma_get_int_status(void)
{
    return m_dma_reg->dma_int;
}



u32 kdp_dma_get_ch_int_status(s32 Channel)
{
    volatile u32 IntStatus = 0;
    
    if((m_dma_reg->dma_int >> Channel) & 0x01)
    {
        if((m_dma_reg->dma_int_tc >> Channel) & 0x01)
            IntStatus |= 1;
        if((m_dma_reg->dma_int_err >> Channel) & 0x01)
            IntStatus |= 2;
    }
        
    return IntStatus;
}


void kdp_dma_init(u32 M0_BigEndian, u32 M1_BigEndian, u32 Sync)
{		
    m_dma_reg->dma_csr = (M0_BigEndian ? DMA_CSR_M0ENDIAN : 0) | 
    (M1_BigEndian ? DMA_CSR_M1ENDIAN : 0) | DMA_CSR_DMACEN;
    
    m_dma_reg->dma_sync = Sync; 
}


void kdp_dma_enable_ch(s32 Channel)
{
    u32 reg;
        
    reg = *(u32 *)&m_dma_reg->dma_ch[Channel].csr;
    reg |= DMA_CSR_CH_ENABLE;
    *(u32 *)&m_dma_reg->dma_ch[Channel].csr = reg;
}

void kdp_dma_disable_ch(s32 Channel)
{
    u32 reg;
    
    reg = *(u32 *)&m_dma_reg->dma_ch[Channel].csr;
    reg &= ~DMA_CSR_CH_ENABLE;
    *(u32 *)&m_dma_reg->dma_ch[Channel].csr = reg;
}

void kdp_dma_enable(unsigned int value)
{
    unsigned int reg;
    
    value &= 0x1;	// check input
    
    reg = m_dma_reg->dma_csr;
    
    reg &= ~(0x1);
    reg |= value;
    
    m_dma_reg->dma_csr = reg;
}

void kdp_dma_clear_ch_int_status(s32 Channel)
{
    m_dma_reg->dma_int_tc_clr = 1 << Channel;
    m_dma_reg->dma_int_err_clr = 1 << Channel;
}


void kdp_dma_set_ch_cfg(s32 Channel, dma_ch_csr Csr)
{
    m_dma_reg->dma_ch[Channel].csr = Csr;
}

dma_ch_csr kdp_dma_get_ch_cfg(s32 Channel)
{
    return m_dma_reg->dma_ch[Channel].csr;
}


void kdp_dma_set_ch_cn_cfg(s32 Channel, dma_ch_cfg CnCfg)
{
    m_dma_reg->dma_ch[Channel].cfg = CnCfg;
}

dma_ch_cfg kdp_dma_get_ch_cn_cfg(s32 Channel)
{
    return m_dma_reg->dma_ch[Channel].cfg;
}

void kdp_dma_ch_int_mask(s32 Channel, dma_ch_cfg Mask)
{
    m_dma_reg->dma_ch[Channel].cfg = Mask;
}

void kdp_dma_ch_linklist(s32 Channel, dma_ch_llp LLP)
{
    m_dma_reg->dma_ch[Channel].llp = LLP;
}

void kdp_dma_ch_data_ctrl(s32 Channel, u32 SrcAddr, u32 DstAddr, u32 Size)
{
    m_dma_reg->dma_ch[Channel].src_addr = SrcAddr/*CPU_TO_AHB_ADDRSPACE(SrcAddr)*/;
    m_dma_reg->dma_ch[Channel].dst_addr = DstAddr/*CPU_TO_AHB_ADDRSPACE(DstAddr)*/;
    m_dma_reg->dma_ch[Channel].size = Size;
}

void kdp_dma_link_transfer(
u32 Channel,   // use which channel for AHB DMA, 0..7
u32 LinkAddr,  // Link-List address
u32 LLPCount,  // total link-list node
u32 SrcAddr,   // source begin address
u32 DstAddr,   // dest begin address
u32 Size,      // total bytes
u32 SrcWidth,  // source width 8/16/32 bits -> 0/1/2
u32 DstWidth,  // dest width 8/16/32 bits -> 0/1/2
u32 SrcSize,   // source burst size, How many "SrcWidth" will be transmmited at one times ?
u32 SrcCtrl,   // source address change : Inc/dec/fixed --> 0/1/2
u32 DstCtrl,   // dest address change : Inc/dec/fixed --> 0/1/2
u32 Priority,  // priority for this chaanel 0(low)/1/2/3(high)
u32 Mode,      // Normal/Hardwire,   0/1
int req)
{
    dma_ch DMAChannel;
    u32 LLPSize, i, offset;
    dma_lld *LLP;
    u32 Count = 0;
    u32 AHB_SrcAddr = /*CPU_TO_AHB_ADDRSPACE*/(SrcAddr);
    u32 AHB_DstAddr = /*CPU_TO_AHB_ADDRSPACE*/(DstAddr);
//	static u16 jeff_suck_dick=0;
    int burst_size = burst_size_selection[SrcSize];			// ref: [18:16] in Cn_CSR
        
    LLP = (dma_lld *)LinkAddr;
    *((unsigned int *)&DMAChannel.csr) = 0;		// clear value of csr;
     
    Size = Size / (1 << SrcWidth);		// how many unit want to transfer
    
    if(LLPCount && LinkAddr )
    {
			
    LLPSize = 240*2;
    if (req != 0)		// memory to memory does not have this restriction
    {
        LLPSize = RoundDown(CN_SIZE, burst_size);			// how many cycle a descriptor can transfer
        ASSERT((Size%burst_size)==0);
    }
    Count = LLPCount;		// how many link-list structure need to fill
   // ASSERT(Count<=LLPCount);

    // At last, 2 part
		
    if (Count > 0)
    {
			
            offset = 320 << 2;		
					 			
           for(i = 0; i < Count ;i++)
           {
						 
								if (SrcCtrl == 0)  // increase
									 LLP[i].src_addr = (u32)AHB_SrcAddr + ((i+1) * offset);
								else if(SrcCtrl==1) // decrease
									 LLP[i].src_addr = (u32)AHB_SrcAddr - ((i+1) * offset);
								else if(SrcCtrl==2)	// fixed
									 LLP[i].src_addr = (u32)AHB_SrcAddr;
					
								if(DstCtrl == 0)  
									 LLP[i].dst_addr = (u32)AHB_DstAddr + ((i+1) * offset);
								else if(DstCtrl == 1)	// Decrease
									 LLP[i].dst_addr = (u32)AHB_DstAddr - ((i+1) * offset);
								else if(DstCtrl == 2)
									 LLP[i].dst_addr = (u32)AHB_DstAddr;
							
								*((u32 *)&(LLP[i].llp)) = 0;
								LLP[i].llp.link_list_addr = /*CPU_TO_AHB_ADDRSPACE*/((u32)&LLP[i+1]) >> 2;
					
				 
								*((u32 *)&(LLP[i].llp_ctrl)) = 0;
								LLP[i].llp_ctrl.tc_msk = 1;
								LLP[i].llp_ctrl.src_width = SrcWidth; /* source transfer size */
								LLP[i].llp_ctrl.dst_width = DstWidth; /* destination transfer size */
								LLP[i].llp_ctrl.src_ctrl = SrcCtrl; /* source increment, decrement or fix */
								LLP[i].llp_ctrl.dst_ctrl = DstCtrl; /* destination increment, decrement or fix */
								LLP[i].llp_ctrl.src_sel = 0; /* source AHB master id */
								LLP[i].llp_ctrl.dst_sel = 0; /* destination AHB master id */
								LLP[i].llp_ctrl.ff_th = 2; //FIE7021 fifo threshold value = 4
								LLP[i].size = LLPSize;
								  
							Size -= LLPSize;
						
        }
           LLP[i-1].llp.link_list_addr = 0;
           LLP[i-1].llp_ctrl.tc_msk = 0;	// Enable tc status
           LLP[i-1].size = LLPSize;
           Size = LLPSize;
    }
    
#ifdef DBG_DMA
    for(i = 0; i < LLPCount;i++)
    {
        kdp_printf("src=%0.8X, dst=%0.8X, link=%0.8X, ctrl=%.8X\n", LLP[i].src_addr, LLP[i].dst_addr,
            *(u32 *)&(LLP[i].llp), *(u32 *)&(LLP[i].llp_ctrl));

    }
#endif
}
    /* program channel */
    kdp_dma_clear_ch_int_status(Channel);

    /* program channel CSR */
       DMAChannel.csr.ff_th = 2; //FIE7021 fifo threshold value = 4
       DMAChannel.csr.priority = Priority; /* priority */
       DMAChannel.csr.prot = 0; /* PROT 1-3 bits */
       DMAChannel.csr.src_size = SrcSize; /* source burst size */
    //FIE7021 this bit should be 0 when change other bits
       DMAChannel.csr.abt = 0; /* NOT transaction abort */
       DMAChannel.csr.src_width = SrcWidth; /* source transfer size */
       DMAChannel.csr.dst_width = DstWidth; /* destination transfer size */
       DMAChannel.csr.mode = Mode; /* Normal mode or Hardware handshake mode */
       DMAChannel.csr.src_ctrl = SrcCtrl; /* source increment, decrement or fix */
       DMAChannel.csr.dst_ctrl = DstCtrl; /* destination increment, decrement or fix */
       DMAChannel.csr.src_sel = 0; /* source AHB master id */
       DMAChannel.csr.dst_sel = 0; /* destination AHB master id */

    DMAChannel.csr.reserved1 = 0;
    DMAChannel.csr.reserved0 = 0;
    
    /* program channel CFG */
    DMAChannel.cfg.int_tc_msk = 0;				// Enable tc status
    DMAChannel.cfg.int_err_msk = 0;
    DMAChannel.cfg.src_rs = req;
    DMAChannel.cfg.dst_rs = req;
    DMAChannel.cfg.src_he = (SrcCtrl == 2);		// SrcCtrl==2 means fix source address ==> peripheral
    DMAChannel.cfg.dst_he = (DstCtrl == 2);
    DMAChannel.cfg.busy = 0;			
    DMAChannel.cfg.reserved1 = 0;		
    DMAChannel.cfg.llp_cnt = 0;			
    DMAChannel.cfg.reserved2 = 0;
    
   /* program channel llp */
   *((u32 *)&(DMAChannel.llp)) = 0;

    if (Count > 0)
    {
        DMAChannel.csr.tc_msk = 1; /* enable terminal count */
        DMAChannel.llp.link_list_addr = /*CPU_TO_AHB_ADDRSPACE*/((u32)&LLP[0]) >> 2;
    }
    else
    {
        DMAChannel.csr.tc_msk = 0; /* no LLP */
    }
    
    kdp_dma_set_ch_cfg(Channel, DMAChannel.csr);
    kdp_dma_ch_int_mask(Channel, DMAChannel.cfg);
    kdp_dma_ch_linklist(Channel, DMAChannel.llp);
       
       /* porgram address and size */
       kdp_dma_ch_data_ctrl(Channel, SrcAddr, DstAddr, 240*2);
}


void kdp_dma_linkmode(
u32 Channel,   // use which channel for AHB DMA, 0..7
u32 LinkAddr,  // Link-List address
u32 LLPCount,  // total link-list node
u32 SrcAddr,   // source begin address
u32 DstAddr,   // dest begin address
u32 Size,      // total bytes
u32 SrcWidth,  // source width 8/16/32 bits -> 0/1/2
u32 DstWidth,  // dest width 8/16/32 bits -> 0/1/2
u32 SrcSize,   // source burst size, How many "SrcWidth" will be transmmited at one times ?
u32 SrcCtrl,   // source address change : Inc/dec/fixed --> 0/1/2
u32 DstCtrl,   // dest address change : Inc/dec/fixed --> 0/1/2
u32 Priority,  // priority for this chaanel 0(low)/1/2/3(high)
u32 Mode,      // Normal/Hardwire,   0/1
int    req
)
{
    dma_ch DMAChannel;
    u32 LLPSize, i, offset;
    dma_lld *LLP;
    u32 Count = 0;
    u32 AHB_SrcAddr = /*CPU_TO_AHB_ADDRSPACE*/(SrcAddr);
    u32 AHB_DstAddr = /*CPU_TO_AHB_ADDRSPACE*/(DstAddr);
    int burst_size = burst_size_selection[SrcSize];			// ref: [18:16] in Cn_CSR
        
    LLP = (dma_lld *)LinkAddr;
    *((unsigned int *)&DMAChannel.csr) = 0;		// clear value of csr;
    
#ifdef DBG_DMA
    kdp_printf("Ch%d, Src=%08X, Dst=%08X, Size=%08X SrcWidth=%db, DstWidth=%db, SrcSize=%dB\n"
           "SrcCtrl=%s, DstCtrl=%s, Priority=%d, Mode=%s, LLPCnt = %d\n",
           Channel, SrcAddr, DstAddr, Size, 1 << (SrcWidth + 3), 1 << (DstWidth + 3), 
           ((SrcSize == 0) ? 1 : 1 << (SrcSize+1)), 
           ((SrcCtrl == 0) ? "Inc" : ((SrcCtrl == 1) ? "Dec" : "Fix")),
           ((DstCtrl == 0) ? "Inc" : ((DstCtrl == 1) ? "Dec" : "Fix")),
           Priority, ((Mode == 0) ? "Normal" : "HW"), LLPCount);
#endif	
    
    Size = Size / (1 << SrcWidth);		// how many unit want to transfer
    
    if(LLPCount && LinkAddr)
    {
    //LLPSize = CN_SIZE;
    if (req != 0)		// memory to memory does not have this restriction
    {
        LLPSize = RoundDown(CN_SIZE, burst_size);			// how many cycle a descriptor can transfer
        ASSERT((Size%burst_size)==0);
    }
    Count = divRoundDown(Size, LLPSize);		// how many link-list structure need to fill
    ASSERT(Count<=LLPCount);

    // At last, 2 part
    if (Count > 0)
    {
            offset = LLPSize << SrcWidth;						
           for(i = 0; i < Count ;i++)
           {
              if (SrcCtrl == 0)  // increase
                 LLP[i].src_addr = (u32)AHB_SrcAddr + ((i+1) * offset);
              else if(SrcCtrl==1) // decrease
                 LLP[i].src_addr = (u32)AHB_SrcAddr - ((i+1) * offset);
              else if(SrcCtrl==2)	// fixed
                 LLP[i].src_addr = (u32)AHB_SrcAddr;
        
              if(DstCtrl == 0)  
                 LLP[i].dst_addr = (u32)AHB_DstAddr + ((i+1) * offset);
              else if(DstCtrl == 1)	// Decrease
                 LLP[i].dst_addr = (u32)AHB_DstAddr - ((i+1) * offset);
              else if(DstCtrl == 2)
                 LLP[i].dst_addr = (u32)AHB_DstAddr;
            
              *((u32 *)&(LLP[i].llp)) = 0;
              LLP[i].llp.link_list_addr = /*CPU_TO_AHB_ADDRSPACE*/((u32)&LLP[i+1]) >> 2;
        
              *((u32 *)&(LLP[i].llp_ctrl)) = 0;
              LLP[i].llp_ctrl.tc_msk = 1;
              LLP[i].llp_ctrl.src_width = SrcWidth; /* source transfer size */
              LLP[i].llp_ctrl.dst_width = DstWidth; /* destination transfer size */
              LLP[i].llp_ctrl.src_ctrl = SrcCtrl; /* source increment, decrement or fix */
              LLP[i].llp_ctrl.dst_ctrl = DstCtrl; /* destination increment, decrement or fix */
              LLP[i].llp_ctrl.src_sel = 0; /* source AHB master id */
              LLP[i].llp_ctrl.dst_sel = 0; /* destination AHB master id */
              LLP[i].llp_ctrl.ff_th = 2; //FIE7021 fifo threshold value = 4
            LLP[i].size = LLPSize;
            Size -= LLPSize;
        }
           LLP[i-1].llp.link_list_addr = 0;
           LLP[i-1].llp_ctrl.tc_msk = 0;	// Enable tc status
           LLP[i-1].size = Size;
           Size = LLPSize;
    }
    
#ifdef DBG_DMA
    for(i = 0; i < LLPCount;i++)
    {
        kdp_printf("src=%0.8X, dst=%0.8X, link=%0.8X, ctrl=%.8X\n", LLP[i].src_addr, LLP[i].dst_addr,
            *(u32 *)&(LLP[i].llp), *(u32 *)&(LLP[i].llp_ctrl));

    }
#endif
}
    /* program channel */
    kdp_dma_clear_ch_int_status(Channel);

    /* program channel CSR */
       DMAChannel.csr.ff_th = 2; //FIE7021 fifo threshold value = 4
       DMAChannel.csr.priority = Priority; /* priority */
       DMAChannel.csr.prot = 0; /* PROT 1-3 bits */
       DMAChannel.csr.src_size = SrcSize; /* source burst size */
    //FIE7021 this bit should be 0 when change other bits
       DMAChannel.csr.abt = 0; /* NOT transaction abort */
       DMAChannel.csr.src_width = SrcWidth; /* source transfer size */
       DMAChannel.csr.dst_width = DstWidth; /* destination transfer size */
       DMAChannel.csr.mode = Mode; /* Normal mode or Hardware handshake mode */
       DMAChannel.csr.src_ctrl = SrcCtrl; /* source increment, decrement or fix */
       DMAChannel.csr.dst_ctrl = DstCtrl; /* destination increment, decrement or fix */
       DMAChannel.csr.src_sel = 0; /* source AHB master id */
       DMAChannel.csr.dst_sel = 0; /* destination AHB master id */

    DMAChannel.csr.reserved1 = 0;
    DMAChannel.csr.reserved0 = 0;
    
    /* program channel CFG */
    DMAChannel.cfg.int_tc_msk = 0;				// Enable tc status
    DMAChannel.cfg.int_err_msk = 0;
    DMAChannel.cfg.src_rs = req;
    DMAChannel.cfg.dst_rs = req;
    DMAChannel.cfg.src_he = (SrcCtrl == 2);		// SrcCtrl==2 means fix source address ==> peripheral
    DMAChannel.cfg.dst_he = (DstCtrl == 2);
    DMAChannel.cfg.busy = 0;			
    DMAChannel.cfg.reserved1 = 0;		
    DMAChannel.cfg.llp_cnt = 0;			
    DMAChannel.cfg.reserved2 = 0;
    
   /* program channel llp */
   *((u32 *)&(DMAChannel.llp)) = 0;

    if (Count > 0)
    {
        DMAChannel.csr.tc_msk = 1; /* enable terminal count */
        DMAChannel.llp.link_list_addr = /*CPU_TO_AHB_ADDRSPACE*/((u32)&LLP[0]) >> 2;
    }
    else
    {
        DMAChannel.csr.tc_msk = 0; /* no LLP */
    }
    
    kdp_dma_set_ch_cfg(Channel, DMAChannel.csr);
    kdp_dma_ch_int_mask(Channel, DMAChannel.cfg);
    kdp_dma_ch_linklist(Channel, DMAChannel.llp);
       
       /* porgram address and size */
       kdp_dma_ch_data_ctrl(Channel, SrcAddr, DstAddr, Size);
}


void kdp_dma_normal_mode(
u32 Channel,   // use which channel for AHB DMA, 0..7
u32 SrcAddr,   // source begin address
u32 DstAddr,   // dest begin address
u32 Size,      // total bytes
u32 SrcWidth,  // source width 8/16/32 bits -> 0/1/2
u32 DstWidth,  // dest width 8/16/32 bits -> 0/1/2
u32 SrcSize,   // source burst size, How many "SrcWidth" will be transmmited at one times ?
u32 SrcCtrl,   // source address change : Inc/dec/fixed --> 0/1/2
u32 DstCtrl,   // dest address change : Inc/dec/fixed --> 0/1/2
u32 Priority,  // priority for this chaanel 0(low)/1/2/3(high)
u32 Mode,      // Normal/Hardwire,   0/1
int    req
)
{	    
    kdp_dma_linkmode(
            Channel,   // use which channel for AHB DMA, 0..7
            NULL,
            0,  // total link-list node
            SrcAddr,   // source begin address
            DstAddr,   // dest begin address
            Size,      // total bytes
            SrcWidth,  // source width 8/16/32 bits -> 0/1/2
            DstWidth,  // dest width 8/16/32 bits -> 0/1/2
            SrcSize,   // source burst size, How many "SrcWidth" will be transmmited at one times ?
            SrcCtrl,   // source address change : Inc/dec/fixed --> 0/1/2
            DstCtrl,   // dest address change : Inc/dec/fixed --> 0/1/2
            Priority,  // priority for this chaanel 0(low)/1/2/3(high)
            Mode,      // Normal/Hardwire,   0/1
            req);
}


void kdp_dma_set_interrupt(u32 channel, u32 tcintr, u32 errintr, u32 abtintr)
{
    dma_ch_cfg cfg;
//	int i;
    
    cfg =  kdp_dma_get_ch_cn_cfg(channel); //ycmo091007 add
    
    if(tcintr)
        cfg.int_tc_msk = 0;	// Enable terminal count interrupt
    else
        cfg.int_tc_msk = 1;	// Disable terminal count interrupt
        
    if(errintr)
        cfg.int_err_msk = 0;	// Enable error interrupt
    else
        cfg.int_err_msk = 1;	// Disable error interrupt
        
    if(abtintr)
        cfg.int_abt_msk = 0;	// Enable abort interrupt	
    else
        cfg.int_abt_msk = 1;	// Disable abort interrupt	
    
    kdp_dma_ch_int_mask(channel, cfg);	
}

void kdp_dma_reset_ch(u8 channel)
{
    u32 base = DMAC_FTDMAC020_PA_BASE_SCPU+DMA_CHANNEL0_BASE+channel*DMA_CHANNEL_OFFSET;	
    
    outw(base+DMA_CHANNEL_CSR_OFFSET,0);
    outw(base+DMA_CHANNEL_CFG_OFFSET,7);
    outw(base+DMA_CHANNEL_SRCADDR_OFFSET,0);
    outw(base+DMA_CHANNEL_DSTADDR_OFFSET,0);
    outw(base+DMA_CHANNEL_LLP_OFFSET,0);
    outw(base+DMA_CHANNEL_SIZE_OFFSET,0);
}

void kdp_dma_clear_all_interrupt()
{
    // Clear all interrupt source
    outw(DMAC_FTDMAC020_PA_BASE_SCPU+DMA_INT_TC_CLR,0xFF);
    outw(DMAC_FTDMAC020_PA_BASE_SCPU+DMA_INT_ERRABT_CLR,0xFF00FF);	
}

void kdp_dma_wait_int_status(u32 Channel)
{
    u32 choffset;
    volatile u32 status;
    
    choffset = 1 << Channel;

    while((inw(DMAC_FTDMAC020_PA_BASE_SCPU+DMA_TC)&choffset)==0)
        ;

    
    kdp_dma_disable_ch(Channel);
}

int kdp_dma_get_ch_num()
{
    //kdp_printf("channel_num reg: %d\n",readl(DMAC_FTDMAC020_PA_BASE_SCPU+DMA_FEATURE));
    return (readl(DMAC_FTDMAC020_PA_BASE_SCPU+DMA_FEATURE)>>12)&0xf;
}

void kdp_dma_enable_dma_int(void)
{
    __dma_int_occurred = 0;
    NVIC_SetVector((IRQn_Type)DMA_FTDMAC020_0_IRQ, (uint32_t)AHB_DMA_IRQHandler);
    NVIC_EnableIRQ(DMA_FTDMAC020_0_IRQ);
    
}

void kdp_dma_disable_dma_int(void)
{
    NVIC_DisableIRQ(DMA_FTDMAC020_0_IRQ);
}

void kdp_dma_enable_dma_tc_int(void)
{
    __dma_tc_int_occurred = 0;
    NVIC_SetVector((IRQn_Type)DMA_FTDMAC020_0_TC_IRQ, (uint32_t)AHB_DMA_TC_IRQHandler);
    NVIC_EnableIRQ(DMA_FTDMAC020_0_TC_IRQ);	  
}

void kdp_dma_disable_dma_tc_int(void)
{
    NVIC_DisableIRQ(DMA_FTDMAC020_0_TC_IRQ);	
}

void kdp_dma_wait_dma_int(u32 channel)
{	
    while(!(__dma_int_occurred & (1 << channel)))
    {
       //kdp_printf("kdp_dma_wait_dma_int: __dma_int_occurred %d\n", __dma_int_occurred);
      __WFE (); 								   // Power-Down until next Event/Interrupt
    }	
    
    __dma_int_occurred &= ~(1 << channel);
}

void kdp_dma_wait_dma_tc_int(u32 channel)
{	
    while(!(__dma_tc_int_occurred & (1 << channel)))
    {
      __WFE (); 								   // Power-Down until next Event/Interrupt
    }	
    
    __dma_tc_int_occurred &= ~(1 << channel);
}
