#include "base.h"
#include "io.h"
#include "Driver_Common.h"
#include "kneron_mozart.h"
#include "kdp520_pwm_timer.h"
#include "pinmux.h"
#include "dbg.h"


static u32 TimerBase[MAX_TIMER+1] ={0, PWM_FTPWMTMR010_PA_BASE+0x10, PWM_FTPWMTMR010_PA_BASE+0x20,PWM_FTPWMTMR010_PA_BASE+0x30,PWM_FTPWMTMR010_PA_BASE+0x40,PWM_FTPWMTMR010_PA_BASE+0x50,PWM_FTPWMTMR010_PA_BASE+0x60,PWM_FTPWMTMR010_PA_BASE+0x70};
static u32 CNTBBase[MAX_TIMER+1] ={0, PWM_FTPWMTMR010_PA_BASE+0x14, PWM_FTPWMTMR010_PA_BASE+0x24,PWM_FTPWMTMR010_PA_BASE+0x34,PWM_FTPWMTMR010_PA_BASE+0x44,PWM_FTPWMTMR010_PA_BASE+0x54,PWM_FTPWMTMR010_PA_BASE+0x64,PWM_FTPWMTMR010_PA_BASE+0x74};
static u32 CMPBBase[MAX_TIMER+1] ={0, PWM_FTPWMTMR010_PA_BASE+0x18, PWM_FTPWMTMR010_PA_BASE+0x28,PWM_FTPWMTMR010_PA_BASE+0x38,PWM_FTPWMTMR010_PA_BASE+0x48,PWM_FTPWMTMR010_PA_BASE+0x58,PWM_FTPWMTMR010_PA_BASE+0x68,PWM_FTPWMTMR010_PA_BASE+0x78};

kdp_timer_control *timer_control[MAX_TIMER+1];
static kdp_timer_struct ftimer[MAX_TIMER+1];

u32 t1_tick = 0, t2_tick = 0, t3_tick = 0, t4_tick = 0, t5_tick = 0, t6_tick = 0;

//funtion prototype
s32 _get_timer_tick(pwmtimer timer);
s32 _set_timer_tick(u32 timer,u32 clk_tick);
s32 _set_timer_clk_source(pwmtimer timer,u32 clk);
//s32 Timer_ConnectIsr(u32 timer,PrHandler handler);
void _print_pwm_reg(pwmtimer timer);

typedef void (*FNTimerISR)(void);

static void PWMTimer1_IRQHandler(void)
{	
 	kdp_timer_int_clear(PWMTIMER1);
    ++t1_tick;
}

static void PWMTimer2_IRQHandler(void)
{	
 	kdp_timer_int_clear(PWMTIMER2);
    ++t2_tick;
}

static void PWMTimer3_IRQHandler(void)
{	
 	kdp_timer_int_clear(PWMTIMER3);
    ++t3_tick;
}

static void PWMTimer4_IRQHandler(void)
{	
 	kdp_timer_int_clear(PWMTIMER4);
    ++t4_tick;
}

static void PWMTimer5_IRQHandler(void)
{	
 	kdp_timer_int_clear(PWMTIMER5);
    ++t5_tick;
}

static void PWMTimer6_IRQHandler(void)
{	
 	kdp_timer_int_clear(PWMTIMER6);
    ++t6_tick;
}


/* Routine to disable a timer and free-up the associated IRQ */
s32 kdp_timer_close(pwmtimer timer)
{
    u32 timer_irq;

    if (timer == 0 || timer > MAX_TIMER)
        return FALSE;

    if(!kdp_timer_disable(timer))
        return FALSE;   /* Stop the timer first */

    switch(timer)
    {
        case PWMTIMER1:
            timer_irq = PWM_FTPWMTMR010_1_IRQ;
        break;
        case PWMTIMER2:
            timer_irq = PWM_FTPWMTMR010_2_IRQ;
        break;
        case PWMTIMER3:
            timer_irq = PWM_FTPWMTMR010_3_IRQ;
        break;
        case PWMTIMER4:
            timer_irq = PWM_FTPWMTMR010_4_IRQ;
        break;
        case PWMTIMER5:
            timer_irq = PWM_FTPWMTMR010_5_IRQ;
        break;
        case PWMTIMER6:
            timer_irq = PWM_FTPWMTMR010_6_IRQ;
        break;
        default:
            return FALSE;
    }	
		
    NVIC_DisableIRQ((IRQn_Type)timer_irq);
		
    return TRUE;
}


s32 kdp_timer_ioctrl(timeriotype iotype, pwmtimer timer, u32 tick)
{
    switch(iotype)
    {
        case IO_TIMER_RESETALL:
            kdp_timer_resetall();
            break;
        case IO_TIMER_GETTICK:
            return _get_timer_tick(timer);
         // break;
        case IO_TIMER_SETTICK:
            return _set_timer_tick(timer,tick);
         // break;
        case IO_TIMER_SETCLKSRC:
            return _set_timer_clk_source(timer,tick);
         // break;
        default:
            return FALSE;
    }

    return TRUE;
}

/* Routine to start the specified timer & enable the interrupt */
//bool kdp_timer_init(pwmtimer timer,u32 tick)
bool kdp_timer_init(pwmtimer timer, u32 tick)
{
    kdp_timer_struct *ctimer=&ftimer[timer];
    u32 timer_irq;
    FNTimerISR isr;

    if (timer == 0 || timer > MAX_TIMER)
    return FALSE;

    switch(timer)
    {
        case PWMTIMER1:
            t1_tick = 0;
            timer_irq = PWM_FTPWMTMR010_1_IRQ;		
            isr = PWMTimer1_IRQHandler;
        break;
        case PWMTIMER2:
            t2_tick = 0;
            timer_irq = PWM_FTPWMTMR010_2_IRQ;		
            isr = PWMTimer2_IRQHandler;
        break;
        case PWMTIMER3:
            t3_tick = 0;
            timer_irq = PWM_FTPWMTMR010_3_IRQ;		
            isr = PWMTimer3_IRQHandler;
        break;
        case PWMTIMER4:
            t4_tick = 0;
            timer_irq = PWM_FTPWMTMR010_4_IRQ;		
            isr = PWMTimer4_IRQHandler;
        break;
        case PWMTIMER5:
            t5_tick = 0;
            timer_irq = PWM_FTPWMTMR010_5_IRQ;		
            isr = PWMTimer5_IRQHandler;
        break;
        case PWMTIMER6:
            t6_tick = 0;
            timer_irq = PWM_FTPWMTMR010_6_IRQ;		
            isr = PWMTimer6_IRQHandler;
        break;
        default:
            return FALSE;
    }

    timer_control[timer]=(kdp_timer_control *)(TimerBase[timer]);
		kdp_timer_close(timer);

    /* Set the timer tick */
    if(!kdp_timer_ioctrl(IO_TIMER_SETTICK,timer,tick))
        return FALSE;

    kdp_timer_autoreloadvalue(timer,ctimer->Tick);
    /*enable auto and int bit */
    kdp_timer_autoreloadenable(timer);

    if (!kdp_timer_int_enable(timer))
        return FALSE;

    NVIC_SetVector((IRQn_Type)timer_irq, (uint32_t)isr);    
    NVIC_EnableIRQ((IRQn_Type)timer_irq);

    /* Start the timer ticking */
    if(!kdp_timer_enable(timer))
        return FALSE;
		
    return TRUE;
}

s32 kdp_timer_autoreloadenable(pwmtimer timer)
{
    //timer_struct *ctimer=&ftimer[timer];

    if ((timer == 0) || (timer > MAX_TIMER))
        return FALSE;

    timer_control[timer]->TmAutoLoad=1;
    return TRUE;
}


s32 kdp_timer_counter(pwmtimer timer)
{
    return inw(TimerBase[timer] + TIMER_CNTO);
}


s32 kdp_timer_int_enable(pwmtimer timer)
{
    if ((timer == 0) || (timer > MAX_TIMER))
        return FALSE;

    timer_control[timer]->TmIntEn=1;
    return TRUE;
}

s32 kdp_timer_int_disable(u32 timer)
{
    //timer_struct *ctimer=&ftimer[timer];

    if ((timer == 0) || (timer > MAX_TIMER))
        return FALSE;

    timer_control[timer]->TmIntEn=0;

    return TRUE;
}


s32 kdp_timer_intmode_enable(u32 timer,u32 mode)
{
    //timer_struct *ctimer=&ftimer[timer];

    if ((timer == 0) || (timer > MAX_TIMER))
        return FALSE;

    timer_control[timer]->TmIntMode=mode;

    return TRUE;
}

s32 _timer_dma_enable(u32 timer)
{

    //timer_struct *ctimer=&ftimer[timer];

    if ((timer == 0) || (timer > MAX_TIMER))
        return FALSE;

      timer_control[timer]->TmDmaEn=1;

    return TRUE;
}


s32 _timer_deadzZone_enable(u32 timer,u32 offset)
{
    //timer_struct *ctimer=&ftimer[timer];

    if ((timer == 0) || (timer > MAX_TIMER))
        return FALSE;

      timer_control[timer]->TmDeadZone=offset;

    return TRUE;
}

/* This routine starts the specified timer hardware. */
s32 kdp_timer_enable(pwmtimer timer)
{
    kdp_timer_struct *ctimer=&ftimer[timer];

    if ((timer == 0) || (timer > MAX_TIMER))
        return FALSE;

    if(ctimer->Running==TRUE)
        return FALSE;
		timer_control[timer]->TmUpdate=1;
		timer_control[timer]->TmStart=1;

    //set the timer status =true
    ctimer->Running=TRUE;

    return TRUE;
}


/* This routine stops the specified timer hardware. */
s32 kdp_timer_disable(pwmtimer timer)
{
    kdp_timer_struct *ctimer=&ftimer[timer];

    if ((timer == 0) || (timer > MAX_TIMER))
        return FALSE;

    /* Disable the Control register bit */
    timer_control[timer]->TmStart=0;
    timer_control[timer]->TmUpdate=0;
    timer_control[timer]->TmOutInv=0;
    timer_control[timer]->TmDmaEn=0;
    timer_control[timer]->TmIntEn=0;
    timer_control[timer]->TmDeadZone=0;
    timer_control[timer]->TmSrc=0;
    timer_control[timer]->TmAutoLoad=0;

     //set the timer status=false
    ctimer->Running=FALSE;

    return TRUE;
}

/* This routine starts the specified timer hardware. */
s32 kdp_timer_int_clear(pwmtimer timer)
{
    int value;

		if ((timer == 0) || (timer > MAX_TIMER))
       return FALSE;

    value=1<<(timer-1);
    outw(PWM_FTPWMTMR010_PA_BASE + TIMER_INTSTAT, value);

    return TRUE;
}

void kdp_timer_cmpvalue(pwmtimer timer, u32 value)
{
    outw(TimerBase[timer] + TIMER_COMPARE, value);
}

s32 _set_timer_clk_source(pwmtimer timer,u32 clk)
{
	if ((timer == 0) || (timer > MAX_TIMER))
		return FALSE;

	timer_control[timer]->TmSrc=clk;

   	return TRUE;
}

#if 0
/* Routine to initialise install requested timer. Stops the timer. */
s32 Timer_ConnectIsr(u32 timer,PrHandler handler)
{
    timer_struct *ctimer=&ftimer[timer];
    u32 i;

    i = fLib_Timer_Vectors[timer];

	if (request_irq(i, handler, IRQF_SHARED | IRQF_DISABLED, "timer", 0) < 0)
	{
		return FALSE;
	}

    ctimer->Handler = handler;
    ctimer->IntNum = i;     /* INT number */

    return timer;
}
#endif

u32 kdp_current_t1_tick(void)
{
    return t1_tick;
}

u32 kdp_current_t2_tick(void)
{
    return t2_tick;
}

u32 kdp_current_t3_tick(void)
{
    return t3_tick;
}

u32 kdp_current_t4_tick(void)
{
    return t4_tick;
}

u32 kdp_current_t5_tick(void)
{
    return t5_tick;
}

u32 kdp_current_t6_tick(void)
{
    return t6_tick;
}

void kdp_timer_tick_reset(pwmtimer timer)
{
    if ((timer == 0) || (timer > MAX_PWM_TIMER))
    {
        return;
    }

    switch(timer)
    {
        case PWMTIMER1:
            t1_tick = 0;
        break;
        case PWMTIMER2:
            t2_tick = 0;
        break;
        case PWMTIMER3:
            t3_tick = 0;
        break;
        case PWMTIMER4:
            t4_tick = 0;
        break;
        case PWMTIMER5:
            t5_tick = 0;
        break;
        case PWMTIMER6:
            t6_tick = 0;
        break;
        default:
            return;
    }   

    return;
}

/////////////////////////////////////////////////////
//
//  Only for detail function call subroutine
//
/////////////////////////////////////////////////////


/* Start-up routine to initialise the timers to a known state */
void kdp_timer_resetall(void)
{
    u32 i;

    //reset all timer to default value
    for (i = 1; i <= MAX_TIMER; i++)
        kdp_timer_disable((pwmtimer)i);

}

s32 _get_timer_tick(pwmtimer timer)
{
    u32 cur_tick;

    volatile kdp_timer_struct *ctimer = &ftimer[timer];

    if ((timer == 0) || (timer > MAX_TIMER))
        return FALSE;

    cur_tick=ctimer->Tick;

    return cur_tick;
}

s32 _set_timer_tick(u32 timer,u32 clk_tick)
{
    volatile kdp_timer_struct *ctimer = &ftimer[timer];

    if ((timer == 0) || (timer > MAX_TIMER))
        return FALSE;

    ctimer->Tick=clk_tick;

    return TRUE;
}

void kdp_timer_autoreloadvalue(pwmtimer timer, u32 value)
{
    outw(TimerBase[timer] + TIMER_LOAD, value);
}

// End of file - timer.c


// Start of PWM function
/* Routine to start the specified timer & enable the PWM function */
bool kdp_pwm_config(pwmtimer timer, pwmpolarity polarity, int duty, int period)
{
    u32 val = 0;
			
    if (timer == 0 || timer > MAX_TIMER)
        return FALSE;

    _print_pwm_reg(timer);
		
    if(timer == PWMTIMER6)
    {
        val = PINMUX_LC_DATA12_GET();
        DSG("LC_DATA12_GET = 0x%4x",val);
        PINMUX_LC_DATA12_SET(PINMUX_LC_DATA12_PWM5);
        val = PINMUX_LC_DATA12_GET();
        DSG("LC_DATA12_GET = 0x%4x",val);
    }
    
    timer_control[timer]=(kdp_timer_control *)(TimerBase[timer]);
    timer_control[timer]->TmStart = 1;
    timer_control[timer]->TmUpdate = 1;
    timer_control[timer]->TmOutInv = polarity;
    timer_control[timer]->TmAutoLoad = 1;
	
    val = 0;
    val = val | timer_control[timer]->TmStart << 1;
    val = val | timer_control[timer]->TmUpdate << 2;
    val = val | timer_control[timer]->TmOutInv << 3;
    val = val | timer_control[timer]->TmAutoLoad << 4;
		
    outw(TimerBase[timer], val);
    outw(CNTBBase[timer], period);
    outw(CMPBBase[timer], duty);
		
    _print_pwm_reg(timer);
    
    return TRUE;
}

bool kdp_pwm_enable(pwmtimer timer)
{
    u32 val = 0;
		
    val = inw(TimerBase[timer]);
    timer_control[timer]->TmPwmEn = 1;
    val = val | timer_control[timer]->TmPwmEn << 8;
    outw(TimerBase[timer], val);
		
    _print_pwm_reg(timer);
    
    return TRUE;
}

bool kdp_pwm_disable(pwmtimer timer)
{
    u32 val = 0;
		
    if(1 != timer_control[timer]->TmPwmEn)
    {
        DSG("pwm%d is not enable!", timer);
        return FALSE;
    }
		
    val = inw(TimerBase[timer]);
    timer_control[timer]->TmPwmEn = 0;
    val = val & 0xFEFF;
    outw(TimerBase[timer], val);
		
    _print_pwm_reg(timer);
    
    return TRUE;
}

void _print_pwm_reg(pwmtimer timer)
{
    u32 val = 0;
	
    val = inw(TimerBase[timer]);
    DSG("T%d_CTRL = 0x%4x", timer, val);
    val = inw(CNTBBase[timer]);
    DSG("T%d_CNTB = 0x%4x", timer, val);
    val = inw(CMPBBase[timer]);
    DSG("T%d_CMPB = 0x%4x", timer, val);
    val = inw(CMPBBase[timer] + 0x4);
    DSG("T%d_CNTO = 0x%4x", timer, val);
}

// End of PWM function

