/*
 * @name : board_kdp520.c
 * @brief : Setup code for KL520
 *
 * Copyright (C) 2019 Kneron, Inc. All rights reserved.
 *
 */
#ifdef V2K_ENABLE
#include "config/board_kdp520.h"
#include <stdio.h> 
#include <string.h> 
#include "kneron_mozart.h"
#include "framework/mm.h"
#include "framework/init.h"
#include "framework/framework_driver.h"
#include "framework/ioport.h"
#include "framework/v2k_image.h"
#include "media/camera/sensor.h"
#include "media/camera/sys_camera.h"
#include "utility.h"
#include "dbg.h"


#if I2C_0_ENABLE == YES

struct ioport_setting INITDATA _io_i2c_0[] = {
    {
        .start      = IIC_FTIIC010_0_PA_BASE,
        .flags      = IORESOURCE_MEM,
    }, {
        .start      = IIC_FTIIC010_0_IRQ,
        .flags      = IORESOURCE_IRQ,
    },
};
struct core_device INITDATA kdp520_i2c_0 = {
    .name           = "kdp520_i2c",
    .ioport         = _io_i2c_0,
    .num_ioports    = ARRAY_SIZE(_io_i2c_0) ,
    .uuid           = 0,
    .pin_ctx = {
        .platform_data = NULL, //&default_i2c_data,
    },
};
#endif
#if I2C_1_ENABLE == YES
struct ioport_setting INITDATA _io_i2c_1[] = {
    {
        .start      = IIC_FTIIC010_1_PA_BASE,
        .flags      = IORESOURCE_MEM,
    }, {
        .start      = IIC_FTIIC010_1_IRQ,
        .flags      = IORESOURCE_IRQ,
    },
};
struct core_device INITDATA kdp520_i2c_1 = {
    .name           = "kdp520_i2c",
    .ioport         = _io_i2c_1,
    .num_ioports    = ARRAY_SIZE(_io_i2c_1) ,
    .uuid           = 1,
    .pin_ctx = {
        .platform_data = NULL, //&default_i2c_data,
    },
};
#endif
#if I2C_2_ENABLE == YES
struct ioport_setting INITDATA _io_i2c_2[] = {
    {
        .start      = IIC_FTIIC010_2_PA_BASE,
        .flags      = IORESOURCE_MEM,
    }, {
        .start      = IIC_FTIIC010_2_IRQ,
        .flags      = IORESOURCE_IRQ,
    },
};
struct core_device INITDATA kdp520_i2c_2 = {
    .name           = "kdp520_i2c",
    .ioport         = _io_i2c_2,
    .num_ioports    = ARRAY_SIZE(_io_i2c_2) ,
    .uuid           = 2,
    .pin_ctx = {
        .platform_data = NULL, //&default_i2c_data,
    },
};
#endif
#if I2C_3_ENABLE == YES
struct ioport_setting INITDATA io_i2c3[] = {
    {
        .start      = IIC_FTIIC010_3_PA_BASE,
        .flags      = IORESOURCE_MEM,
    }, {
        .start      = IIC_FTIIC010_3_IRQ,
        .flags      = IORESOURCE_IRQ,
    },
};
struct core_device INITDATA kdp520_i2c_3 = {
    .name           = "kdp520_i2c",
    .ioport         = io_i2c3,
    .num_ioports    = ARRAY_SIZE(io_i2c3) ,
    .uuid           = 3,
    .pin_ctx = {
        .platform_data = NULL, //&default_i2c_data,
    },    
};
#endif

/* camera pin_context */
static struct sys_camera_info INITDATA _camera_info_0[] = {
    { .name = "sensor-hmx2056", .slave_addr = 0x24 },
    { .name = "sensor-ov9286", .slave_addr = 0x10 },
    { .name = "sensor-hmxrica", .slave_addr = 0x38 },
    { .name = "sensor-gc2145", .slave_addr = 0x3C },
    { .name = "sensor-sc132gs", .slave_addr = 0x30 },
};

#if V2K_ENABLE_TYPE == V2K_ENABLE_HMX2056

    static struct sys_camera_desc hmx2056_link = {
        .connect_to     = 10, 
        .info           = &_camera_info_0[0], 
        .i2c_adapter_id	= 0,
    };
    struct core_device hmx2056_link_device = {
        .uuid           = 4,
        .pin_ctx = {
            .platform_data = &hmx2056_link,
        },
    };

#elif V2K_ENABLE_TYPE == V2K_ENABLE_OV9286

    static struct sys_camera_desc ov9286_link = {
        .connect_to     = 10, 
        .info           = &_camera_info_0[1], 
        .i2c_adapter_id	= 0,
    };
    struct core_device ov9286_link_device = {
        .uuid           = 4,
        .pin_ctx = {
            .platform_data = &ov9286_link,
        },
    };

#elif V2K_ENABLE_TYPE == V2K_ENABLE_HMX2056_OV9286

    static struct sys_camera_desc hmx2056_link = {
        .connect_to     = 10,
        .info           = &_camera_info_0[0], 
        .i2c_adapter_id	= 0,
    };
    struct core_device hmx2056_link_device = {
        .uuid           = 4,
        .pin_ctx = {
            .platform_data = &hmx2056_link,
        },
    };
    static struct sys_camera_desc ov9286_link = {
        .connect_to     = 11,
        .info           = &_camera_info_0[1], 
        .i2c_adapter_id	= 0,
    };
    struct core_device ov9286_link_device = {
        .uuid           = 5,
        .pin_ctx = {
            .platform_data = &ov9286_link,
        },
    };

#elif V2K_ENABLE_TYPE == V2K_ENABLE_HMXRICA

    static struct sys_camera_desc hmxrica_link = {
        .connect_to     = 10, // mipi rx 0
        .info           = &_camera_info_0[2], 
        .i2c_adapter_id	= 0,
    };
    struct core_device hmxrica_link_device = {
        .uuid           = 4,
        .pin_ctx = {
            .platform_data = &hmxrica_link,
        },
    };

#elif (V2K_ENABLE_TYPE == V2K_ENABLE_GC2145) || (V2K_ENABLE_TYPE == V2K_ENABLE_GC2145_SC132GS)
    static struct sys_camera_desc gc2145_link = {
        .connect_to     = 10,
        .info           = &_camera_info_0[3], 
        .i2c_adapter_id	= 0,
    };
    struct core_device gc2145_link_device = {
        .uuid           = 4,
        .pin_ctx = {
            .platform_data = &gc2145_link,
        },
    };
    #if (V2K_ENABLE_TYPE == V2K_ENABLE_GC2145_SC132GS)
        static struct sys_camera_desc sc132gs_link = {
            .connect_to     = 11,
            .info           = &_camera_info_0[4], 
            .i2c_adapter_id	= 0,
        };
        struct core_device sc132gs_link_device = {
            .uuid           = 5,
            .pin_ctx = {
                .platform_data = &sc132gs_link,
            },
        };
    #endif
#elif (V2K_ENABLE_TYPE == V2K_ENABLE_SC132GS) || (V2K_ENABLE_TYPE == V2K_ENABLE_SC132GS_GC2145)
    static struct sys_camera_desc gc2145_link = {
        .connect_to     = 11, 
        .info           = &_camera_info_0[3], 
        .i2c_adapter_id	= 0,
    };
    struct core_device gc2145_link_device = {
        .uuid           = 4,
        .pin_ctx = {
            .platform_data = &gc2145_link,
        },
    };
    static struct sys_camera_desc sc132gs_link = {
        .connect_to     = 10, 
        .info           = &_camera_info_0[4], 
        .i2c_adapter_id	= 0,
    };
    struct core_device sc132gs_link_device = {
        .uuid           = 5,
        .pin_ctx = {
            .platform_data = &sc132gs_link,
        },
    };
#endif

struct core_device kdp520_camera_dev = {
    .name           = "kdp520_camera_dev",
    .uuid           = 6, //nothing
    .pin_ctx = {
    #if V2K_ENABLE_TYPE == V2K_ENABLE_HMX2056
        .platform_data = &hmx2056_link, 
    #elif V2K_ENABLE_TYPE == V2K_ENABLE_OV9286
        .platform_data = &ov9286_link,
    #elif V2K_ENABLE_TYPE == V2K_ENABLE_HMX2056_OV9286
        .platform_data = &hmx2056_link,
    #elif V2K_ENABLE_TYPE == V2K_ENABLE_HMXRICA
        .platform_data = &hmxrica_link,
    #elif V2K_ENABLE_TYPE == V2K_ENABLE_GC2145
        .platform_data = &gc2145_link,
    #elif V2K_ENABLE_TYPE == V2K_ENABLE_SC132GS
        .platform_data = &sc132gs_link,
    #elif V2K_ENABLE_TYPE == V2K_ENABLE_GC2145_SC132GS
        .platform_data = &gc2145_link,
    #elif V2K_ENABLE_TYPE == V2K_ENABLE_SC132GS_GC2145
        .platform_data = &sc132gs_link,
    #endif
    },
};
#if (V2K_ENABLE_TYPE == V2K_ENABLE_HMX2056_OV9286)
    struct core_device kdp520_camera_dev_1 = {
        .name           = "kdp520_camera_dev_1",
        .uuid           = 7, //nothing
        .pin_ctx = {
            .platform_data = &ov9286_link,
        },
    };
#elif  (V2K_ENABLE_TYPE == V2K_ENABLE_GC2145_SC132GS)
    struct core_device kdp520_camera_dev_1 = {
        .name           = "kdp520_camera_dev_1",
        .uuid           = 7, //nothing
        .pin_ctx = {
            .platform_data = &sc132gs_link,
        },
    };
#elif  (V2K_ENABLE_TYPE == V2K_ENABLE_SC132GS_GC2145)
    struct core_device kdp520_camera_dev_1 = {
        .name           = "kdp520_camera_dev_1",
        .uuid           = 7, //nothing
        .pin_ctx = {
            .platform_data = &gc2145_link,
        },
    };    
#endif

#if MIPI_0_RX_ENABLE == YES
static struct ioport_setting csi2rx_resources0[] = {
    [0] = {
        .start = DPI2AHB_CSR_PA_BASE,
        .flags = IORESOURCE_BUS,
    },
    [1] = {
        .start = CSIRX_FTCSIRX100_PA_BASE,
        .flags = IORESOURCE_MEM,
    },
};
struct core_device kdp520_csi2rx0 = {
    .name           = "kdp520_csi2rx0",
    .ioport         = csi2rx_resources0,
    .num_ioports    = ARRAY_SIZE(csi2rx_resources0),
    .uuid           = 10, 
};
#endif

#if MIPI_1_RX_ENABLE == YES
static struct ioport_setting csi2rx_resources1[] = {
    [0] = {
        .start = DPI2AHB_CSR_1_PA_BASE,
        .flags = IORESOURCE_BUS,
    },
    [1] = {
        .start = CSIRX_FTCSIRX100_1_PA_BASE,
        .flags = IORESOURCE_MEM,
    },
};
struct core_device kdp520_csi2rx1 = {
    .name           = "kdp520_csi2rx1",
    .ioport         = csi2rx_resources1,
    .num_ioports    = ARRAY_SIZE(csi2rx_resources1),
    .uuid           = 11, 
};
#endif

#if DISPLAY_DEVICE == DISPLAY_DEVICE_LCDC

    static struct ioport_setting lcdc_resources[] = {
        [0] = {
            .start = LCD_FTLCDC210_PA_BASE,
            .flags = IORESOURCE_BUS,
        },
    };

    struct core_device kdp520_lcdc = {
        .name           = "kdp520_lcdc",
        .ioport         = lcdc_resources,
        .num_ioports    = ARRAY_SIZE(lcdc_resources),        
    };
#elif DISPLAY_DEVICE == DISPLAY_DEVICE_LCM

    static struct ioport_setting lcm_resources[] = {
        [0] = {
            .start = SLCD_FTLCDC210_PA_BASE,
            .flags = IORESOURCE_BUS,
        },
    };
    struct core_device kdp520_lcm = {
        .name           = "kdp520_lcm",
        .ioport         = lcm_resources,
        .num_ioports    = ARRAY_SIZE(lcm_resources),
//        .pin_ctx = {
//            .platform_data = &lcm_info,
//        },
    };
#endif


struct vm_area_struct ddr_vm_area = {
    .vm_start       = KDP_DDR_BASE_MOZART,
    .vm_end         = KDP_DDR_END,
};

#endif
