/*
 * Kneron Application Dynamic Model Execution APIs
 *
 * Copyright (C) 2019 Kneron, Inc. All rights reserved.
 *
 */
 
#ifndef __KDP_APP_DME_H_
#define __KDP_APP_DME_H_

#include <stdint.h>
#include "kdp_app.h"
#include "ipc.h"

typedef struct kdp_app_bounding_box_s {
    float x1;                       /// top-left corner:  x
    float y1;                       /// top-left corner:  y
    float x2;                       /// bottom-right corner:  x
    float y2;                       /// bottom-right corner:  y

    float score;                    /// probability score
    int class_num;                  /// class # (of many) with highest probability
} kdp_app_bounding_box_t;

#define OBJECT_DETECTION_MAX    80

typedef struct kdp_app_dme_res_s {
    uint32_t    class_count;        /// total class count
    uint32_t    box_count;          /// boxes of all classes
    kdp_app_bounding_box_t	boxes[OBJECT_DETECTION_MAX];  /// box information 
} kdp_app_dme_res_t;

/**
 * @brief init necessary data for dynamic model execution
 * @return OK 
 */
int32_t kdp_app_dme_init(void);

/**
 * @brief do dynamic model execution (DME) to get detection output or raw output
 * @param [out] p_out_p dynamic model execution result
 * @param [in] img_conf image configuration
 * @return OK or Err code
 */
int32_t kdp_app_dme(kdp_app_dme_res_t* p_out_p, struct kdp_dme_conf_s img_conf);

#endif
