/*
 * Kneron Application LW3D APIs
 *
 * Copyright (C) 2019 Kneron, Inc. All rights reserved.
 *
 */
 
#ifndef __KDP_APP_LW3D_H_
#define __KDP_APP_LW3D_H_

#include <stdint.h>
#include "kdp_app.h"
#include "kdp_app_fid.h"
// To use new FDR. If using old FDR (FDSmallbox+Onet+VGG10), comment this Macro
//#define FDSSD_ONETBLUR_VGG

/* ==================
 *  LW3D application
 *      kdp_app_lw3d.c
 * ================== */
 
#define NIR_IMG_SOURCE_CROP_H 480
#define NIR_IMG_SOURCE_CROP_W 480

enum kdp_app_lw3d_status_code_e {
    KDP_APP_LW3D_WRONG_USAGE = 0x300,   //KDP_APP_LW3D_CODES
    KDP_APP_LW3D_NO_FACE,
    KDP_APP_LW3D_FACE_BOUNDARY_FAIL,
    KDP_APP_LW3D_BAD_POSE,
    KDP_APP_LW3D_NO_ALIVE,
    KDP_APP_LW3D_FR_FAIL
};

enum _kdp_app_lw3d_face_boundary_check_e {
    RGB_FACE_CHECK = 0x10,
    NIR_FACE_CHECK
};

enum kdp_app_lw3d_face_liveness_process_e {
    LV_RESET = 0x10,
    LV_VOTE,
    LV_DICISION
};

enum kdp_app_lw3d_op_e{
    KDP_APP_LW3D_OP_EXTRACT,
    KDP_APP_LW3D_OP_INFERENCE,
    KDP_APP_LW3D_OP_REGISTER,
};

//inferance/register
//---------------------------------
typedef struct kdp_app_lw3d_inout_s {
    uint16_t op;       			    //0: inference   1-5: register with faceId(1-5)
    uint16_t user_id;    			//as input: user id got from Host
                                    //as output: user id returned by KL520 
    uint16_t face_id;
    float rgb_thresh_fid;           //rgb threshold value for face recognition
    float nir_thresh_fid;           //nir threshold value for face recognition
} kdp_app_lw3d_inout_t;


typedef struct facedet_result_s kdp_app_lw3d_face_det_res_t;
typedef struct landmark_result_s kdp_app_lw3d_face_lm_res_t;
typedef struct fr_result_s kdp_app_lw3d_face_fmap_t;


/**
 * @brief do face recognition for inference or register
 * @param inputPtr points
 *          { mode;
 *            user_id; //as input (or ouput) }
 *        mode = 0   : capture + fd/fr + 1:n compare
 *        mode = 1-5 : capture + fd/fr + register in DDR DB
 *        outputPtr: N/A
 * @return OK or Err code
 */
int32_t kdp_app_lw3d(
    kdp_app_lw3d_inout_t        *p_inout_p,
    kdp_app_lw3d_face_det_res_t *p_fd_out_p,
    kdp_app_lw3d_face_lm_res_t  *p_lm_out_p,
    kdp_app_lw3d_face_fmap_t    *p_fr_out_p,
    kdp_app_lw3d_face_det_res_t *p_nir_fd_out_p,
    kdp_app_lw3d_face_lm_res_t  *p_nir_lm_out_p,
    kdp_app_lw3d_face_fmap_t    *p_nir_fr_out_p
);


/**
 * @brief  init necessary data structure
 */
int32_t kdp_app_lw3d_init(void);

/**
 * @brief get image setting for HOST's request
 * @param[out] image_buf descriptor     : image buf count, data addr alignment, base addr and size
 * @param[in]  rgb image config param   : rgb image width, height and format
 * @param[in]  nir image config param   : nir image width, height and format
 * @return OK (0) or error code
 */
int kdp_app_lw3d_query_img_transfer_setting(
    kdp_app_image_buffer_desc_t *buf_desc_p,
    kdp_img_conf_t *rgb_img_config_p,
    kdp_img_conf_t *nir_img_config_p);

/**
 * @brief get feature map of cam1(nir) only
 * @param[inout] *p_inout_p image information and settings
 * @param[out] *p_fd_out_p FD result
 * @param[out] *p_lm_out_p LM result
 * @param[out] *p_fr_out_p FR result
 * @return OK (0) or error code
 */
int32_t kdp_app_nir_fdr_only(
    kdp_app_fid_inout_t     *p_inout_p,
    kdp_app_face_det_res_t  *p_fd_out_p,
    kdp_app_face_lm_res_t   *p_lm_out_p,
    kdp_app_face_fmap_t     *p_fr_out_p);

/**
 * @brief do FD/FR 
 * @param[in] distance_flag distance flag
 * @param[inout] *p_inout_p image information and settings
 * @param[out] *p_fd_out_p FD result
 * @param[out] *p_lm_out_p LM result
 * @param[out] *p_fr_out_p FR result
 * @return OK (0) or error code
 */
int32_t kdp_app_nir_fdr(int distance_flag,
    kdp_app_fid_inout_t     *p_inout_p,
    kdp_app_face_det_res_t  *p_fd_out_p,
    kdp_app_face_lm_res_t   *p_lm_out_p,
    kdp_app_face_fmap_t     *p_fr_out_p);

#endif
