#include "customizedCode.h"
#include "kdp_memxfer.h"    //for flash access
#include "kdp_memory.h"     //for ddr functions 
#include "dbg.h"
#include "kdp_ddr_table.h"  //for default ddr table
#include "kdp_app.h"
#include <stdint.h>
extern const struct s_kdp_memxfer kdp_memxfer_module;

struct bounding_box_s_l {
    float x1;           // top-left corner:  x
    float y1;           // top-left corner:  y
    float x2;           // bottom-right corner:  x
    float y2;           // bottom-right corner:  y

    float score;        // probability score
    int class_num;      // class # (of many) with highest probability
};

#define YOLO_DETECTION_MAX_L    20

typedef struct yolo_result_s_l {
    uint32_t    class_count;            // total class count
    uint32_t    box_count;              /* boxes of all classes */
    struct bounding_box_s_l boxes[YOLO_DETECTION_MAX_L];       /* [box_count] */
} yolo_result_s_l_t;

static struct yolo_result_s_l        *p_obj_det_output_s1;   //strcture defined in ipc.h



int32_t model_app_fd_customer_l(
    struct yolo_result_s_l *p_fd_output_p,
    kdp_img_conf_t      *p_img_setting_p,
    enum model_type model_type_p) {
    int status;

    //------------------------------------------------------------------------------
    if(p_img_setting_p == 0 ) {
        err_msg("[ERR] no image setting is specified\n");
    } else {
        DSG("%d %d %d 0x%x ",p_img_setting_p->image_col, p_img_setting_p->image_row, 
            p_img_setting_p->image_ch, p_img_setting_p->image_format);
    }

    //------------------------------------------------------------------------------
    DSG("[CUSTOMER] Got model id [%d] \n",model_type_p);
    kdp_app_config_model_input_output(model_type_p, p_img_setting_p,p_fd_output_p);

    status = kdp_app_run_model();

    //output result
    //------------------------------------------------------------------------------
    if(status != IMAGE_STATE_DONE)
        return 1;

    return 0;
}

int32_t kdp_app_fd_customer_l(yolo_result_s_l_t* p_out_p/*out*/) {
    int32_t ret;
    if(p_out_p == 0) {
        if(p_obj_det_output_s1 == 0)
            p_obj_det_output_s1 = (struct yolo_result_s_l*)kdp_ddr_reserve(sizeof(struct yolo_result_s_l));
        p_out_p = (yolo_result_s_l_t*)p_obj_det_output_s1;
    }
    
    //struct kdp_image_s img_setting;
    kdp_img_conf_t img_setting ;
    img_setting.image_col       = 224;
    img_setting.image_row       = 224;
    img_setting.image_ch        = 4;
    img_setting.image_format    = NPU_FORMAT_RGBA8888 | IMAGE_FORMAT_SUB128 | IMAGE_FORMAT_BYPASS_PRE ;
    img_setting.image_memory_address  = KDP_DDR_BASE;

    ret = model_app_fd_customer_l((struct yolo_result_s_l *)p_out_p, &img_setting, TINY_YOLO_V3 );
    if(ret != 0) {
        return ret;
    }

    return ret;
}

void customer_model_run(void *argument) {

    struct yolo_result_s_l* p_fd_out_s = (struct yolo_result_s_l*)kdp_ddr_reserve(sizeof(struct yolo_result_s_l));

    kdp_memxfer_module.init(MEMXFER_OPS_DMA, MEMXFER_OPS_DMA);
    // specified flash address for image
    kdp_memxfer_module.flash_to_ddr(KDP_DDR_BASE, 0xA66000, 200704 );
    int ret = kdp_app_fd_customer_l( (yolo_result_s_l_t *)p_fd_out_s);

    struct yolo_result_s* p_fd_output_p =(struct yolo_result_s*) p_fd_out_s;
    DSG("[CUSTOMER] OBJ_DET (classCount,BoxCount)= %d,%d\n",
        p_fd_output_p->class_count,
        p_fd_output_p->box_count);

    if (p_fd_output_p->box_count > 0) {
        int print_count = (p_fd_output_p->box_count > 3) ? 3 : p_fd_output_p->box_count;
        DSG("(print 3 boxes at most)\n");
        
        for (int i = 0; i < print_count; i++) {
            DSG("box[%d]:class=%d, (x1,y1,x2,y2) = (%f, %f, %f, %f) @ score=%f\n", i,
                p_fd_output_p->boxes[i].class_num,
                p_fd_output_p->boxes[i].x1,
                p_fd_output_p->boxes[i].y1,
                p_fd_output_p->boxes[i].x2,
                p_fd_output_p->boxes[i].y2,
                p_fd_output_p->boxes[i].score);
        }
    }
}
















