/* --------------------------------------------------------------------------
 * Copyright (c) 2013-2016 ARM Limited. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *      Name:    main.c
 *      Purpose: RTX for Kneron
 *
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include "kdp_uart.h"
#include "DrvDMAC020.h"
#include "kneron_mozart.h"                // Device header
#include "kneron_mozart_ext.h"
#include "cmsis_os2.h"                    // ARM::CMSIS:RTOS2:Keil RTX5
#include "Driver_Common.h"
#include "base.h"
#include "io.h"
#include "string.h"
#include "ddr.h"
#include "host_com.h"
#include "delay.h"
#ifdef KL520
#include "scu_extreg.h"
#include "bootloader.h"
#include "power.h"
#include "system.h"
#include "clock.h"
#include "power_manager.h"
#include "scu_reg.h"
#endif

#include "kdp_memory.h"     //for ddr_malloc
#include "kdp_ddr_table.h"  //for default ddr table
#include "version.h"
#include "dbg.h"
#include "pinmux.h"
#ifdef USE_KDP_CAMERA
#include "media/kdp_camera.h"
#else
#include "v2k_cam.h"
#include "framework/framework.h"
#endif

#ifdef BOARD_CUSTOMER1
#include "sample_app/user_io.h"
#include "sample_app/touch_panel.h"
int kdp_drv_init(void);
#endif

#ifdef MOZART_CONSOLE
#include "sample_app/sample_app_console.h"
#endif


#include "customizedCode.h"


#define MSG_DBG_LOG
#define MSG_MEM_BYTE_WRITE     0

#include "kdp_i2c.h"
#include "kdp_uart.h"
extern void kdp_uart_app_uart0_log(void);
extern void kdp_uart_app_host_com(void);
extern void kdp_com_init(void);

/**
 * @info(), Get kernel information
 */

extern u32 bootup_status;
extern u32 warm_boot;

void info(int from_boot_loader) {
    //char infobuf[100];
    //osVersion_t osv;
    //osStatus_t status;
    uint32_t rom_table;

    if (from_boot_loader) {
        DSG("\n=== From Bootloader ===");
    } else {
        DSG("\n=== From JTag debug ===");
    }

    rom_table = inw(CPU_ID_STS_ADDR);
    if (CPU_ID_SCPU == rom_table) {
        DSG("=== [I am SCPU] firmware version: SDK v%d.%d.%d build%d ===", g_fw_misc.major,
                                                                           g_fw_misc.minor,
                                                                           g_fw_misc.update,
                                                                           g_fw_misc.build);
    }
    else if (CPU_ID_NCPU == rom_table) {
        DSG("=== [Who am I? It must be something wrong. ===");
    }

    DSG("=== This is customer_tutorial mode ===");
    
    //status = osKernelGetInfo(&osv, infobuf, sizeof(infobuf));
    //if(status == osOK) {	
    //    DSG("+-------------------Keil RTX5----------------------+");
    //    DSG("Kernel Information: %s", infobuf);
    //    DSG("Kernel Version    : %d", osv.kernel);
    //    DSG("Kernel API Version: %d", osv.api);      
    //}

    if (warm_boot) {
        DSG("  -> Bootup Status: 0x%x (warm boot) <-", bootup_status);
    } else {
        DSG("  -> Bootup Status: 0x%x <-", bootup_status);
    }

    if (bootup_status & SCU_REG_BTUP_STS_HWR)
        DSG("                  : Hardware Reset");
    if (bootup_status & SCU_REG_BTUP_STS_WDR)
        DSG("                  : Watchdog Reset");
    if (bootup_status & SCU_REG_BTUP_STS_SMR)
        DSG("                  : Dormant Reset");
    if (bootup_status & SCU_REG_BTUP_STS_PMR)
        DSG("                  : Poweroff Reset");
    if (bootup_status & SCU_REG_BTUP_STS_PWRBTN_STS)
        DSG("                  : Power Button Reset");
    if (bootup_status & SCU_REG_BTUP_STS_RTC_BTUPTS)
        DSG("                  : RTC Alarm Reset");
    if (bootup_status & SCU_REG_BTUP_STS_PMR1)
        DSG("                  : Power Mode 1 Reset");
    if (bootup_status & SCU_REG_BTUP_STS_PMR2)
        DSG("                  : Power Mode 2 Reset");
}

/**
 * @brief main, main dispatch function
 */
int main(void) {
    int from_boot_loader = 1;

    from_boot_loader = bootloader_handler();

    if (from_boot_loader)
        system_check_fw_image(SCPU_FW);

    system_init();
    system_init_ncpu();
    ddr_init(DDR_INIT_ALL);

    if (from_boot_loader)
        load_ncpu_fw(-1);  // load & launch ncpu fw from flash, skip if using debugger (ncpu loaded via JTAG)
    else
        load_ncpu_fw(0);  // just launch ncpu

    pinmux_init();
    kdp_ddr_init(KDP_DDR_HEAP_HEAD_FOR_MALLOC+1, KDP_DDR_MODEL_RESERVED_END);
    kdp_uart_init();
    kdp_uart_app_uart0_log();   // for log

    info(from_boot_loader);
    SystemCoreClockUpdate();            // System Initialization
    osKernelInitialize();               // Initialize CMSIS-RTOS
    power_manager_init();

    kdp_com_init();                     // NPU channel initialization

    osThreadId_t tid1 = osThreadNew(customer_model_run, NULL, NULL);
    if (osKernelGetState() == osKernelReady) {
        osKernelStart();
    }

    while(1) {
    }
}


