/*
 * Kneron Host UART Communication Application
 *
 * Copyright (C) 2019 Kneron, Inc. All rights reserved.
 *
 */
 
#include <stdio.h>
#include <string.h>
#include <stdarg.h>

#include "cmsis_os2.h"
#include "kdp_uart.h"
#include "Driver_Common.h"
#include "io.h"
#include "kneron_mozart.h"  
#include "dbg.h"
#include "com.h"
#include "msg.h"
#include "uart_com.h"

static  kdp_uart_hdl_t  handle0;
static  kdp_uart_hdl_t  handle4;
//static  kdp_uart_api_sts_t sts0;
static  kdp_uart_api_sts_t sts4;

static  osThreadId_t tid_uart_com;

bool UART0_Rx = false;
bool UART0_Tx = false;

bool UART1_Rx = false;
bool UART1_Tx = false;

bool UART2_Rx = false;
bool UART2_Tx = false;

bool UART3_Rx = false;
bool UART3_Tx = false;

bool UART4_Rx = false;
bool UART4_Tx = false;

void fLib_printf(const char *f, ...);    /* variable arguments */

void UART0_callback(uint32_t event)
{
    if (event & ARM_USART_EVENT_RECEIVE_COMPLETE)
    {
        UART0_Rx = true;
    }

    if (event & ARM_USART_EVENT_RX_TIMEOUT)
    {
        UART0_Rx = true;
    }

}

void UART1_callback(uint32_t event)
{
    if (event & ARM_USART_EVENT_RECEIVE_COMPLETE)
    {
        UART1_Rx = true;
    }

    if (event & ARM_USART_EVENT_SEND_COMPLETE)
    {
        UART1_Tx = true;
    }
}

void UART2_callback(uint32_t event)
{
    if (event & ARM_USART_EVENT_RECEIVE_COMPLETE)
    {
        UART2_Rx = true;
    }

    if (event & ARM_USART_EVENT_SEND_COMPLETE)
    {
        UART2_Tx = true;
    }
}

void UART3_callback(uint32_t event)
{
    if (event & ARM_USART_EVENT_RECEIVE_COMPLETE)
    {
        UART3_Rx = true;
    }

    if (event & ARM_USART_EVENT_SEND_COMPLETE)
    {
        UART3_Tx = true;
    }
}

void UART4_callback(uint32_t event)
{
    if (event & ARM_USART_EVENT_RECEIVE_COMPLETE)
    {
        UART4_Rx = true;
        osThreadFlagsSet(tid_uart_com, FLAG_UART4_RX_DONE);
    }

    if (event & ARM_USART_EVENT_SEND_COMPLETE)
    {
        UART4_Tx = true;
        osThreadFlagsSet(tid_uart_com, FLAG_UART4_TX_DONE);
    }

    if (event & ARM_USART_EVENT_RX_TIMEOUT)
    {
        UART4_Rx = true;
        osThreadFlagsSet(tid_uart_com, FLAG_UART4_RX_TIMEOUT);
    }

}

void kdp_uart_app_com4(void)
{
    int32_t ret;

    handle4 = kdp_uart_open(UART4_DEV, UART_MODE_ASYN_RX | UART_MODE_SYNC_TX, UART4_callback);
    if (handle4 == UART_FAIL)
    {
        dbg_msg("Open failed\n");
        return;
    }

    ret = kdp_uart_power_control(handle4, ARM_POWER_FULL);

    if (ret != UART_API_RETURN_SUCCESS)
    {
        dbg_msg("Power on failed\n");
        return;
    }

    KDP_UART_CONFIG_t cfg;
    cfg.baudrate = BAUD_921600;
    cfg.data_bits = 8;
    cfg.frame_length = 0;
    cfg.stop_bits = 1;
    cfg.parity_mode = PARITY_NONE;
    cfg.fifo_en = TRUE;

    ret = kdp_uart_control(handle4, UART_CTRL_CONFIG, (void *)&cfg);

    if (ret != UART_API_RETURN_SUCCESS)
    {
        dbg_msg("UART config failed\n");
        return;
    }

    UART4_Rx = FALSE;
    UART4_Tx = FALSE;

}

void kdp_uart_app_uart0_log(void)
{
    int32_t ret;

    handle0 = kdp_uart_open(UART0_DEV, UART_MODE_SYNC_RX | UART_MODE_SYNC_TX, UART0_callback);
    if (handle0 == UART_FAIL)
    {
        dbg_msg("Open failed\n");
        return;
    }

    ret = kdp_uart_power_control(handle0, ARM_POWER_FULL);

    if (ret != UART_API_RETURN_SUCCESS)
    {
        dbg_msg("Power on failed\n");
        return;
    }

    KDP_UART_CONFIG_t cfg;
    cfg.baudrate = BAUD_115200; // BAUD_921600;
    cfg.data_bits = 8;
    cfg.frame_length = 0;
    cfg.stop_bits = 1;
    cfg.parity_mode = PARITY_NONE;
    cfg.fifo_en = FALSE;

    ret = kdp_uart_control(handle0, UART_CTRL_CONFIG, (void *)&cfg);

    if (ret != UART_API_RETURN_SUCCESS)
    {
        dbg_msg("UART config failed\n");
        return;
    }

    UART0_Rx = FALSE;
    UART0_Tx = FALSE;

}

static char uart_buffer[256];
const char uart_error_msg[30] = "ERROR: UART Buffer Overrun.\r\n\0";
void fLib_printf(const char *f, ...)    /* variable arguments */
{
    va_list arg_ptr;

    va_start(arg_ptr, f);
    vsprintf(&uart_buffer[0], f, arg_ptr);
    va_end(arg_ptr);

    UART0_Tx = FALSE;
    if (strlen(uart_buffer) > 256) {
        kdp_uart_write(handle0, (u8*) uart_error_msg, strlen(uart_error_msg));
        uart_buffer[255] = 0;  // truncate the original output
    }
    kdp_uart_write(handle0, (u8*) uart_buffer, strlen(uart_buffer));
}

char uart_com_log_getc(void)
{
    char c;
    UART0_Rx = FALSE;
    kdp_uart_read(handle0, (uint8_t *)&c, 1);
    return c;
}

void uart_com_log_putc(char Ch)
{
    char cc;

    if (Ch != '\0') {
        cc = Ch;
        kdp_uart_write(handle0, (u8*)&cc, 1);
    }

    if (Ch == '\n') {
        cc = '\r';
        kdp_uart_write(handle0, (u8*)&cc, 1);
    }
}

void uart_com_log_puts(char *str)
{
    char *cp;
    for (cp = str; *cp != 0; cp++)
        uart_com_log_putc(*cp);
}

int uart_com_log_echo_gets(char *buf, int len)
{
    char    *cp;
    char    data;
    u32  count;
    count = 0;
    cp = buf;

    do
    {
        kdp_uart_read(handle0, (uint8_t *)&data, 1);

        switch (data)
        {
        case RETURN_KEY:
            if (count < len)
            {
                *cp = '\0';
                uart_com_log_putc('\n');
            }
            break;
        case BACKSP_KEY:
        case DELETE_KEY:
            if ((count > 0) && (count < len))
            {
                count--;
                *(--cp) = '\0';
                uart_com_log_puts("\b \b");
            }
            break;
        default:
            if (data > 0x1F && data < 0x7F && count < len)
            {
                *cp = (char)data;
                cp++;
                count++;
                uart_com_log_putc(data);
            }
            break;
        }
    } while (data != RETURN_KEY);

    return (count);
}


static uint32_t flags_to_notify, data_size;
static osThreadId_t tid_to_notify;
static uint32_t kdp_uart_data;
static u8 *data_start;

void uart_host_comm_init(uint32_t flag)
{
      // the same as set_notify
    tid_to_notify = osThreadGetId();
    flags_to_notify = flag;
}

int kdp_uart_com_read(uint8_t *BufferPtr, uint32_t BufferLen)
{
    kdp_uart_data = 1;
    data_start = BufferPtr;
    osThreadFlagsWait(flags_to_notify, osFlagsWaitAll, osWaitForever);  // put calling thread to sleep
    return data_size;
}

static void uart_comm_thread(void *argument)
{
    int32_t flags;

    uint32_t msg_buf_max_len = MSG_DATA_BUF_MAX + sizeof(MsgHdr) + sizeof(RspPram) + 4;

    UART4_Rx = FALSE;
    memset(msg_rbuf, 0, msg_buf_max_len);
    kdp_uart_data = 0;
    kdp_uart_read(handle4, msg_rbuf, msg_buf_max_len);

    for (;;) {
        flags = osThreadFlagsWait(FLAG_UART4_RX_DONE | FLAG_UART4_RX_TIMEOUT, osFlagsWaitAny, osWaitForever);
        osThreadFlagsClear(flags);

        if (flags & (FLAG_UART4_RX_DONE | FLAG_UART4_RX_TIMEOUT))
        {
            uint32_t count = kdp_uart_GetRxCount(handle4);
            if (kdp_uart_data) {
                memcpy(data_start, msg_rbuf, count);
                if (count < data_size) {  // only partial data read
                    // TO DO: copy partial data and restart the data read
                    err_msg("[UART] data block read failed\r\n");
                    data_size = count;
                }
                kdp_uart_data = 0;  // exit data mode
                osThreadFlagsSet(tid_to_notify, flags_to_notify);  // wake up calling thread
            }
            else {
                osThreadFlagsSet(tid_to_notify, flags_to_notify);  // wake up calling thread
            }

            UART4_Rx = FALSE;
            sts4 = kdp_uart_read(handle4, msg_rbuf, msg_buf_max_len);
            if (sts4 != UART_API_RX_BUSY)
            {
                dbg_msg("failed on kdp_uart_read, error code = %d\n", sts4);
            }
        }
    }
}

void kdp_uart_com_init(void)
{
//    kdp_uart_init();
    kdp_uart_app_com4();

    tid_uart_com = osThreadNew(uart_comm_thread, NULL, NULL);
}

void kdp_uart_com_write(uint8_t *p, int32_t len)
{
    kdp_uart_write(handle4, p, len);
}

int kdp_com_getc(uint8_t *BufferPtr)
{
    int32_t flags;
    kdp_uart_read(handle4, BufferPtr, 1);
    flags = osThreadFlagsWait(FLAG_UART4_RX_DONE | FLAG_UART4_RX_TIMEOUT, osFlagsWaitAny, osWaitForever);
    osThreadFlagsClear(flags);

    return 0;

}

void kdp_com_putc(char ch)
{
    kdp_uart_write(handle4, (uint8_t *)&ch, 1);
}

