/*
 * @name : v2k_cam.c
 * @brief : V2K sample user application
 *
 * Copyright (C) 2019 Kneron, Inc. All rights reserved.
 *
 */
#ifndef USE_KDP_CAMERA
#ifdef V2K_ENABLE
#include "config/board_kdp520.h"
#include "v2k_cam.h"
#include <string.h> 
#include <stdio.h> 
#include <stdlib.h>
#include <cmsis_os2.h>
#include "base.h"
#include "framework/framework.h"
#include "framework/framework_errno.h"
#include "framework/v2k_ioctl.h"
#include "framework/mm.h"
#include "framework/kdp_dev.h"
#include "framework/mutex.h"
#include "framework/event.h"
#include "framework/framework_driver.h"
//driver include file. need a much better refactor
#include "pinmux.h"
#include "system.h"
#include "framework/v2k_image.h" 
#include "media/display/kdp_display.h"
#include "kdp_uart.h"
#include "kdp520_pwm_timer.h"
#include "dbg.h"
#include "media/camera/sys_camera.h"
#include "uart_com.h"

/* wait for the host command to end the loop */
#define FRAME_INFINITE              (0xFFFFFFFFUL)
#define FRAME_COUNT_TO_END          1000
#define BUFFER_COUNT                2

#define V2K_SUCCEEDED(__ret__)      (0 == __ret__)
#define V2K_SUCCEEDED_RET(__ret__)  { if (!V2K_SUCCEEDED(__ret__)) return __ret__; }

#define FLAG_V2K_CAM_START          0x0001
#define FLAGS_MIPI0_READY_EVT       0x2001



//#define PERPETUAL_RUN
#ifdef PERPETUAL_RUN
#define V2K_CAM_PERPETUAL_RUN_PREVIEW
#endif

#define V2K_CAM_MIPI_CAPTURE
//#define TEST_HOST_COMM


struct camera_buffer_info {
    void* start;
    unsigned int length;
};

struct v2k_cam_context inst_cam[IMGSRC_NUM] = { 0 };

extern int gval;

extern void small_delay(int us);
extern void delay_ms(int msec);

struct mutex cam_lock;
enum camera_state m_state[IMGSRC_NUM];

osThreadId_t tid_v2k_cam_0;
osThreadId_t tid_v2k_cam_1;

#if 0
static void _default_process_image(
		unsigned int cam_idx, struct v2k_buffer *buf, 
        void *user_start, u32 user_length)
{
    //++m_frame_number;
    //inst_cam[cam_idx].curr_addr = buf->type;
//    DSG("<%s> buffer[%d]: type=%x start=0x%x len=%u frames=%d ts=%llu", 
//        __func__, 
//        buf->index,
//        buf->type, 
//        (unsigned int)user_start, 
//        user_length,
//        m_frame_number,
//        buf->timestamp);
}
static fn_v2k_cam_newimage _on_newimage = _default_process_image;
#endif

//unsigned int kdp_camera_get_buffer_cnt(unsigned int cam_idx) 
//{
//    return 0;//inst_cam[cam_idx].request_bufs.count;
//}

int kdp_camera_capture_frame(
		unsigned int cam_idx, unsigned int *addr, unsigned int *size)
{
    int ret = 0;
    
    while (inst_cam[cam_idx].old_addr == inst_cam[cam_idx].curr_addr)
        osDelay(1);
        
    mutex_lock(&cam_lock);
    
    if ((addr) && (size)) {
        *addr = (unsigned int)inst_cam[cam_idx].app_buf_infos[0].start;
        *size = inst_cam[cam_idx].app_buf_infos[0].length;
    }
    
    mutex_unlock(&cam_lock);    
    
    inst_cam[cam_idx].old_addr = inst_cam[cam_idx].curr_addr;
    
    return ret;
}

#if 0
void register_v2k_cam_newimage(fn_v2k_cam_newimage fn) {
    if (fn)
        _on_newimage = fn;
}

static int _read_frame(unsigned int cam_idx)
{
    int ret;
    struct v2k_buffer buf;

#if 0
//    ret = kdp_dev_ioctl(fd, V2K_VIOC_DQBUF, (void*)&buf);
//    DSG("V2K_VIOC_DQBUF ret=%x", ret);    
//    ret = kdp_dev_ioctl(fd, V2K_VIOC_QBUF, (void*)&buf);
//    DSG("V2K_VIOC_QBUF ret=%x", ret);
//    
//    DSG("(unsigned int)app_buf_infos[0].start=%x length=%u", (unsigned int)app_buf_infos[0].start,app_buf_infos[0].length);
//    DSG("press any key for snapshot");
//    kdp_getchar(DEBUG_CONSOLE);
//    osDelay(1000);    
#else    
    ret = kdp_dev_ioctl(inst_cam[cam_idx].handle, V2K_VIOC_DQBUF, (void*)&buf);

    if (V2K_SUCCEEDED(ret)) {
        {
            mutex_lock(&cam_lock);
            
            _on_newimage(cam_idx, &buf, 
                inst_cam[cam_idx].app_buf_infos[buf.index].start, 
                inst_cam[cam_idx].app_buf_infos[buf.index].length);
            
            mutex_unlock(&cam_lock);
        }
        ret = kdp_dev_ioctl(inst_cam[cam_idx].handle, V2K_VIOC_QBUF, (void*)&buf);
    }
#endif
    return ret;
}
#endif

static int _cam_get_device_info(unsigned int cam_idx, struct v2k_capability *lpCap)
{
    return kdp_dev_ioctl(inst_cam[cam_idx].handle, V2K_VIOC_QUERYCAP, (void*)lpCap);
}

static int _cam_set_frame_format(
        unsigned int cam_idx, 
        unsigned int width,
        unsigned int height,
        unsigned int pixelformat)
{
    int ret;
    struct v2k_format fmt;
    memset(&fmt, 0, sizeof(fmt));

    fmt.width = width;
    fmt.height = height;
    fmt.pixelformat = pixelformat;
    fmt.field = V2K_FIELD_ANY; //V2K_FIELD_INTERLACED;	

    ret = kdp_dev_ioctl(inst_cam[cam_idx].handle, V2K_VIOC_S_FMT, (void*)&fmt);
    if (V2K_SUCCEEDED(ret)) {
        DSG("v2k_cam: set video format, OK");
    } else {
        DSG("v2k_cam: set video format, ERROR(%d)", ret);
    }

    return ret;
}

static int _cam_get_frame_format(unsigned int cam_idx, struct v2k_format *lpfmt)
{
    return kdp_dev_ioctl(inst_cam[cam_idx].handle, V2K_VIOC_G_FMT, (void*)lpfmt);
}

static int _cam_init_mem_mapping(unsigned int cam_idx)
{
    int ret;
    int i;
    struct v2k_buffer buf;
    struct v2k_requestbuffers request_bufs;
    request_bufs.count = BUFFER_COUNT;
    //inst_cam[cam_idx].request_bufs.count = BUFFER_COUNT;
    DSG("[%s] BUFFER_COUNT=%x", __func__, BUFFER_COUNT);
    ret = kdp_dev_ioctl(
		inst_cam[cam_idx].handle, V2K_VIOC_REQBUFS, (void*)&request_bufs);
    if (V2K_SUCCEEDED(ret)) {
        inst_cam[cam_idx].app_buf_infos = (struct camera_buffer_info*)
			calloc(request_bufs.count, sizeof(struct camera_buffer_info)); 
        DSG("[%s] nst_cam[cam_idx].app_buf_infos=%p", __func__, inst_cam[cam_idx].app_buf_infos);        
        if (!inst_cam[cam_idx].app_buf_infos) {
            return -KDP_FRAMEWORK_ERRNO_NOMEM;
        }

        for (i = 0; i < request_bufs.count; ++i) {
            buf.index = i;
            buf.length = 0;

            DSG("[%s] V2K_VIOC_QUERYBUF", __func__);        
            ret = kdp_dev_ioctl(inst_cam[cam_idx].handle, V2K_VIOC_QUERYBUF, (void*)&buf);
            if (!V2K_SUCCEEDED(ret)) {
                DSG("v2k_cam: Query the status of a buffer, ERROR(%d)", ret);
                break;
            }

            inst_cam[cam_idx].app_buf_infos[i].length = buf.length;
            inst_cam[cam_idx].app_buf_infos[i].start = (void*)kdp_mmap(buf.length, 0, inst_cam[cam_idx].handle, buf.offset);
            //DSG("<%s> i=%d buf.offset=0x%x length=%d start=0x%x", __func__, i, buf.offset, app_buf_infos[i].length, app_buf_infos[i].start);
            if (MAP_FAILED == inst_cam[cam_idx].app_buf_infos[i].start) {
                ret = -1;
                break;
            }
        }

    } else {
        DSG("v2k_cam: Initiate Memory Mapping, ERROR(%d)", ret);
    }
 
    return ret;
}

static int _cam_start_capturing(unsigned int cam_idx)
{
    int ret;
    unsigned int i;

    for (i = 0; i < BUFFER_COUNT; ++i)
    {
        inst_cam[cam_idx].api_buf.index = i;
        //DSG("inst_cam[%d].api_buf.index=%d", cam_idx, inst_cam[cam_idx].api_buf.index);
        ret = kdp_dev_ioctl(
			inst_cam[cam_idx].handle, V2K_VIOC_QBUF, (void*)&(inst_cam[cam_idx].api_buf));
        if (!V2K_SUCCEEDED(ret)) {   
            DSG("v2k_cam: Exchange a buffer with the driver, ERROR(%d)", ret);
            break;
        }
    }
    //if (0 == cam_idx) 
    {
        DSG("[%s] start", __func__);
        ret = kdp_dev_ioctl(inst_cam[cam_idx].handle, V2K_VIOC_STREAMON, 0);
        //DSG("[%s] ret=%d end", __func__, ret);
        if (!V2K_SUCCEEDED(ret)) {
            DSG("v2k_cam: Start streaming I/O, ERROR(%d)", ret);
    }
}
    return ret;
}
#if 0
static int _cam_loop(unsigned int cam_idx)
{
    unsigned int count = 0;
    int ret;
    
    do {        
        for (;;) {
            
            ret = _read_frame(cam_idx);
            if (V2K_SUCCEEDED(ret))
                break;
            if (0 == count)
                set_event(evt_source_data_ready_id, FLAGS_SOURCE_READY_EVT);
            switch (ret) {
                case KDP_FRAMEWORK_ERRNO_AGAIN:
                    //sleep
                    continue;
                default:
                    DSG("v2k_cam: failed to _read_frame");
                    return ret;
            }
        }
    
#if FRAME_COUNT_TO_END == FRAME_INFINITE
    } while (1);
#else
    } while (count++ < FRAME_COUNT_TO_END);
#endif
    
    return 0;
}
#endif

static int _cam_stop_capturing(unsigned int cam_idx)
{
    int ret;

    ret = kdp_dev_ioctl(inst_cam[cam_idx].handle, V2K_VIOC_STREAMOFF, 0);
    if (ret) {
        DSG("v2k_cam: Stop streaming I/O, OK");   
    } else {
        DSG("v2k_cam: Stop streaming I/O, ERROR(%d)", ret);   
    }

    return ret;
}

void how_to_use_api_mipi0()
{
    int ret;
    static int first = 0;
    const unsigned int cam_idx = 0;    
    unsigned int frame_addr = 0;
    unsigned int frame_size = 0;
    unsigned int width = 0;
    unsigned int height = 0;
    unsigned int pixelformat = 0;
    struct v2k_capability cap;
    struct v2k_format fmt;
    char fmtstr[8];
    memset(&cap, 0, sizeof(cap));
    memset(&fmt, 0, sizeof(fmt));

#if IMGSRC_0_RES == RES_640_480
    width = VGA_LANDSCAPE_WIDTH;
    height = VGA_LANDSCAPE_HEIGHT;
#elif IMGSRC_0_RES == RES_480_272
    width = TFT43_WIDTH;
    height = TFT43_HEIGHT;
#endif

#if IMGSRC_0_FORMAT == IMAGE_FORMAT_RGB565
    pixelformat = V2K_PIX_FMT_RGB565;
#elif IMGSRC_0_FORMAT == IMAGE_FORMAT_RAW10
    pixelformat = V2K_PIX_FMT_RAW10;
#elif IMGSRC_0_FORMAT == IMAGE_FORMAT_RAW8
    pixelformat = V2K_PIX_FMT_RAW8;   
#endif    
    
    inst_cam[cam_idx].handle = kdp_camera_open(cam_idx);
    if (-1 == inst_cam[cam_idx].handle)
        return;

    if (0 != kdp_camera_get_device_info(cam_idx, &cap))
        return;

    DSG("===== Capability Informations ====="); 
    DSG("   driver: %s", cap.driver); 
    DSG("   desc: %s", cap.desc); 
    DSG("   version: %08X", cap.version);

    if (0 != kdp_camera_set_frame_format(cam_idx, width, height, pixelformat))
        return;
    
    if (0 != kdp_camera_get_frame_format(cam_idx, &fmt))
        return;    

    DSG("===== Stream Format Informations =====");         
    DSG("   width: %d", fmt.width); 
    DSG("   height: %d", fmt.height); 

    memset(fmtstr, 0, 8); 
    memcpy(fmtstr, &fmt.pixelformat, 4); 
    DSG("   pixelformat: %s", fmtstr); 
    DSG("   field: %d", fmt.field); 
    DSG("   bytesperline: %d", fmt.bytesperline); 
    DSG("   sizeimage: %d", fmt.sizeimage); 
    DSG("   colorspace: %d", fmt.colorspace);

    if (0 != kdp_camera_buffer_init(cam_idx))
        return;

    kdp_camera_start(cam_idx, 0);
    {
        kdp_timer_init(PWMTIMER1, PWMTMR_1MSEC_PERIOD);
        delay_ms(50);
        kdp_timer_close(PWMTIMER1);  
    }

    m_state[cam_idx] = CAMERA_STATE_RUNNING;
    
    DSG(" will send evt_mipi0_ready_id");
    set_event(evt_source_data_ready_id, FLAGS_SOURCE_READY_EVT);
    //set_event(evt_mipi0_ready_id, FLAGS_MIPI0_READY_EVT);
    
    while (CAMERA_STATE_RUNNING == m_state[cam_idx]) {
        DSG("[v2k_cam] 0 ask buffer first=%d", first);
        ret = kdp_camera_buffer_capture(cam_idx, &frame_addr, &frame_size);
        DSG("[v2k_cam] 0 got buffer ret=%x", ret);
        if (0 != ret) break;

        DSG("[v2k_cam] 0 addr=%x size=%d waiting", frame_addr, frame_size);
        if (first != 0)
        {
            //set_event(evt_source_data_ready_id, FLAGS_SOURCE_READY_EVT);
            DSG("kdp_getchar 1");
            uart_com_log_getc();
            DSG("kdp_getchar 2");
        }
        else {
            //set_event(evt_source_data_ready_id, FLAGS_SOURCE_READY_EVT);
            //set_event(evt_mipi0_ready_id, FLAGS_MIPI0_READY_EVT);
            ++first;
        }
        
        //DSG("[v2k_cam] 0 put buffer in slot");
        ret = kdp_camera_buffer_prepare(cam_idx);
        //DSG("[v2k_cam] 0 put buffer in slot done ret=%x", ret);
        if (0 != ret) break;
        //DSG("how_to_use_api after kdp_camera_buffer_prepare");
        
        //osDelay(5000);
    }

    kdp_camera_stop(cam_idx);
    kdp_camera_close(cam_idx);
}

void how_to_use_api_mipi1()
{
    int ret;
    static int first = 0;
    const unsigned int cam_idx = 1;    
    unsigned int frame_addr = 0;
    unsigned int frame_size = 0;
    unsigned int width = 0;
    unsigned int height = 0;
    unsigned int pixelformat = 0;
    struct v2k_capability cap;
    struct v2k_format fmt;
    char fmtstr[8];
    memset(&cap, 0, sizeof(cap));
    memset(&fmt, 0, sizeof(fmt));

#if IMGSRC_1_RES == RES_640_480
    width = VGA_LANDSCAPE_WIDTH;
    height = VGA_LANDSCAPE_HEIGHT;
#elif IMGSRC_1_RES == RES_480_272
    width = TFT43_WIDTH;
    height = TFT43_HEIGHT;    
#endif

#if IMGSRC_1_FORMAT == IMAGE_FORMAT_RGB565
    pixelformat = V2K_PIX_FMT_RGB565;
#elif IMGSRC_1_FORMAT == IMAGE_FORMAT_RAW10
    pixelformat = V2K_PIX_FMT_RAW10;
#elif IMGSRC_1_FORMAT == IMAGE_FORMAT_RAW8
    pixelformat = V2K_PIX_FMT_RAW8;   
#endif
    
    inst_cam[cam_idx].handle = kdp_camera_open(cam_idx);
    if (-1 == inst_cam[cam_idx].handle)
        return;

    if (0 != kdp_camera_get_device_info(cam_idx, &cap))
        return;

    DSG("===== Capability Informations ====="); 
    DSG("   driver: %s", cap.driver); 
    DSG("   desc: %s", cap.desc); 
    DSG("   version: %08X", cap.version);

    if (0 != kdp_camera_set_frame_format(cam_idx, width, height, pixelformat))
        return;
    
    if (0 != kdp_camera_get_frame_format(cam_idx, &fmt))
        return;    

    DSG("===== Stream Format Informations =====");         
    DSG("   width: %d", fmt.width); 
    DSG("   height: %d", fmt.height); 

    memset(fmtstr, 0, 8); 
    memcpy(fmtstr, &fmt.pixelformat, 4); 
    DSG("   pixelformat: %s", fmtstr); 
    DSG("   field: %d", fmt.field); 
    DSG("   bytesperline: %d", fmt.bytesperline); 
    DSG("   sizeimage: %d", fmt.sizeimage); 
    DSG("   colorspace: %d", fmt.colorspace);

    if (0 != kdp_camera_buffer_init(cam_idx))
        return;

    kdp_camera_start(cam_idx, 0);    
    {
        kdp_timer_init(PWMTIMER1, PWMTMR_1MSEC_PERIOD);
        delay_ms(50);
        kdp_timer_close(PWMTIMER1);
    }
    
    m_state[cam_idx] = CAMERA_STATE_RUNNING;
    
    DSG(" will send evt_mipi0_ready_id");
    set_event(evt_source_data_ready_id, FLAGS_SOURCE_READY_EVT);
    //set_event(evt_mipi0_ready_id, FLAGS_MIPI0_READY_EVT);
    
    while (CAMERA_STATE_RUNNING == m_state[cam_idx]) {
        DSG("[v2k_cam] 0 ask buffer first=%d", first);
        ret = kdp_camera_buffer_capture(cam_idx, &frame_addr, &frame_size);
        DSG("[v2k_cam] 0 got buffer ret=%x", ret);
        if (0 != ret) break;

        DSG("[v2k_cam] 0 addr=%x size=%d waiting", frame_addr, frame_size);
        if (first != 0)
        {
            //set_event(evt_source_data_ready_id, FLAGS_SOURCE_READY_EVT);
            DSG("kdp_getchar 1");
            uart_com_log_getc();
            DSG("kdp_getchar 2");
        }
        else {
            //set_event(evt_source_data_ready_id, FLAGS_SOURCE_READY_EVT);
            //set_event(evt_mipi0_ready_id, FLAGS_MIPI0_READY_EVT);
            ++first;
        }
        
        DSG("[v2k_cam] 0 put buffer in slot");
        ret = kdp_camera_buffer_prepare(cam_idx);
        DSG("[v2k_cam] 0 put buffer in slot done ret=%x", ret);
        if (0 != ret) break;
        //DSG("how_to_use_api after kdp_camera_buffer_prepare");
        
        //osDelay(5000);
    }

    kdp_camera_stop(cam_idx);
    kdp_camera_close(cam_idx);
}

static void v2k_cam_thread_mipi0(void *argument)
{
    how_to_use_api_mipi0();
}

#if 0
static void v2k_cam_thread_mipi1(void *argument)
{
    how_to_use_api_mipi1();
}
#endif

int v2k_camera_init(bool all)
{    
    sys_camera_global_init();

    if (all) {
        evt_source_data_ready_id = create_event();    
        evt_mipi0_ready_id = create_event();

        //inst_cam[0].dev_idx = 0;
        tid_v2k_cam_0 = osThreadNew(v2k_cam_thread_mipi0, (void*)&inst_cam[0], NULL);    
    #if (V2K_ENABLE_TYPE == V2K_ENABLE_HMX2056_OV9286)
        //inst_cam[1].dev_idx = 1;
        tid_v2k_cam_1 = osThreadNew(v2k_cam_thread_mipi1, (void*)&inst_cam[1], NULL);
    #endif
    }

    return 0;
}

int kdp_camera_open(unsigned int cam_idx)
{
    int handle = -1;
	char dev_name[16];

	sprintf(dev_name, "kdp_dev/cam%d", cam_idx);
    handle = kdp_dev_open(dev_name);
    if (-1 == handle) {
        DSG("v2k_cam : Failed to open device (%s)", dev_name);
        return -1;
    }

    inst_cam[cam_idx].handle = handle;

    return 0;
}

int kdp_camera_close(unsigned int cam_idx)
{
    if (inst_cam[cam_idx].app_buf_infos)
        free(inst_cam[cam_idx].app_buf_infos);

    kdp_dev_close(inst_cam[cam_idx].handle);
    
    return 0;
}

int kdp_camera_get_device_info(unsigned int cam_idx, struct v2k_capability *lpCap)
{
    return _cam_get_device_info(cam_idx, lpCap);
}

int kdp_camera_set_frame_format(unsigned int cam_idx, 
        unsigned int width,
        unsigned int height,
        unsigned int pixelformat)
{
    return _cam_set_frame_format(cam_idx, width, height, pixelformat);
}

int kdp_camera_get_frame_format(unsigned int cam_idx, struct v2k_format *lpfmt)
{
    return _cam_get_frame_format(cam_idx, lpfmt);
}

int kdp_camera_buffer_init(unsigned int cam_idx)
{
    return _cam_init_mem_mapping(cam_idx);
}

int kdp_camera_start(unsigned int cam_idx, unsigned int flags)
{
    int ret = _cam_start_capturing(cam_idx);
    
    return ret;
}

int kdp_camera_stop(unsigned int cam_idx)
{
    return _cam_stop_capturing(cam_idx); 
}

int kdp_camera_buffer_prepare(unsigned int cam_idx)
{
    int ret = 0;
    ret = kdp_dev_ioctl(
		inst_cam[cam_idx].handle, V2K_VIOC_QBUF, (void*)&(inst_cam[cam_idx].api_buf));

    return ret;
}

int kdp_camera_buffer_capture(
		unsigned int cam_idx, unsigned int *addr, unsigned int *size)
{
    int ret = 0;
    //DSG("[%s] cam_idx=%d inst_cam[cam_idx].handle=%d", __func__, cam_idx, inst_cam[cam_idx].handle);
    if ((addr) && (size)) {
        ret = kdp_dev_ioctl(
			inst_cam[cam_idx].handle, V2K_VIOC_DQBUF, (void*)&(inst_cam[cam_idx].api_buf));
        if (V2K_SUCCEEDED(ret)) {
            {
                *addr = (unsigned int)inst_cam[cam_idx].app_buf_infos[0].start;
                *size = inst_cam[cam_idx].app_buf_infos[0].length;
            }
        }
    }

    return ret;
}

int kdp_camera_buffer_read(
		unsigned int cam_idx, unsigned int *addr, unsigned int *size)
{
    int ret = -1;
    if (0 == kdp_camera_buffer_capture(cam_idx, addr, size) &&
        0 == kdp_camera_buffer_prepare(cam_idx))    
        ret = 0;

    return ret;
//    unsigned int frame_addr;
//    unsigned int frame_size;
//    if (0 == kdp_camera_buffer_capture(&frame_addr, &frame_size) &&
//        0 == kdp_camera_buffer_prepare())
//    {
//        DSG("frame_addr=%x frame_size=%u", frame_addr,frame_size);
//        DSG("press any key for snapshot");
//        kdp_getchar(DEBUG_CONSOLE);
//        osDelay(1000);
//    }
}
#else

int v2k_camera_init(int all) { return 0; }

#endif
#endif

